/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;
using System.Resources;
using System.Reflection;

namespace NetSim.Core
{
	/// <summary>
	/// Provides access to the assembly resources (strings etc..)
	/// </summary>
	public class Resources
	{
		/// <summary>
		/// Resource manager of strings.
		/// </summary>
		static ResourceManager strings; 

		/// <summary>
		/// Prevents this class to be instantiated.
		/// </summary>
		private Resources()
		{
		}

		/// <summary>
		/// Initializes the class.
		/// </summary>
		static Resources()
		{
			strings = new ResourceManager("NetSim.Core.Strings", Assembly.GetExecutingAssembly());
		}

		/// <summary>
		/// Retrieves a string resource.
		/// </summary>
		/// <param name="id">The string identifier.</param>
		/// <returns>The string.</returns>
		public static string GetString(string id)
		{
			return strings.GetString(id);
		}
    
		/// <summary>
		/// Retrieves formatted string resource.
		/// </summary>
		/// <param name="id">The string identifier.</param>
		/// <param name="args">An <see cref="System.Object"/> array containing zero or more objects to format.</param>
		/// <returns>The formatted string resource.</returns>
		/// <exception cref="ArgumentNullException">The string resource with <paramref name="id"/> doesn't exist.</exception>
		/// <exception cref="FormatException">The string resource is not valid formatting string for specified arguments.</exception>
		public static string GetString(string id, params object[] args)
		{
			return String.Format(strings.GetString(id), args);
		}

	}
}
