/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;

namespace NetSim.Core
{
	/// <summary>
	/// Base class for exceptions thrown by the simulator
	/// </summary>
	public class SimulatorException : ApplicationException
	{
		public SimulatorException(string message) : base(message)
		{
		}

		public SimulatorException(string message, Exception innerException) : base(message, innerException)
		{
		}
	}

	/// <summary>
	/// Exception thrown while initializing the simulator.
	/// </summary>
	public class InitializationException : SimulatorException
	{
		public InitializationException(string message) : base(message)
		{
		}

		public InitializationException(string message, Exception innerException) : base(message, innerException)
		{
		}
	}

	/// <summary>
	/// Exception thrown if error occurs while creating the packet.
	/// </summary>
	public class PacketCreationException : SimulatorException
	{
		public PacketCreationException(string message) : base(message)
		{
		}

		public PacketCreationException(string message, Exception innerException) : base(message, innerException)
		{
		}
	}
}
