/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;

namespace NetSim.Core
{
	/// <summary>
	/// Implemented by the simulator objects that implement enabling/disabling or that can inform
	/// if these features are available.
	/// </summary>
	public interface ISuspendable
	{
		/// <summary>
		/// Returns <b>true</b> if the object can be suspended and enabled again (ie. <see cref="Enabled"/>
		/// property can be set).
		/// </summary>
		bool CanDisable { get; }

		/// <summary>
		/// Returns <b>true</b> if the object can be reconfigured; ie. its state can be changed to the initial
		/// state or if it has a configuration file, the configuration can be reloaded (by calling
		/// <see cref="Reconfigure"/>).
		/// </summary>
		bool CanReconfigure { get; }

		/// <summary>
		/// Gets or sets whether the object is enabled or disabled. Get property can be accessed always,
		/// before setting you should verify if it is supported via <see cref="CanDisable"/>. Othewise
		/// <see cref="InvalidOperationException"/> is thrown.
		/// </summary>
		bool Enabled { get; set; }

		/// <summary>
		/// Moves the object to the initial state. If it has a configuration file, the configuration is
		/// refreshed.
		/// </summary>
		void Reconfigure();
	}

}
