/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;
using System.Xml;
using System.Collections;
using BasicObjectKind = NetSim.Core.BasicObjectKind;
using NodeItemKind = NetSim.Core.NodeItemKind;

namespace NetSim.Gui
{
	/// <summary>
	/// Summary description for GuiProjectData.
	/// </summary>
	public class GuiProjectData : XmlData
	{
		private XmlDocument document;
		
		private const string DefaultIcon = "default.gif";
		private readonly System.Drawing.Point DefaultLocation = new System.Drawing.Point(30,30);

		// names of XML nodes in GUI config
		private static string[] XmlNodeNames = new string[(int)BasicObjectKind._max];
		private static string[] XmlNodeParentNames = new string[(int)BasicObjectKind._max];

		static GuiProjectData()
		{
			XmlNodeNames[(int)BasicObjectKind.Node] = "Node";
			XmlNodeNames[(int)BasicObjectKind.Link] = "Link";
			XmlNodeParentNames[(int)BasicObjectKind.Node] = "Nodes";
			XmlNodeParentNames[(int)BasicObjectKind.Link] = "Links";
		}

		public GuiProjectData()
		{
		}

		public override void LoadDocument(System.Xml.XmlDocument document)
		{
			this.document = document;
		}

		public override void SaveDocument(out System.Xml.XmlDocument document)
		{
			document = this.document;
		}

		public XmlNode AddBasicItem(BasicObjectKind kind, string name, string icon, System.Drawing.Point point, System.Drawing.Color color)
		{
			if (icon == null || icon == String.Empty)
				icon = DefaultIcon;
			if (point == System.Drawing.Point.Empty)
				point = DefaultLocation;
			
			// write color only if it is valid
			string color_str = String.Empty;
			if (color != System.Drawing.Color.Empty)
				color_str = String.Format("color=\"{0}\" ", color.ToArgb().ToString("x"));

			XmlNode parent_el = GetXmlNode(this.document.DocumentElement, XmlNodeParentNames[(int)kind], true);
			XmlDocumentFragment frag = parent_el.OwnerDocument.CreateDocumentFragment();
			frag.InnerXml = String.Format("<{5} name=\"{0}\" icon=\"{1}\" x=\"{2}\" y=\"{3}\" {4}/>",
				name, icon, point.X, point.Y, color_str, XmlNodeNames[(int)kind]);
			
			return parent_el.AppendChild(frag.FirstChild);
		}

		public XmlNode GetBasicItem(BasicObjectKind kind, string name)
		{
			XmlNode parent_el = GetXmlNode(this.document.DocumentElement, XmlNodeParentNames[(int)kind], true);
			foreach (XmlNode n in parent_el.ChildNodes)
			{
				if (n.Name != XmlNodeNames[(int)kind]) continue;

				XmlAttribute attr_name = n.Attributes["name"];
				if (attr_name != null && attr_name.InnerText == name)
					return n;
			}
			return null;
		}

	}
}
