/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Text;

namespace NetSim.Gui
{
	/// <summary>
	/// Summary description for ErrorForm.
	/// </summary>
	public class ErrorForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.TextBox errorTextBox;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Button closeButton;
		private System.Windows.Forms.TextBox detailsTextBox;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Button continueButton;

		public bool ShouldContinue { get { return shouldContinue; } }
		private bool shouldContinue = false;

		public ErrorForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.label1 = new System.Windows.Forms.Label();
			this.errorTextBox = new System.Windows.Forms.TextBox();
			this.label2 = new System.Windows.Forms.Label();
			this.detailsTextBox = new System.Windows.Forms.TextBox();
			this.closeButton = new System.Windows.Forms.Button();
			this.continueButton = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(208, 16);
			this.label1.TabIndex = 0;
			this.label1.Text = "The application encountered an error:";
			// 
			// errorTextBox
			// 
			this.errorTextBox.BackColor = System.Drawing.SystemColors.Control;
			this.errorTextBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
			this.errorTextBox.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.errorTextBox.Location = new System.Drawing.Point(8, 24);
			this.errorTextBox.Name = "errorTextBox";
			this.errorTextBox.Size = new System.Drawing.Size(400, 13);
			this.errorTextBox.TabIndex = 1;
			this.errorTextBox.Text = "";
			// 
			// label2
			// 
			this.label2.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.label2.Location = new System.Drawing.Point(8, 40);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(448, 40);
			this.label2.TabIndex = 2;
			this.label2.Text = "Below is more detailed description. Please report the error to the author with de" +
				"tailed description of what you were doing. Attach your network and simulator con" +
				"figuration files if possible.";
			// 
			// detailsTextBox
			// 
			this.detailsTextBox.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.detailsTextBox.BackColor = System.Drawing.SystemColors.Control;
			this.detailsTextBox.Location = new System.Drawing.Point(8, 88);
			this.detailsTextBox.Multiline = true;
			this.detailsTextBox.Name = "detailsTextBox";
			this.detailsTextBox.ReadOnly = true;
			this.detailsTextBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.detailsTextBox.Size = new System.Drawing.Size(448, 208);
			this.detailsTextBox.TabIndex = 3;
			this.detailsTextBox.Text = "";
			// 
			// closeButton
			// 
			this.closeButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.closeButton.Location = new System.Drawing.Point(296, 304);
			this.closeButton.Name = "closeButton";
			this.closeButton.Size = new System.Drawing.Size(144, 23);
			this.closeButton.TabIndex = 5;
			this.closeButton.Text = "Close";
			this.closeButton.Click += new System.EventHandler(this.closeButton_Click);
			// 
			// continueButton
			// 
			this.continueButton.Location = new System.Drawing.Point(128, 304);
			this.continueButton.Name = "continueButton";
			this.continueButton.Size = new System.Drawing.Size(160, 23);
			this.continueButton.TabIndex = 4;
			this.continueButton.Text = "Continue in work";
			this.continueButton.Visible = false;
			this.continueButton.Click += new System.EventHandler(this.continueButton_Click);
			// 
			// ErrorForm
			// 
			this.AcceptButton = this.continueButton;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(464, 333);
			this.Controls.Add(this.continueButton);
			this.Controls.Add(this.closeButton);
			this.Controls.Add(this.detailsTextBox);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.errorTextBox);
			this.Controls.Add(this.label1);
			this.MinimizeBox = false;
			this.Name = "ErrorForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Network Simulator Error";
			this.ResumeLayout(false);

		}
		#endregion

		private void closeButton_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		public void SetError(Exception e, bool canRecover)
		{
			if (canRecover)
			{
				continueButton.Visible = true;
				closeButton.Text = "Exit application";
			}

			this.errorTextBox.Text = e.Message;
			StringBuilder details = new StringBuilder();
				
			while (e != null) // go through all inner exceptions
			{
				details.Append(e.Message);
				details.Append(Environment.NewLine);
				details.Append("Stack trace:");
				details.Append(Environment.NewLine);
				details.Append(e.StackTrace);
				details.Append(Environment.NewLine);
				details.Append("-------------------------------------------------------");
				details.Append(Environment.NewLine);

				e = e.InnerException;
			}
				
			this.detailsTextBox.Text = details.ToString();
		}

		private void continueButton_Click(object sender, System.EventArgs e)
		{
			this.shouldContinue = true;

			// notify the user that restart is recommended
			MessageBox.Show(this, "The application could be in an inconsistent state. It is strongly recommended to save your work immediately and restart the application.",
				"Recommendation", MessageBoxButtons.OK, MessageBoxIcon.Information);

			this.Close();
		}
	}
}
