/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using System.Text;
using System.Diagnostics;
using BasicObjectKind = NetSim.Core.BasicObjectKind;
using NodeItemKind = NetSim.Core.NodeItemKind;
using Microsoft.Win32;

namespace NetSim.Gui
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.TabPage nodesTabPage;
		private System.Windows.Forms.TabPage linksTabPage;
		private System.Windows.Forms.TabControl leftTabControl;
		private System.Windows.Forms.ListView nodesListView;
		private System.Windows.Forms.ListView linksListView;
		internal System.Windows.Forms.ImageList iconsImageList;
		private System.Windows.Forms.StatusBar statusBar1;
		private System.Windows.Forms.TabControl bottomTabControl;
		private System.Windows.Forms.TabPage outputTabPage;
		private System.Windows.Forms.Splitter bottomSplitter;
		private System.Windows.Forms.TabPage errorsTabPage;
		private System.Windows.Forms.TextBox outputTextBox;
		private System.Windows.Forms.ToolBar mainToolBar;
		private System.Windows.Forms.ToolBarButton startToolBarButton;
		private System.Windows.Forms.ToolBarButton stopToolBarButton;
		private System.Windows.Forms.ImageList toolBarIconsImageList;
		private System.Windows.Forms.ToolBarButton separator1;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.MenuItem fileOpenMenuItem;
		private System.Windows.Forms.MenuItem fileSaveMenuItem;
		private System.Windows.Forms.MenuItem filePrintMenuItem;
		private System.Windows.Forms.MenuItem fileExitMenuItem;
		private System.Windows.Forms.MenuItem fileNewMenuItem;
		private System.Windows.Forms.OpenFileDialog openProjectFileDialog;
		private System.Windows.Forms.SaveFileDialog newProjectFileDialog;
		private System.Windows.Forms.MenuItem fileSeparatorMenuItem1;
		private System.Windows.Forms.MenuItem fileSeparatorMenuItem2;
		private System.Windows.Forms.MenuItem fileCloseMenuItem;
		private System.Windows.Forms.MenuItem propertiesMenuItem;
		private System.Windows.Forms.MenuItem renameMenuItem;
		private System.Windows.Forms.MenuItem fileMenuItem;
		private System.Windows.Forms.MenuItem helpMenuItem;
		private System.Windows.Forms.MenuItem helpAboutMenuItem;
		private System.Windows.Forms.MenuItem simulationMenuItem;
		private System.Windows.Forms.MenuItem simulationStartMenuItem;
		private System.Windows.Forms.MenuItem simulationStopMenuItem;
		private System.Windows.Forms.MenuItem simulationPauseMenuItem;
		private System.Windows.Forms.MenuItem deleteMenuItem;
		private System.Windows.Forms.MenuItem editConfigFileMenuItem;
		private System.Windows.Forms.MenuItem captionMenuItem;
		private System.Windows.Forms.Panel leftPanel;
		private System.Windows.Forms.Splitter leftSplitter;
		private System.Windows.Forms.Label leftDescriptionLabel;
		private System.Windows.Forms.Splitter leftDescriptionSplitter;
		internal System.Windows.Forms.Panel mainPanel;
		private System.Windows.Forms.ListView errorsListView;
		private System.Windows.Forms.ColumnHeader iconColumnHeader;
		private System.Windows.Forms.ColumnHeader descColumnHeader;
		private System.Windows.Forms.MenuItem toolsMenuItem;
		private System.Windows.Forms.MenuItem toolsAdaptersMenuItem;
		private System.Windows.Forms.MenuItem toolsModulesMenuItem;
		private System.Windows.Forms.MenuItem toolsApplicationsMenuItem;
		private System.Windows.Forms.ComboBox zoomComboBox;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.PrintDialog printDialog;
		internal System.Windows.Forms.ContextMenu nodeContextMenu;
		internal NodeItemsListForm nodeItemsListForm;

		//
		// Above properties added by the designer, below ours
		//

		private const string DefaultCaption = "Network Simulator";
		
		// default main configuration file
		private static string ConfigFile = "GuiConfig.xml";
		// project file specified as an command line argument
		private static string FirstProjectToOpen = null;

		internal const string NewProjectFile = "Templates/Projects/NewProject/NewProject.xml";
		private const string ProjectExtension = "xml";

		internal GeneralData generalData;
		internal ProjectData projectData = null;

		private float DefaultConnectionWidth = 3f;
		private Pen connectionDrawPen = new Pen(Color.FromKnownColor(KnownColor.DarkBlue), 3f);
		private Pen connectionDeletePen = new Pen(Color.FromKnownColor(KnownColor.Window), 3f);

		/// <summary>
		/// Paths of icons added to the <see cref="iconsImageList"/>. Used to recognise if there is an icon already
		/// and it is not necessary to add it twice.
		/// </summary>
		// generics: <string>
		private ArrayList iconsImageListPaths = new ArrayList();
		private System.Windows.Forms.MenuItem appearanceMenuItem;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem toolsCenterDiagramMenuItem;
		private System.Windows.Forms.TabPage commandTabPage;
		internal System.Windows.Forms.TextBox commandTextBox;
		private System.Windows.Forms.Label commandDescriptionLabel;
		private System.Windows.Forms.ToolBarButton pauseToolBarButton;
		private System.Windows.Forms.CheckBox autoHideCheckBox;
		private System.Windows.Forms.ToolBarButton saveToolBarButton;
		private System.Windows.Forms.MenuItem enableDisableMenuItem;
		private System.Windows.Forms.MenuItem helpCheckForUpdatesMenuItem;
		private System.Windows.Forms.OpenFileDialog openMainConfigFileDialog;
		private System.Windows.Forms.MenuItem helpLicenseMenuItem;

		// generics <System.Drawing.Image>
		private ArrayList iconsImageListImages = new ArrayList();

		/// <summary>
		/// Returns an <see cref="System.Drawing.Image"/> object for the file specified in <paramref name="path"/>.
		/// If the file doesn't exist, default picture is returned. The image is added into the
		/// <see cref="iconsImageListImages"/> collection if not present there.
		/// </summary>
		/// <param name="path">Path to picture file.</param>
		/// <param name="index">Index in <see cref="iconsImageListImages"/> collection.</param>
		/// <returns>Image object.</returns>
		internal System.Drawing.Image GetImage(string path, out int index)
		{
			if (!System.IO.File.Exists(path)) // return default picture
			{
				index = 0;
				return iconsImageList.Images[index];
			}

			// find existing icon if present
			for (int i = 1; i < iconsImageListPaths.Count-1; i++)
			{
				if (iconsImageListPaths[i].Equals(path))
				{
					index = i;
					return (System.Drawing.Image)iconsImageListImages[i];
				}
			}

			// not found, add to the list
			System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(path);
			index = iconsImageList.Images.Add(bitmap, System.Drawing.Color.Empty);
			iconsImageListPaths.Add(path);
			iconsImageListImages.Add(bitmap);

			return bitmap;
		}

		private static MainForm instance;
		public static MainForm Instance
		{
			get
			{
				Debug.Assert(instance != null);
				return instance;
			}
		}

		private MainForm()
		{
			// Required for Windows Form Designer support
			InitializeComponent();
		}

		/// <summary>
		/// Initializes data used by GUI.
		/// </summary>
		/// <param name="splash">Reference to the splash screen that is shown during the initialization
		/// process. It should be hidden or sent to the back when some user interaction is needed.</param>
		/// <returns><b>True</b> on success, <b>false</b> if the application execution shouldn't continue.</returns>
		private bool Initialize(SplashScreen splash)
		{
			// init console for output messages
			NetSim.Gui.Console.Out = new GuiConsoleWriter(this.outputTextBox);
			NetSim.Gui.Console.WriteLine("Initializing Application...");

			// add one item to icon paths for default item added in the designer
			iconsImageListPaths.Add(String.Empty);
			iconsImageListImages.Add(null);

			// register handlers
			mainPanel.Paint += new PaintEventHandler(mainPanel_Paint);

			// create class holding all non-project specific data for the application
			generalData = new GeneralData();

			// initialize data according to configuration file
			string config_file = ConfigFile;
			if (!System.IO.File.Exists(config_file))
			{
				splash.Visible = false;
				string msg = String.Format("Cannot find main configuration file '{0}'. Would you like to locate it yourself? If not, application will close.", ConfigFile);
				DialogResult dr = MessageBox.Show(this, msg, "NetSim: Error", MessageBoxButtons.YesNo, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
				if (dr == DialogResult.Yes)
				{
					DialogResult dr2 = openMainConfigFileDialog.ShowDialog();
					if (dr2 == DialogResult.OK && System.IO.File.Exists(openMainConfigFileDialog.FileName))
						config_file = openMainConfigFileDialog.FileName;
					else
						dr = DialogResult.No;
				}

				splash.Visible = true;
				if (dr == DialogResult.No)
					return false;
			}

			generalData.Initialize(config_file);
			NetSim.Gui.Console.WriteLine("Initialized.");

			//
			// Init lists of predefined objects
			//
			InitPredefinedNodesList(nodesListView.Items, generalData.PredefinedNodes);
			InitPredefinedLinksList(linksListView.Items, generalData.PredefinedLinks);

			this.nodeItemsListForm = new NodeItemsListForm();
			this.nodeItemsListForm.Initialize(NodeItemKind.Adapter, generalData.PredefinedAdapters);
			this.nodeItemsListForm.Initialize(NodeItemKind.Module, generalData.PredefinedModules);
			this.nodeItemsListForm.Initialize(NodeItemKind.Application, generalData.PredefinedApplications);

			// success
			return true;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.fileMenuItem = new System.Windows.Forms.MenuItem();
			this.fileNewMenuItem = new System.Windows.Forms.MenuItem();
			this.fileOpenMenuItem = new System.Windows.Forms.MenuItem();
			this.fileSaveMenuItem = new System.Windows.Forms.MenuItem();
			this.fileCloseMenuItem = new System.Windows.Forms.MenuItem();
			this.fileSeparatorMenuItem1 = new System.Windows.Forms.MenuItem();
			this.filePrintMenuItem = new System.Windows.Forms.MenuItem();
			this.fileSeparatorMenuItem2 = new System.Windows.Forms.MenuItem();
			this.fileExitMenuItem = new System.Windows.Forms.MenuItem();
			this.simulationMenuItem = new System.Windows.Forms.MenuItem();
			this.simulationStartMenuItem = new System.Windows.Forms.MenuItem();
			this.simulationStopMenuItem = new System.Windows.Forms.MenuItem();
			this.simulationPauseMenuItem = new System.Windows.Forms.MenuItem();
			this.toolsMenuItem = new System.Windows.Forms.MenuItem();
			this.toolsAdaptersMenuItem = new System.Windows.Forms.MenuItem();
			this.toolsModulesMenuItem = new System.Windows.Forms.MenuItem();
			this.toolsApplicationsMenuItem = new System.Windows.Forms.MenuItem();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.toolsCenterDiagramMenuItem = new System.Windows.Forms.MenuItem();
			this.helpMenuItem = new System.Windows.Forms.MenuItem();
			this.helpCheckForUpdatesMenuItem = new System.Windows.Forms.MenuItem();
			this.helpAboutMenuItem = new System.Windows.Forms.MenuItem();
			this.helpLicenseMenuItem = new System.Windows.Forms.MenuItem();
			this.leftTabControl = new System.Windows.Forms.TabControl();
			this.nodesTabPage = new System.Windows.Forms.TabPage();
			this.nodesListView = new System.Windows.Forms.ListView();
			this.iconsImageList = new System.Windows.Forms.ImageList(this.components);
			this.linksTabPage = new System.Windows.Forms.TabPage();
			this.linksListView = new System.Windows.Forms.ListView();
			this.bottomTabControl = new System.Windows.Forms.TabControl();
			this.outputTabPage = new System.Windows.Forms.TabPage();
			this.outputTextBox = new System.Windows.Forms.TextBox();
			this.commandTabPage = new System.Windows.Forms.TabPage();
			this.commandTextBox = new System.Windows.Forms.TextBox();
			this.commandDescriptionLabel = new System.Windows.Forms.Label();
			this.errorsTabPage = new System.Windows.Forms.TabPage();
			this.errorsListView = new System.Windows.Forms.ListView();
			this.iconColumnHeader = new System.Windows.Forms.ColumnHeader();
			this.descColumnHeader = new System.Windows.Forms.ColumnHeader();
			this.bottomSplitter = new System.Windows.Forms.Splitter();
			this.statusBar1 = new System.Windows.Forms.StatusBar();
			this.mainToolBar = new System.Windows.Forms.ToolBar();
			this.saveToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.separator1 = new System.Windows.Forms.ToolBarButton();
			this.startToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.stopToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.pauseToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.toolBarIconsImageList = new System.Windows.Forms.ImageList(this.components);
			this.openProjectFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.newProjectFileDialog = new System.Windows.Forms.SaveFileDialog();
			this.zoomComboBox = new System.Windows.Forms.ComboBox();
			this.label1 = new System.Windows.Forms.Label();
			this.printDialog = new System.Windows.Forms.PrintDialog();
			this.nodeContextMenu = new System.Windows.Forms.ContextMenu();
			this.captionMenuItem = new System.Windows.Forms.MenuItem();
			this.enableDisableMenuItem = new System.Windows.Forms.MenuItem();
			this.propertiesMenuItem = new System.Windows.Forms.MenuItem();
			this.renameMenuItem = new System.Windows.Forms.MenuItem();
			this.deleteMenuItem = new System.Windows.Forms.MenuItem();
			this.editConfigFileMenuItem = new System.Windows.Forms.MenuItem();
			this.appearanceMenuItem = new System.Windows.Forms.MenuItem();
			this.leftPanel = new System.Windows.Forms.Panel();
			this.leftDescriptionSplitter = new System.Windows.Forms.Splitter();
			this.leftDescriptionLabel = new System.Windows.Forms.Label();
			this.leftSplitter = new System.Windows.Forms.Splitter();
			this.mainPanel = new System.Windows.Forms.Panel();
			this.autoHideCheckBox = new System.Windows.Forms.CheckBox();
			this.openMainConfigFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.leftTabControl.SuspendLayout();
			this.nodesTabPage.SuspendLayout();
			this.linksTabPage.SuspendLayout();
			this.bottomTabControl.SuspendLayout();
			this.outputTabPage.SuspendLayout();
			this.commandTabPage.SuspendLayout();
			this.errorsTabPage.SuspendLayout();
			this.leftPanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																																						 this.fileMenuItem,
																																						 this.simulationMenuItem,
																																						 this.toolsMenuItem,
																																						 this.helpMenuItem});
			// 
			// fileMenuItem
			// 
			this.fileMenuItem.Index = 0;
			this.fileMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																																								 this.fileNewMenuItem,
																																								 this.fileOpenMenuItem,
																																								 this.fileSaveMenuItem,
																																								 this.fileCloseMenuItem,
																																								 this.fileSeparatorMenuItem1,
																																								 this.filePrintMenuItem,
																																								 this.fileSeparatorMenuItem2,
																																								 this.fileExitMenuItem});
			this.fileMenuItem.Text = "&File";
			// 
			// fileNewMenuItem
			// 
			this.fileNewMenuItem.Index = 0;
			this.fileNewMenuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlN;
			this.fileNewMenuItem.Text = "&New Project ...";
			this.fileNewMenuItem.Click += new System.EventHandler(this.fileNewMenuItem_Click);
			// 
			// fileOpenMenuItem
			// 
			this.fileOpenMenuItem.Index = 1;
			this.fileOpenMenuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlO;
			this.fileOpenMenuItem.Text = "&Open ...";
			this.fileOpenMenuItem.Click += new System.EventHandler(this.fileOpenMenuItem_Click);
			// 
			// fileSaveMenuItem
			// 
			this.fileSaveMenuItem.Index = 2;
			this.fileSaveMenuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
			this.fileSaveMenuItem.Text = "&Save";
			this.fileSaveMenuItem.Click += new System.EventHandler(this.fileSaveMenuItem_Click);
			// 
			// fileCloseMenuItem
			// 
			this.fileCloseMenuItem.Index = 3;
			this.fileCloseMenuItem.Text = "Close";
			this.fileCloseMenuItem.Click += new System.EventHandler(this.fileCloseMenuItem_Click);
			// 
			// fileSeparatorMenuItem1
			// 
			this.fileSeparatorMenuItem1.Index = 4;
			this.fileSeparatorMenuItem1.Text = "-";
			this.fileSeparatorMenuItem1.Visible = false;
			// 
			// filePrintMenuItem
			// 
			this.filePrintMenuItem.Index = 5;
			this.filePrintMenuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlP;
			this.filePrintMenuItem.Text = "&Print ...";
			this.filePrintMenuItem.Visible = false;
			this.filePrintMenuItem.Click += new System.EventHandler(this.filePrintMenuItem_Click);
			// 
			// fileSeparatorMenuItem2
			// 
			this.fileSeparatorMenuItem2.Index = 6;
			this.fileSeparatorMenuItem2.Text = "-";
			// 
			// fileExitMenuItem
			// 
			this.fileExitMenuItem.Index = 7;
			this.fileExitMenuItem.Shortcut = System.Windows.Forms.Shortcut.AltF4;
			this.fileExitMenuItem.Text = "E&xit";
			this.fileExitMenuItem.Click += new System.EventHandler(this.fileExitMenuItem_Click);
			// 
			// simulationMenuItem
			// 
			this.simulationMenuItem.Enabled = false;
			this.simulationMenuItem.Index = 1;
			this.simulationMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																																											 this.simulationStartMenuItem,
																																											 this.simulationStopMenuItem,
																																											 this.simulationPauseMenuItem});
			this.simulationMenuItem.Text = "&Simulation";
			// 
			// simulationStartMenuItem
			// 
			this.simulationStartMenuItem.Index = 0;
			this.simulationStartMenuItem.Text = "Start";
			this.simulationStartMenuItem.Click += new System.EventHandler(this.simulationStartMenuItem_Click);
			// 
			// simulationStopMenuItem
			// 
			this.simulationStopMenuItem.Enabled = false;
			this.simulationStopMenuItem.Index = 1;
			this.simulationStopMenuItem.Text = "Stop";
			this.simulationStopMenuItem.Click += new System.EventHandler(this.simulationStopMenuItem_Click);
			// 
			// simulationPauseMenuItem
			// 
			this.simulationPauseMenuItem.Enabled = false;
			this.simulationPauseMenuItem.Index = 2;
			this.simulationPauseMenuItem.Text = "Pause";
			this.simulationPauseMenuItem.Click += new System.EventHandler(this.simulationPauseMenuItem_Click);
			// 
			// toolsMenuItem
			// 
			this.toolsMenuItem.Index = 2;
			this.toolsMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																																									this.toolsAdaptersMenuItem,
																																									this.toolsModulesMenuItem,
																																									this.toolsApplicationsMenuItem,
																																									this.menuItem1,
																																									this.toolsCenterDiagramMenuItem});
			this.toolsMenuItem.Text = "&Tools";
			// 
			// toolsAdaptersMenuItem
			// 
			this.toolsAdaptersMenuItem.Index = 0;
			this.toolsAdaptersMenuItem.Text = "&Adapters ...";
			this.toolsAdaptersMenuItem.Click += new System.EventHandler(this.toolsAdaptersMenuItem_Click);
			// 
			// toolsModulesMenuItem
			// 
			this.toolsModulesMenuItem.Index = 1;
			this.toolsModulesMenuItem.Text = "&Modules ...";
			this.toolsModulesMenuItem.Click += new System.EventHandler(this.toolsModulesMenuItem_Click);
			// 
			// toolsApplicationsMenuItem
			// 
			this.toolsApplicationsMenuItem.Index = 2;
			this.toolsApplicationsMenuItem.Text = "A&pplications ...";
			this.toolsApplicationsMenuItem.Click += new System.EventHandler(this.toolsApplicationsMenuItem_Click);
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 3;
			this.menuItem1.Text = "-";
			// 
			// toolsCenterDiagramMenuItem
			// 
			this.toolsCenterDiagramMenuItem.Index = 4;
			this.toolsCenterDiagramMenuItem.Text = "Center diagram";
			this.toolsCenterDiagramMenuItem.Click += new System.EventHandler(this.toolsCenterDiagramMenuItem_Click);
			// 
			// helpMenuItem
			// 
			this.helpMenuItem.Index = 3;
			this.helpMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																																								 this.helpCheckForUpdatesMenuItem,
																																								 this.helpAboutMenuItem,
																																								 this.helpLicenseMenuItem});
			this.helpMenuItem.Text = "&Help";
			// 
			// helpCheckForUpdatesMenuItem
			// 
			this.helpCheckForUpdatesMenuItem.Index = 0;
			this.helpCheckForUpdatesMenuItem.Text = "Check for updates";
			this.helpCheckForUpdatesMenuItem.Click += new System.EventHandler(this.helpCheckForUpdatesMenuItem_Click);
			// 
			// helpAboutMenuItem
			// 
			this.helpAboutMenuItem.Index = 1;
			this.helpAboutMenuItem.Text = "About ...";
			this.helpAboutMenuItem.Click += new System.EventHandler(this.helpAboutMenuItem_Click);
			// 
			// helpLicenseMenuItem
			// 
			this.helpLicenseMenuItem.Index = 2;
			this.helpLicenseMenuItem.Text = "License";
			this.helpLicenseMenuItem.Click += new System.EventHandler(this.helpLicenseMenuItem_Click);
			// 
			// leftTabControl
			// 
			this.leftTabControl.Controls.Add(this.nodesTabPage);
			this.leftTabControl.Controls.Add(this.linksTabPage);
			this.leftTabControl.Dock = System.Windows.Forms.DockStyle.Fill;
			this.leftTabControl.Location = new System.Drawing.Point(0, 0);
			this.leftTabControl.Name = "leftTabControl";
			this.leftTabControl.SelectedIndex = 0;
			this.leftTabControl.Size = new System.Drawing.Size(192, 382);
			this.leftTabControl.TabIndex = 1;
			this.leftTabControl.SelectedIndexChanged += new System.EventHandler(this.leftTabControl_SelectedIndexChanged);
			// 
			// nodesTabPage
			// 
			this.nodesTabPage.Controls.Add(this.nodesListView);
			this.nodesTabPage.Location = new System.Drawing.Point(4, 22);
			this.nodesTabPage.Name = "nodesTabPage";
			this.nodesTabPage.Size = new System.Drawing.Size(184, 356);
			this.nodesTabPage.TabIndex = 0;
			this.nodesTabPage.Text = "Nodes";
			// 
			// nodesListView
			// 
			this.nodesListView.Dock = System.Windows.Forms.DockStyle.Fill;
			this.nodesListView.HideSelection = false;
			this.nodesListView.LargeImageList = this.iconsImageList;
			this.nodesListView.Location = new System.Drawing.Point(0, 0);
			this.nodesListView.MultiSelect = false;
			this.nodesListView.Name = "nodesListView";
			this.nodesListView.Size = new System.Drawing.Size(184, 356);
			this.nodesListView.TabIndex = 0;
			this.nodesListView.ItemDrag += new System.Windows.Forms.ItemDragEventHandler(this.nodesListView_ItemDrag);
			this.nodesListView.SelectedIndexChanged += new System.EventHandler(this.leftListView_SelectedIndexChanged);
			// 
			// iconsImageList
			// 
			this.iconsImageList.ImageSize = new System.Drawing.Size(16, 16);
			this.iconsImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("iconsImageList.ImageStream")));
			this.iconsImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// linksTabPage
			// 
			this.linksTabPage.Controls.Add(this.linksListView);
			this.linksTabPage.Location = new System.Drawing.Point(4, 22);
			this.linksTabPage.Name = "linksTabPage";
			this.linksTabPage.Size = new System.Drawing.Size(184, 356);
			this.linksTabPage.TabIndex = 1;
			this.linksTabPage.Text = "Links";
			// 
			// linksListView
			// 
			this.linksListView.Dock = System.Windows.Forms.DockStyle.Fill;
			this.linksListView.HideSelection = false;
			this.linksListView.LargeImageList = this.iconsImageList;
			this.linksListView.Location = new System.Drawing.Point(0, 0);
			this.linksListView.MultiSelect = false;
			this.linksListView.Name = "linksListView";
			this.linksListView.Size = new System.Drawing.Size(184, 356);
			this.linksListView.TabIndex = 0;
			this.linksListView.ItemDrag += new System.Windows.Forms.ItemDragEventHandler(this.linksListView_ItemDrag);
			this.linksListView.SelectedIndexChanged += new System.EventHandler(this.leftListView_SelectedIndexChanged);
			// 
			// bottomTabControl
			// 
			this.bottomTabControl.Alignment = System.Windows.Forms.TabAlignment.Bottom;
			this.bottomTabControl.Controls.Add(this.outputTabPage);
			this.bottomTabControl.Controls.Add(this.commandTabPage);
			this.bottomTabControl.Controls.Add(this.errorsTabPage);
			this.bottomTabControl.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.bottomTabControl.Location = new System.Drawing.Point(0, 485);
			this.bottomTabControl.Name = "bottomTabControl";
			this.bottomTabControl.SelectedIndex = 0;
			this.bottomTabControl.Size = new System.Drawing.Size(896, 104);
			this.bottomTabControl.TabIndex = 2;
			this.bottomTabControl.Click += new System.EventHandler(this.bottomTabControl_Click);
			this.bottomTabControl.MouseHover += new System.EventHandler(this.bottomTabControl_MouseHover);
			this.bottomTabControl.MouseLeave += new System.EventHandler(this.bottomTabControl_MouseLeave);
			// 
			// outputTabPage
			// 
			this.outputTabPage.Controls.Add(this.outputTextBox);
			this.outputTabPage.Location = new System.Drawing.Point(4, 4);
			this.outputTabPage.Name = "outputTabPage";
			this.outputTabPage.Size = new System.Drawing.Size(888, 78);
			this.outputTabPage.TabIndex = 0;
			this.outputTabPage.Text = "Output";
			// 
			// outputTextBox
			// 
			this.outputTextBox.BackColor = System.Drawing.SystemColors.Window;
			this.outputTextBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.outputTextBox.Location = new System.Drawing.Point(0, 0);
			this.outputTextBox.Multiline = true;
			this.outputTextBox.Name = "outputTextBox";
			this.outputTextBox.ReadOnly = true;
			this.outputTextBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.outputTextBox.Size = new System.Drawing.Size(888, 78);
			this.outputTextBox.TabIndex = 0;
			this.outputTextBox.Text = "";
			this.outputTextBox.MouseLeave += new System.EventHandler(this.outputTextBox_MouseLeave);
			// 
			// commandTabPage
			// 
			this.commandTabPage.Controls.Add(this.commandTextBox);
			this.commandTabPage.Controls.Add(this.commandDescriptionLabel);
			this.commandTabPage.Location = new System.Drawing.Point(4, 4);
			this.commandTabPage.Name = "commandTabPage";
			this.commandTabPage.Size = new System.Drawing.Size(888, 78);
			this.commandTabPage.TabIndex = 2;
			this.commandTabPage.Text = "Command Window";
			// 
			// commandTextBox
			// 
			this.commandTextBox.AcceptsReturn = true;
			this.commandTextBox.AcceptsTab = true;
			this.commandTextBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.commandTextBox.Location = new System.Drawing.Point(0, 14);
			this.commandTextBox.Multiline = true;
			this.commandTextBox.Name = "commandTextBox";
			this.commandTextBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.commandTextBox.Size = new System.Drawing.Size(888, 64);
			this.commandTextBox.TabIndex = 0;
			this.commandTextBox.Text = "";
			this.commandTextBox.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.commandTextBox_KeyPress);
			this.commandTextBox.MouseLeave += new System.EventHandler(this.commandTextBox_MouseLeave);
			// 
			// commandDescriptionLabel
			// 
			this.commandDescriptionLabel.BackColor = System.Drawing.SystemColors.Control;
			this.commandDescriptionLabel.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.commandDescriptionLabel.Dock = System.Windows.Forms.DockStyle.Top;
			this.commandDescriptionLabel.Font = new System.Drawing.Font("Microsoft Sans Serif", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.commandDescriptionLabel.ForeColor = System.Drawing.SystemColors.InfoText;
			this.commandDescriptionLabel.Location = new System.Drawing.Point(0, 0);
			this.commandDescriptionLabel.Name = "commandDescriptionLabel";
			this.commandDescriptionLabel.Size = new System.Drawing.Size(888, 14);
			this.commandDescriptionLabel.TabIndex = 1;
			this.commandDescriptionLabel.Text = "Simulator Console - available only if the simulator is running.";
			this.commandDescriptionLabel.MouseLeave += new System.EventHandler(this.commandDescriptionLabel_MouseLeave);
			// 
			// errorsTabPage
			// 
			this.errorsTabPage.Controls.Add(this.errorsListView);
			this.errorsTabPage.Location = new System.Drawing.Point(4, 4);
			this.errorsTabPage.Name = "errorsTabPage";
			this.errorsTabPage.Size = new System.Drawing.Size(888, 78);
			this.errorsTabPage.TabIndex = 1;
			this.errorsTabPage.Text = "Simulator Event Log";
			// 
			// errorsListView
			// 
			this.errorsListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																																										 this.iconColumnHeader,
																																										 this.descColumnHeader});
			this.errorsListView.Dock = System.Windows.Forms.DockStyle.Fill;
			this.errorsListView.FullRowSelect = true;
			this.errorsListView.GridLines = true;
			this.errorsListView.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.errorsListView.Location = new System.Drawing.Point(0, 0);
			this.errorsListView.MultiSelect = false;
			this.errorsListView.Name = "errorsListView";
			this.errorsListView.Size = new System.Drawing.Size(888, 78);
			this.errorsListView.TabIndex = 0;
			this.errorsListView.View = System.Windows.Forms.View.Details;
			this.errorsListView.DoubleClick += new System.EventHandler(this.errorsListView_DoubleClick);
			this.errorsListView.MouseLeave += new System.EventHandler(this.errorsListView_MouseLeave);
			// 
			// iconColumnHeader
			// 
			this.iconColumnHeader.Text = "";
			this.iconColumnHeader.Width = 50;
			// 
			// descColumnHeader
			// 
			this.descColumnHeader.Text = "Description";
			this.descColumnHeader.Width = 500;
			// 
			// bottomSplitter
			// 
			this.bottomSplitter.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.bottomSplitter.Location = new System.Drawing.Point(0, 481);
			this.bottomSplitter.Name = "bottomSplitter";
			this.bottomSplitter.Size = new System.Drawing.Size(896, 4);
			this.bottomSplitter.TabIndex = 6;
			this.bottomSplitter.TabStop = false;
			this.bottomSplitter.MouseLeave += new System.EventHandler(this.bottomSplitter_MouseLeave);
			// 
			// statusBar1
			// 
			this.statusBar1.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.statusBar1.Dock = System.Windows.Forms.DockStyle.None;
			this.statusBar1.Location = new System.Drawing.Point(0, 590);
			this.statusBar1.Name = "statusBar1";
			this.statusBar1.ShowPanels = true;
			this.statusBar1.Size = new System.Drawing.Size(896, 18);
			this.statusBar1.TabIndex = 8;
			this.statusBar1.Text = "statusBar1";
			// 
			// mainToolBar
			// 
			this.mainToolBar.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.mainToolBar.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																																									 this.saveToolBarButton,
																																									 this.separator1,
																																									 this.startToolBarButton,
																																									 this.stopToolBarButton,
																																									 this.pauseToolBarButton});
			this.mainToolBar.Dock = System.Windows.Forms.DockStyle.None;
			this.mainToolBar.DropDownArrows = true;
			this.mainToolBar.Enabled = false;
			this.mainToolBar.ImageList = this.toolBarIconsImageList;
			this.mainToolBar.Location = new System.Drawing.Point(0, 0);
			this.mainToolBar.Name = "mainToolBar";
			this.mainToolBar.ShowToolTips = true;
			this.mainToolBar.Size = new System.Drawing.Size(896, 28);
			this.mainToolBar.TabIndex = 0;
			this.mainToolBar.Wrappable = false;
			this.mainToolBar.EnabledChanged += new System.EventHandler(this.mainToolBar_EnabledChanged);
			this.mainToolBar.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.mainToolBar_ButtonClick);
			// 
			// saveToolBarButton
			// 
			this.saveToolBarButton.Enabled = false;
			this.saveToolBarButton.ImageIndex = 3;
			this.saveToolBarButton.Tag = "save";
			// 
			// separator1
			// 
			this.separator1.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// startToolBarButton
			// 
			this.startToolBarButton.Enabled = false;
			this.startToolBarButton.ImageIndex = 0;
			this.startToolBarButton.Tag = "start";
			this.startToolBarButton.ToolTipText = "Start simulation";
			// 
			// stopToolBarButton
			// 
			this.stopToolBarButton.Enabled = false;
			this.stopToolBarButton.ImageIndex = 1;
			this.stopToolBarButton.Tag = "stop";
			this.stopToolBarButton.ToolTipText = "Stop the simulator";
			// 
			// pauseToolBarButton
			// 
			this.pauseToolBarButton.Enabled = false;
			this.pauseToolBarButton.ImageIndex = 2;
			this.pauseToolBarButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.pauseToolBarButton.Tag = "pause";
			this.pauseToolBarButton.ToolTipText = "Pause/Resume Simulation";
			// 
			// toolBarIconsImageList
			// 
			this.toolBarIconsImageList.ImageSize = new System.Drawing.Size(16, 16);
			this.toolBarIconsImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("toolBarIconsImageList.ImageStream")));
			this.toolBarIconsImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// openProjectFileDialog
			// 
			this.openProjectFileDialog.Filter = "XML files|*.xml|All files|*.*";
			this.openProjectFileDialog.FileOk += new System.ComponentModel.CancelEventHandler(this.openProjectFileDialog_FileOk);
			// 
			// newProjectFileDialog
			// 
			this.newProjectFileDialog.AddExtension = false;
			this.newProjectFileDialog.FileName = "MyNetwork";
			this.newProjectFileDialog.InitialDirectory = "Projects";
			this.newProjectFileDialog.RestoreDirectory = true;
			this.newProjectFileDialog.Title = "New Project ...";
			this.newProjectFileDialog.FileOk += new System.ComponentModel.CancelEventHandler(this.newProjectFileDialog_FileOk);
			// 
			// zoomComboBox
			// 
			this.zoomComboBox.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.zoomComboBox.Enabled = false;
			this.zoomComboBox.Items.AddRange(new object[] {
																											"25%",
																											"50%",
																											"75%",
																											"100%",
																											"125%",
																											"150%",
																											"200%",
																											"300%"});
			this.zoomComboBox.Location = new System.Drawing.Point(840, 8);
			this.zoomComboBox.Name = "zoomComboBox";
			this.zoomComboBox.Size = new System.Drawing.Size(48, 21);
			this.zoomComboBox.TabIndex = 10;
			this.zoomComboBox.Tag = "";
			this.zoomComboBox.Text = "100%";
			this.zoomComboBox.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.zoomComboBox_KeyPress);
			this.zoomComboBox.Validating += new System.ComponentModel.CancelEventHandler(this.zoomComboBox_Validating);
			this.zoomComboBox.SelectedIndexChanged += new System.EventHandler(this.zoomComboBox_SelectedIndexChanged);
			// 
			// label1
			// 
			this.label1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label1.Location = new System.Drawing.Point(800, 12);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(40, 16);
			this.label1.TabIndex = 11;
			this.label1.Text = "Zoom:";
			// 
			// nodeContextMenu
			// 
			this.nodeContextMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																																										this.captionMenuItem,
																																										this.enableDisableMenuItem,
																																										this.propertiesMenuItem,
																																										this.renameMenuItem,
																																										this.deleteMenuItem,
																																										this.editConfigFileMenuItem,
																																										this.appearanceMenuItem});
			this.nodeContextMenu.Popup += new System.EventHandler(this.nodeContextMenu_Popup);
			// 
			// captionMenuItem
			// 
			this.captionMenuItem.Index = 0;
			this.captionMenuItem.OwnerDraw = true;
			this.captionMenuItem.Text = "Caption";
			this.captionMenuItem.DrawItem += new System.Windows.Forms.DrawItemEventHandler(this.captionMenuItem_DrawItem);
			this.captionMenuItem.MeasureItem += new System.Windows.Forms.MeasureItemEventHandler(this.captionMenuItem_MeasureItem);
			// 
			// enableDisableMenuItem
			// 
			this.enableDisableMenuItem.Index = 1;
			this.enableDisableMenuItem.Text = "Enable/Disable";
			this.enableDisableMenuItem.Visible = false;
			this.enableDisableMenuItem.Click += new System.EventHandler(this.enableDisableMenuItem_Click);
			// 
			// propertiesMenuItem
			// 
			this.propertiesMenuItem.DefaultItem = true;
			this.propertiesMenuItem.Index = 2;
			this.propertiesMenuItem.Text = "Properties";
			this.propertiesMenuItem.Click += new System.EventHandler(this.propertiesMenuItem_Click);
			// 
			// renameMenuItem
			// 
			this.renameMenuItem.Index = 3;
			this.renameMenuItem.Text = "Rename";
			this.renameMenuItem.Click += new System.EventHandler(this.renameMenuItem_Click);
			// 
			// deleteMenuItem
			// 
			this.deleteMenuItem.Index = 4;
			this.deleteMenuItem.Text = "Delete";
			this.deleteMenuItem.Click += new System.EventHandler(this.deleteMenuItem_Click);
			// 
			// editConfigFileMenuItem
			// 
			this.editConfigFileMenuItem.Index = 5;
			this.editConfigFileMenuItem.Text = "Edit configuration file";
			this.editConfigFileMenuItem.Click += new System.EventHandler(this.editConfigFileMenuItem_Click);
			// 
			// appearanceMenuItem
			// 
			this.appearanceMenuItem.Index = 6;
			this.appearanceMenuItem.Text = "Appearance";
			this.appearanceMenuItem.Click += new System.EventHandler(this.appearanceMenuItem_Click);
			// 
			// leftPanel
			// 
			this.leftPanel.Controls.Add(this.leftTabControl);
			this.leftPanel.Controls.Add(this.leftDescriptionSplitter);
			this.leftPanel.Controls.Add(this.leftDescriptionLabel);
			this.leftPanel.Dock = System.Windows.Forms.DockStyle.Left;
			this.leftPanel.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.leftPanel.Location = new System.Drawing.Point(0, 45);
			this.leftPanel.Name = "leftPanel";
			this.leftPanel.Size = new System.Drawing.Size(192, 436);
			this.leftPanel.TabIndex = 12;
			// 
			// leftDescriptionSplitter
			// 
			this.leftDescriptionSplitter.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.leftDescriptionSplitter.Location = new System.Drawing.Point(0, 382);
			this.leftDescriptionSplitter.MinSize = 5;
			this.leftDescriptionSplitter.Name = "leftDescriptionSplitter";
			this.leftDescriptionSplitter.Size = new System.Drawing.Size(192, 4);
			this.leftDescriptionSplitter.TabIndex = 2;
			this.leftDescriptionSplitter.TabStop = false;
			// 
			// leftDescriptionLabel
			// 
			this.leftDescriptionLabel.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.leftDescriptionLabel.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.leftDescriptionLabel.Location = new System.Drawing.Point(0, 386);
			this.leftDescriptionLabel.Name = "leftDescriptionLabel";
			this.leftDescriptionLabel.Size = new System.Drawing.Size(192, 50);
			this.leftDescriptionLabel.TabIndex = 2;
			// 
			// leftSplitter
			// 
			this.leftSplitter.Location = new System.Drawing.Point(192, 45);
			this.leftSplitter.Name = "leftSplitter";
			this.leftSplitter.Size = new System.Drawing.Size(4, 436);
			this.leftSplitter.TabIndex = 13;
			this.leftSplitter.TabStop = false;
			// 
			// mainPanel
			// 
			this.mainPanel.AllowDrop = true;
			this.mainPanel.AutoScroll = true;
			this.mainPanel.AutoScrollMargin = new System.Drawing.Size(10, 10);
			this.mainPanel.BackColor = System.Drawing.SystemColors.ControlLight;
			this.mainPanel.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.mainPanel.Dock = System.Windows.Forms.DockStyle.Fill;
			this.mainPanel.Enabled = false;
			this.mainPanel.Location = new System.Drawing.Point(196, 45);
			this.mainPanel.Name = "mainPanel";
			this.mainPanel.Size = new System.Drawing.Size(700, 436);
			this.mainPanel.TabIndex = 1;
			this.mainPanel.DragEnter += new System.Windows.Forms.DragEventHandler(this.mainPanel_DragEnter);
			this.mainPanel.DragDrop += new System.Windows.Forms.DragEventHandler(this.mainPanel_DragDrop);
			// 
			// autoHideCheckBox
			// 
			this.autoHideCheckBox.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.autoHideCheckBox.Location = new System.Drawing.Point(810, 570);
			this.autoHideCheckBox.Name = "autoHideCheckBox";
			this.autoHideCheckBox.Size = new System.Drawing.Size(80, 20);
			this.autoHideCheckBox.TabIndex = 3;
			this.autoHideCheckBox.Text = "Auto Hide";
			// 
			// openMainConfigFileDialog
			// 
			this.openMainConfigFileDialog.FileName = "GuiConfig.xml";
			this.openMainConfigFileDialog.Filter = "XML files|*.xml|All files|*.*";
			this.openMainConfigFileDialog.Title = "Main configuration file";
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(896, 609);
			this.Controls.Add(this.autoHideCheckBox);
			this.Controls.Add(this.mainPanel);
			this.Controls.Add(this.leftSplitter);
			this.Controls.Add(this.leftPanel);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.zoomComboBox);
			this.Controls.Add(this.mainToolBar);
			this.Controls.Add(this.statusBar1);
			this.Controls.Add(this.bottomSplitter);
			this.Controls.Add(this.bottomTabControl);
			this.DockPadding.Bottom = 20;
			this.DockPadding.Top = 45;
			this.Menu = this.mainMenu;
			this.MinimumSize = new System.Drawing.Size(300, 150);
			this.Name = "MainForm";
			this.Text = "Network Simulator";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
			this.Load += new System.EventHandler(this.MainForm_Load);
			this.leftTabControl.ResumeLayout(false);
			this.nodesTabPage.ResumeLayout(false);
			this.linksTabPage.ResumeLayout(false);
			this.bottomTabControl.ResumeLayout(false);
			this.outputTabPage.ResumeLayout(false);
			this.commandTabPage.ResumeLayout(false);
			this.errorsTabPage.ResumeLayout(false);
			this.leftPanel.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private static void ParseArgs(string[] args, SplashScreen splash)
		{
			if (args.Length == 0)
				return;

			string new_cur_dir = null;

			bool config_next = false;
			for (int i = 0; i < args.Length; i++)
			{
				if (args[i] == "-c")
				{
					config_next = true;
					continue;
				}

				if (config_next) // config file here
				{
					string path = System.IO.Path.GetFullPath(args[i]);
					if (System.IO.File.Exists(path))
					{
						new_cur_dir = System.IO.Path.GetDirectoryName(path);
						ConfigFile = path;
					}
					else
					{
						splash.Visible = false;
						MessageBox.Show(null, String.Format("Cannot find GUI configuration file '{0}'. Default value of '{1}' will be searched next.",
							path, ConfigFile), "Unable to find configuration file");
						splash.Visible = true;
					}
				}
				else // project file here
				{
					string path = System.IO.Path.GetFullPath(args[i]);
					if (System.IO.File.Exists(path))
					{
						FirstProjectToOpen = path;
					}
					else
					{
						splash.Visible = false;
						MessageBox.Show(null, String.Format("Cannot find project file '{0}'.",
							path), "Unable to open project");
						splash.Visible = true;
					}
				}

			}

			// finally change current directory
			if (new_cur_dir != null)
				Environment.CurrentDirectory = new_cur_dir;
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args) 
		{
			// Handle unhandled thread exceptions
			Application.ThreadException += new ThreadExceptionEventHandler(AppThreadException);

			SplashScreen splash = null;
			try
			{
				// show that app is starting
				splash = new SplashScreen();
				splash.Show();

				ParseArgs(args, splash);

				// initialize
				MainForm main_form = new MainForm();
				instance = main_form;
				bool initialized = main_form.Initialize(splash);

				// end initialization
				splash.Close();
				splash = null;

				// run
				if (initialized)
					Application.Run(main_form);
			}
			catch (Exception e)
			{
				// uncaught exception - display error box
				ErrorForm e_form = new ErrorForm();
				e_form.SetError(e, false);
				e_form.ShowDialog();
			}
			finally
			{
				// make sure the splash screen is closed
				if (splash != null)
					splash.Close();
			}
		}

		/// <summary>
		/// Called if some event handler throws an exception.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		static void AppThreadException(object sender, ThreadExceptionEventArgs e) 
		{
			ErrorForm e_form = new ErrorForm();
			e_form.SetError(e.Exception, true);

			e_form.ShowDialog();

			if (!e_form.ShouldContinue)
				Application.Exit();
		}

		public static void pictureBox_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			PictureBox pb = (PictureBox)sender;
			DisplayElement de = (DisplayElement)pb.Tag;

			DisplayElement.LastSelected = de;
			de.BringToFront();

			if (e.Button == MouseButtons.Left) // just left
			{
				de.Picture.BorderStyle = BorderStyle.FixedSingle;
				de.Dragging = true;
				de.LastX = MousePosition.X;
				de.LastY = MousePosition.Y;
 
				pb.Capture = true;
			}
		}

		public static void pictureBox_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			PictureBox pb = (PictureBox)sender;
			DisplayElement de = (DisplayElement)pb.Tag;

			de.Picture.BorderStyle = BorderStyle.None;
			de.Dragging = false;
			pb.Capture = false;

			// check the position of all elements and possibly move them
			MainForm.Instance.projectData.CheckPositionAllElements(false);
			// redraw all lines after moving elements
			MainForm.Instance.mainPanel.Invalidate();

			MainForm.Instance.projectData.Modified = true;
			MainForm.Instance.mainPanel.ScrollControlIntoView(de.Picture);
		}

		public static void pictureBox_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			PictureBox pb = (PictureBox)sender;
			DisplayElement de = (DisplayElement)pb.Tag;
			if (de.Dragging)
			{
				Point p1 = de.Center;

				int mouse_x = MousePosition.X;
				int mouse_y = MousePosition.Y;

				de.MoveXY(mouse_x - de.LastX, mouse_y - de.LastY);

				de.LastX = mouse_x;
				de.LastY = mouse_y;

				//
				// invalidate regions to repaint lines
				//
				if (de.Config.Kind == BasicObjectKind.Node) // for nodes repaint its links
				{
					System.Drawing.Drawing2D.GraphicsPath path = new System.Drawing.Drawing2D.GraphicsPath();
					path.FillMode = System.Drawing.Drawing2D.FillMode.Winding;

					NodeConfig nc = (NodeConfig)de.Config;
					foreach (AdapterConfig ac in nc.GetItems(NodeItemKind.Adapter))
					{
						DisplayElement link_el = ac.GetLinkElement();
						if (link_el == null)
							continue;

						Point p2 = link_el.Center;
						int width = Math.Abs(p1.X - p2.X) + 10;
						int height = Math.Abs(p1.Y - p2.Y) + 10;
						path.AddRectangle(new Rectangle(Math.Min(p1.X, p2.X) - 5, Math.Min(p1.Y, p2.Y) - 5, width, height));
						path.CloseFigure();
					}

					MainForm.Instance.mainPanel.Invalidate(new Region(path), false);
					path.Dispose();
				}
				else // we don't have information about lines, invalidate whole panel
				{
					MainForm.Instance.mainPanel.Invalidate(false);
				}

			}
		}

		private void bottomTabControl_Click(object sender, System.EventArgs e)
		{
			BottomTabControlHide(false);
		}

		/// <summary>
		/// Called when the project is loaded or closed to enable/disable controls that should be
		/// accessible only if a project is open.
		/// </summary>
		/// <param name="enable"></param>
		private void SetFocusProjectControls(bool enable)
		{
			mainPanel.Enabled = enable;
			mainPanel.AllowDrop = enable;
			zoomComboBox.Enabled = enable;
			fileSaveMenuItem.Enabled = enable;
			fileCloseMenuItem.Enabled = enable;
			filePrintMenuItem.Enabled = enable;
			simulationMenuItem.Enabled = enable;
			toolsCenterDiagramMenuItem.Enabled = enable;
			mainToolBar.Enabled = enable;

			if (enable)
			{
				mainPanel.BackColor = Color.FromKnownColor(KnownColor.Window);
				mainPanel.Focus();
			}
			else
			{
				mainPanel.BackColor = Color.FromKnownColor(KnownColor.ControlLight);
			}

			// and while the project is loaded the simulator is not running etc..
			SetFocusSimulatorControls(false, enable);
		}

		/// <summary>
		/// Called when the simulation is started or exited to enable/disable controls that should be
		/// accessible only if the simulation is running.
		/// </summary>
		/// <param name="enable"><b>True</b> if the simulator is running.</param>
		/// <param name="projectLoaded"><b>True</b> if a project is currently loaded.</param>
		public void SetFocusSimulatorControls(bool enable, bool projectLoaded)
		{
			commandTextBox.ReadOnly = !(enable && projectLoaded);
			commandTextBox.BackColor =
				commandTextBox.ReadOnly ? Color.FromKnownColor(KnownColor.ControlLight)	: Color.FromKnownColor(KnownColor.Window);

			simulationStartMenuItem.Enabled = !enable && projectLoaded;
			startToolBarButton.Enabled = !enable && projectLoaded;

			simulationStopMenuItem.Enabled = enable && projectLoaded;
			stopToolBarButton.Enabled = enable && projectLoaded;

			simulationPauseMenuItem.Enabled = enable && projectLoaded;
			pauseToolBarButton.Enabled = enable && projectLoaded;

			enableDisableMenuItem.Visible = enable;
			renameMenuItem.Enabled = !enable;
			deleteMenuItem.Enabled = !enable;
		}

		private void RepaintElement(DisplayElement de)
		{
			if (de.Config.Kind == BasicObjectKind.Node) // nodes repaint their links
			{
				Graphics gr = mainPanel.CreateGraphics();
				
				NodeConfig nc = (NodeConfig)de.Config;
				Point elem_center = de.Center;
				foreach (AdapterConfig ac in nc.GetItems(NodeItemKind.Adapter))
				{
					DisplayElement link_el = ac.GetLinkElement();
					if (link_el == null)
						continue;

					Point link_center = link_el.Center;
					connectionDrawPen.Width = DefaultConnectionWidth * projectData.Zoom / 100f;
					connectionDrawPen.Color = link_el.Color;
					gr.DrawLine(connectionDrawPen, elem_center, link_center);
				}

				gr.Dispose();
			}
		}

		/// <summary>
		/// Initializes icons in the left nodes panel.
		/// </summary>
		/// <param name="lvCollection">Collection where the icons will be inserted.</param>
		/// <param name="nodes">List of <see cref="PredefinedNode"/> objects.</param>
		private void InitPredefinedNodesList(ListView.ListViewItemCollection lvCollection, IList nodes)
		{
			foreach (PredefinedNode node in nodes)
			{
				ListViewItem item = new ListViewItem(node.Caption, node.IconIndex);
				item.Tag = node;
				lvCollection.Add(item);
			}
		}

		/// <summary>
		/// Initializes icons in the left links panel.
		/// </summary>
		/// <param name="lvCollection">Collection where the icons will be inserted.</param>
		/// <param name="links">List of <see cref="PredefinedLink"/> objects.</param>
		private void InitPredefinedLinksList(ListView.ListViewItemCollection lvCollection, IList links)
		{
			foreach (PredefinedLink link in links)
			{
				ListViewItem item = new ListViewItem(link.Caption, link.IconIndex);
				item.Tag = link;
				lvCollection.Add(item);
			}
		}

		private void fileExitMenuItem_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void fileNewMenuItem_Click(object sender, System.EventArgs e)
		{
			// close current project
			if (!CloseCurrentProject())
				return;

			newProjectFileDialog.ShowDialog();
		}

		private void fileSaveMenuItem_Click(object sender, System.EventArgs e)
		{
			SaveCurrentProject();
		}

		/// <summary>
		/// Removes all controls and drawings for current project and sets <see cref="projectData"/> to <b>null</b>.
		/// </summary>
		/// <returns><b>True</b> if closed.</returns>
		private bool CloseCurrentProject()
		{
			// save if needed
			if (projectData != null)
			{
				if (SimulatorControl.Instance.Running)
				{
					DialogResult result = MessageBox.Show(this, "The simulator is running. Stop the simulation?",
						"End the simulation?", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
					if (result == DialogResult.No)
						return false;

					SimulatorControl.Instance.Stop();
					// previous method waits for the simulator, however, we will better wait some more time..
					Thread.Sleep(1000);
				}

				if (projectData.Modified)
				{
					DialogResult result = MessageBox.Show(this, "Project was modified. Save changes?",
						"Save changes?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question,
						MessageBoxDefaultButton.Button1);

					if (result == DialogResult.Cancel)
						return false;

					if (result == DialogResult.Yes) // do it as the user clicked Save first
						if (!SaveCurrentProject())
							return false;

					projectData.RepaintEvent -= new ProjectData.RepaintDelegate(RepaintElement);
				}
			}

			SetFocusProjectControls(false);
			mainPanel.Controls.Clear();
			outputTextBox.Clear();
			commandTextBox.Clear();
			errorsListView.Items.Clear();
			projectData = null;

			return true;
		}

		private void OpenProject(string configFile)
		{
			try
			{
				projectData = new ProjectData();
				projectData.RepaintEvent += new ProjectData.RepaintDelegate(RepaintElement);
				projectData.Load(configFile);
				SetFocusProjectControls(true);
			}
			catch (Exception e)
			{
				MessageBox.Show(this, "Error loading project: " + e.Message, "Cannot load project",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		/// <summary>
		/// Saves completely current project.
		/// </summary>
		/// <returns><b>True</b> if saved.</returns>
		private bool SaveCurrentProject()
		{
			if (projectData != null && projectData.Modified) // save only if project is loaded and modified
			{
				try { projectData.Save(); }
				catch (Exception e)
				{
					MessageBox.Show(this, String.Format("Unable to save the project: {0}", e.Message), "Error saving project",
						MessageBoxButtons.OK, MessageBoxIcon.Error);
					return false;
				}
			}

			return true;
		}

		private void fileOpenMenuItem_Click(object sender, System.EventArgs e)
		{
			// save if needed
			if (!CloseCurrentProject())
				return;

			openProjectFileDialog.ShowDialog();
		}

		private void newProjectFileDialog_FileOk(object sender, System.ComponentModel.CancelEventArgs e)
		{
			// set back current working directory if changed
			System.IO.Directory.SetCurrentDirectory(generalData.RootDirectory);

			SaveFileDialog dlg = (SaveFileDialog) sender;
			
			if (System.IO.Path.GetExtension(dlg.FileName) != String.Empty)
			{
				MessageBox.Show(this, "Specify the name of your project, the directory of the same name will be created in current directory.",
					"Don't add extension", MessageBoxButtons.OK, MessageBoxIcon.Information);
				e.Cancel = true;
				return;
			}

			try
			{
				// copy the data from template directory
				string copy_from = System.IO.Path.Combine(generalData.RootDirectory, System.IO.Path.GetDirectoryName(NewProjectFile));
				FileSystem.CopyDirectory(copy_from, dlg.FileName, true);

				// rename the project file
				string project_name = System.IO.Path.GetFileName(dlg.FileName);
				string new_file = System.IO.Path.Combine(dlg.FileName, project_name);
				new_file = System.IO.Path.ChangeExtension(new_file, ProjectExtension);
				string rename_from = System.IO.Path.Combine(dlg.FileName, System.IO.Path.GetFileName(NewProjectFile));
				System.IO.File.Move(rename_from, new_file);

				// and open the project
				OpenProject(new_file);
			}
			catch (Exception exc)
			{
				MessageBox.Show(this, String.Format("Error creating the project: {0}", exc.Message),
					"Cannot create new project", MessageBoxButtons.OK, MessageBoxIcon.Error);

				// try to cleanup
				try { System.IO.Directory.Delete(dlg.FileName, true); }
				catch {}
			}
		}

		private void openProjectFileDialog_FileOk(object sender, System.ComponentModel.CancelEventArgs e)
		{
			// set back current working directory if changed
			System.IO.Directory.SetCurrentDirectory(generalData.RootDirectory);

			OpenFileDialog dlg = (OpenFileDialog) sender;
			OpenProject(dlg.FileName);
		}

		private void fileCloseMenuItem_Click(object sender, System.EventArgs e)
		{
			CloseCurrentProject();
		}

		private void mainPanel_DragDrop(object sender, System.Windows.Forms.DragEventArgs e)
		{
			if ((e.Effect & DragDropEffects.Copy) == DragDropEffects.Copy)
			{
				// convert screen coordinates to main-panel relative coordinates
				Point point = mainPanel.PointToScreen(new Point(0,0));
				point.X = e.X - point.X;
				point.Y = e.Y - point.Y;

				if (e.Data.GetDataPresent(typeof(PredefinedNode)))
					projectData.AddBasicItem((PredefinedNode)e.Data.GetData(typeof(PredefinedNode)), point);
				else if (e.Data.GetDataPresent(typeof(PredefinedLink)))
					projectData.AddBasicItem((PredefinedLink)e.Data.GetData(typeof(PredefinedLink)), point);
			}
		}

		private void nodesListView_ItemDrag(object sender, System.Windows.Forms.ItemDragEventArgs e)
		{
			if (e.Button == MouseButtons.Left) // allow only left button
			{
				ListViewItem item = (ListViewItem)e.Item;
				DoDragDrop(item.Tag, DragDropEffects.Copy);
			}
		}

		private void leftListView_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			ListView view = (ListView)sender;
			if (view.SelectedItems.Count == 0)
			{
				leftDescriptionLabel.Text = String.Empty;
			}
			else
			{
				ListViewItem item = view.SelectedItems[0];
				PredefinedObject po = item.Tag as PredefinedObject;
				if (po != null)
				{
					leftDescriptionLabel.Text = po.Description;
				}
			}
		}

		private void mainPanel_DragEnter(object sender, System.Windows.Forms.DragEventArgs e)
		{
			if (
				(e.AllowedEffect & DragDropEffects.Copy) == DragDropEffects.Copy
				&& (e.Data.GetDataPresent(typeof(PredefinedNode)) || e.Data.GetDataPresent(typeof(PredefinedLink)))
				)
			{
				e.Effect = DragDropEffects.Copy;
			}
		}

		private void linksListView_ItemDrag(object sender, System.Windows.Forms.ItemDragEventArgs e)
		{
			if (e.Button == MouseButtons.Left) // allow only left button
			{
				ListViewItem item = (ListViewItem)e.Item;
				DoDragDrop(item.Tag, DragDropEffects.Copy);
			}
		}

		private void zoomComboBox_Validating(object sender, System.ComponentModel.CancelEventArgs e)
		{
			ComboBox cb = (ComboBox)sender;
			
			string text = cb.Text;
			int percents_index = text.IndexOf('%');
			if (percents_index >= 0)
				text = text.Substring(0, percents_index);
			text = text.Trim();

			int value = -1;
			try { value = Int32.Parse(text); }
			catch {}

			if (value >= 0)
			{
				if (value < 25) value = 25;
				if (value > 300) value = 300;
				
				mainPanel.Scale(value / (float)projectData.Zoom);
				mainPanel.Invalidate();
				projectData.Zoom = value;
				projectData.CheckPositionAllElements(true);
			}

			cb.Text = projectData.Zoom.ToString() + "%";
		}

		private void zoomComboBox_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			mainPanel.Focus();
		}

		private void zoomComboBox_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
		{
			if (e.KeyChar == (char)13) // enter
			{
				e.Handled = true;
				mainPanel.Focus();
			}
		}

		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (!CloseCurrentProject())
				e.Cancel = true;
		}

		private void MainForm_Load(object sender, System.EventArgs e)
		{
			SetFocusProjectControls(false);

			// and open project if one was specified on the command line
			if (FirstProjectToOpen != null)
			{
				OpenProject(FirstProjectToOpen);
			}
		}

		private void filePrintMenuItem_Click(object sender, System.EventArgs e)
		{
			// printDialog.ShowDialog(this);
		}

		private void propertiesMenuItem_Click(object sender, System.EventArgs e)
		{
			DisplayElement de = DisplayElement.LastSelected;
			if (de == null)
				return;

			if (de.Config.Kind == BasicObjectKind.Node)
			{
				NodeProperties prop_dlg = new NodeProperties();
				prop_dlg.SetElement(de);
				prop_dlg.ShowDialog(this);
				prop_dlg.Dispose();

				mainPanel.Invalidate(false); // links might have change, redraw..
			}
		}

		private void simulationStartMenuItem_Click(object sender, System.EventArgs e)
		{
			StartSimulator();
		}

		private void StartSimulator()
		{
			if (projectData.Modified)
			{
				DialogResult res = MessageBox.Show(this, "Project is modified. Would you like to save changes before starting the simulator?", "Save?",
					MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1);
				if (res == DialogResult.Yes)
					SaveCurrentProject();
			}

			// clear all simulator output from the previous run
			commandTextBox.Clear();
			commandTextBoxEditFrom = 0;
			errorsListView.Items.Clear();

			try
			{
				Gui.Console.Write("Starting up simulator... ");
				SimulatorControl.Instance.Start(generalData.SimulatorPath, projectData.FileName);
				Gui.Console.WriteLine("Done.");
			}
			catch (Exception exc)
			{
				Gui.Console.WriteLine(exc.Message);
				Gui.Console.WriteLine(exc.ToString());
			}
		}

		private void simulationStopMenuItem_Click(object sender, System.EventArgs e)
		{
			StopSimulator();
		}

		private void StopSimulator()
		{
			try
			{
				Gui.Console.Write("Shutting down simulator... ");
				SimulatorControl.Instance.Stop();
				Gui.Console.WriteLine("Done.");
			}
			catch (Exception exc)
			{
				Gui.Console.WriteLine(exc.Message);
				Gui.Console.WriteLine(exc.ToString());
			}
		}

		private void mainPanel_Paint(object sender, PaintEventArgs e)
		{
			if (projectData != null)
				projectData.RepaintAll();
		}

		private void deleteMenuItem_Click(object sender, System.EventArgs e)
		{
			DisplayElement de = DisplayElement.LastSelected;
			if (de == null)
				return;

			DialogResult res = MessageBox.Show(this, String.Format("Delete '{0}'?", de.Name), "Confirmation",
				MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
			if (res == DialogResult.Yes)
			{
				projectData.DeleteBasicItem(de);
				mainPanel.Invalidate();
			}
		}

		private void renameMenuItem_Click(object sender, System.EventArgs e)
		{
			DisplayElement de = DisplayElement.LastSelected;
			if (de == null)
				return;

			RenameItem rename_dlg = new RenameItem();
			rename_dlg.SetItem(de);
			rename_dlg.ShowDialog(this);
			rename_dlg.Dispose();
		}

		private void captionMenuItem_DrawItem(object sender, System.Windows.Forms.DrawItemEventArgs e)
		{
			DisplayElement de = DisplayElement.LastSelected;
			Font font = new Font("Arial", 8, FontStyle.Bold, GraphicsUnit.Point);
			SizeF string_size = e.Graphics.MeasureString(de.Name, font);

			e.Graphics.FillRectangle(System.Drawing.Brushes.DarkBlue, e.Bounds);

			if (de != null)
				e.Graphics.DrawString(de.Name, font, System.Drawing.Brushes.White, e.Bounds.X + 5, e.Bounds.Y);
		}

		private void captionMenuItem_MeasureItem(object sender, System.Windows.Forms.MeasureItemEventArgs e)
		{
			e.ItemHeight = 14;
		}

		private void nodeContextMenu_Popup(object sender, System.EventArgs e)
		{
			DisplayElement de = DisplayElement.LastSelected;
			if (de == null)
				return;

			// customize items in the menu according to the item clicked
			editConfigFileMenuItem.Enabled = de.Config.HasConfigFile();
			propertiesMenuItem.Visible = de.Config.Kind == BasicObjectKind.Node;
		}

		private void editConfigFileMenuItem_Click(object sender, System.EventArgs e)
		{
			DisplayElement de = DisplayElement.LastSelected;
			if (de == null)
				return;

			de.Config.EditConfigFile();
		}

		private void leftTabControl_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			TabControl tc = (TabControl)sender;
			if (tc.SelectedTab != null)
			{
				// causes actualization of description text
				if (tc.SelectedTab.Name == "nodesTabPage")
					leftListView_SelectedIndexChanged(nodesListView, e);
				else if (tc.SelectedTab.Name == "linksTabPage")
					leftListView_SelectedIndexChanged(linksListView, e);
			}
		}

		private void toolsAdaptersMenuItem_Click(object sender, System.EventArgs e)
		{
			this.nodeItemsListForm.CurrentNodeItem = NodeItemKind.Adapter;
			this.nodeItemsListForm.ShowDialog(this);
		}

		private void toolsModulesMenuItem_Click(object sender, System.EventArgs e)
		{
			this.nodeItemsListForm.CurrentNodeItem = NodeItemKind.Module;
			this.nodeItemsListForm.ShowDialog(this);
		}

		private void toolsApplicationsMenuItem_Click(object sender, System.EventArgs e)
		{
			this.nodeItemsListForm.CurrentNodeItem = NodeItemKind.Application;
			this.nodeItemsListForm.ShowDialog(this);
		}

		private void appearanceMenuItem_Click(object sender, System.EventArgs e)
		{
			DisplayElement de = DisplayElement.LastSelected;
			if (de == null)
				return;

			ItemAppearanceForm appear_dlg = new ItemAppearanceForm();
			appear_dlg.SetItem(de);
			appear_dlg.ShowDialog(this);
			appear_dlg.Dispose();
		}

		private void toolsCenterDiagramMenuItem_Click(object sender, System.EventArgs e)
		{
			projectData.CheckPositionAllElements(true);
			mainPanel.AutoScrollPosition = new System.Drawing.Point(
				(mainPanel.DisplayRectangle.Width - mainPanel.Width) / 2, (mainPanel.DisplayRectangle.Height - mainPanel.Height) / 2);

			mainPanel.Invalidate();
		}

		/// <summary>
		/// The position from which the text is editable. The text from the beginning to this position should
		/// remain unchanged.
		/// </summary>
		private int commandTextBoxEditFrom = 0;
		private void commandTextBox_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
		{
			if (commandTextBox.Disposing)
				return;

			lock (commandTextBox)
			{
				// the user can modify only the last line..
				if (commandTextBox.SelectionStart < commandTextBoxEditFrom ||
					(commandTextBox.SelectionStart == commandTextBoxEditFrom && e.KeyChar == (char)8)) // backspace
				{
					commandTextBox.SelectionLength = 0;
					commandTextBox.SelectionStart = commandTextBox.Text.Length;
					commandTextBox.ScrollToCaret();
					e.Handled = true;
				}
				else if (e.KeyChar == (char)13) // if the user pressed enter, send it to the simulator
				{
					string line = commandTextBox.Text.Substring(commandTextBoxEditFrom,
						commandTextBox.SelectionStart - commandTextBoxEditFrom);
					commandTextBoxEditFrom = commandTextBox.SelectionStart + Environment.NewLine.Length;
				
					SimulatorControl.Instance.Write(line + Environment.NewLine);
				}
			}
		}

		internal void WriteToCommandTextBox(string str)
		{
			lock (commandTextBox)
			{
				if (commandTextBox.Disposing)
					return;

				commandTextBox.Text = commandTextBox.Text.Insert(commandTextBoxEditFrom, str);
				commandTextBoxEditFrom += str.Length;

				if (commandTextBox.Visible)
				{
					commandTextBox.SelectionLength = 0;
					commandTextBox.SelectionStart = commandTextBox.Text.Length;
				}
			}

			if (commandTextBox.Visible)
				commandTextBox.ScrollToCaret();
		}

		private void mainToolBar_EnabledChanged(object sender, System.EventArgs e)
		{
			// with enabled changed for whole toolbar change the same for its items
			bool value = mainToolBar.Enabled;
			foreach (ToolBarButton b in mainToolBar.Buttons)
				b.Enabled = value;
		}

		private void simulationPauseMenuItem_Click(object sender, System.EventArgs e)
		{
			MenuItem pause_item = simulationPauseMenuItem; // do not use sender, it is called also by a toolbar
			pause_item.Checked = !pause_item.Checked;
			pauseToolBarButton.Pushed = pause_item.Checked;
		}

		private int hiddenBottomTabHeightRestore;
		private const int hiddenBottomTabHeight = 25;
		private void BottomTabControlHide(bool hide)
		{
			if (hide && bottomTabControl.Height > hiddenBottomTabHeight)
			{
				hiddenBottomTabHeightRestore = bottomTabControl.Height;
				bottomTabControl.Height = hiddenBottomTabHeight;
			}
			else if (!hide && bottomTabControl.Height == hiddenBottomTabHeight)
			{
				if (hiddenBottomTabHeightRestore < 40)
					hiddenBottomTabHeightRestore = 80;
				bottomTabControl.Height = hiddenBottomTabHeightRestore;
			}
		}

		private void bottomTabControlHideCheck(object state)
		{
			Thread.Sleep(500);
			int border = 7;
			System.Drawing.Point m = MousePosition;
			System.Drawing.Point lt = bottomTabControl.Parent.PointToScreen(bottomTabControl.Location);
			System.Drawing.Point rb = new Point(lt.X + bottomTabControl.Width, lt.Y + bottomTabControl.Height);
			if (m.X < lt.X+border || m.X > rb.X-border || m.Y < lt.Y-4 || m.Y > rb.Y-border)
				BottomTabControlHide(true);
		}

		private void bottomTabControl_MouseLeave(object sender, System.EventArgs e)
		{
			if (autoHideCheckBox.Checked)
			{
				System.Threading.ThreadPool.QueueUserWorkItem(new WaitCallback(bottomTabControlHideCheck));
			}
		}

		private void outputTextBox_MouseLeave(object sender, System.EventArgs e)
		{
			bottomTabControl_MouseLeave(sender, e);
		}

		private void bottomTabControl_MouseHover(object sender, System.EventArgs e)
		{
			BottomTabControlHide(false);
		}

		private void bottomSplitter_MouseLeave(object sender, System.EventArgs e)
		{
			bottomTabControl_MouseLeave(sender, e);
		}

		private void commandDescriptionLabel_MouseLeave(object sender, System.EventArgs e)
		{
			bottomTabControl_MouseLeave(sender, e);
		}

		private void commandTextBox_MouseLeave(object sender, System.EventArgs e)
		{
			bottomTabControl_MouseLeave(sender, e);
		}

		private void errorsListView_MouseLeave(object sender, System.EventArgs e)
		{
			bottomTabControl_MouseLeave(sender, e);
		}

		private void mainToolBar_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			//behave as the command from main menu was selected
			switch (e.Button.Tag.ToString())
			{
				case "save":
					fileSaveMenuItem_Click(sender, null);
					break;
				case "start":
					simulationStartMenuItem_Click(sender, null);
					break;
				case "stop":
					simulationStopMenuItem_Click(sender, null);
					break;
				case "pause":
					simulationPauseMenuItem_Click(sender, e);
					break;
			}
		}

		public void NewLogEntry(NetSim.Core.Event e)
		{
			ListViewItem.ListViewSubItem[] subitems = new ListViewItem.ListViewSubItem[2];
			for (int i = 0; i < subitems.Length; i++)
				subitems[i] = new ListViewItem.ListViewSubItem(); 
			ListViewItem item = new ListViewItem(subitems, 0);
			item.UseItemStyleForSubItems = false;

			switch (e.Type)
			{
				case NetSim.Core.Event.EventType.Error:
					subitems[0].Text = "Error";
					subitems[0].BackColor = Color.FromKnownColor(KnownColor.Red);
					break;
				case NetSim.Core.Event.EventType.Warning:
					subitems[0].Text = "Warning";
					subitems[0].BackColor = Color.FromKnownColor(KnownColor.Coral);
					break;
				case NetSim.Core.Event.EventType.Notice:
					subitems[0].Text = "Notice";
					break;
			}

			subitems[1] = new ListViewItem.ListViewSubItem(item, e.Message);

			item.Tag = e.Exception;
			this.errorsListView.Items.Add(item);
		}

		private void errorsListView_DoubleClick(object sender, System.EventArgs e)
		{
			ListView lv = (ListView)sender;
			if (lv.SelectedItems.Count > 0)
			{
				Gui.Console.WriteLine(lv.SelectedItems[0].Tag.ToString());
			}
		}

		private void helpCheckForUpdatesMenuItem_Click(object sender, System.EventArgs e)
		{
			// prepare the request
			string url = "http://pavelnovak.eu/en/development/netsim/version.txt";
			System.Net.HttpWebRequest req = (System.Net.HttpWebRequest)System.Net.WebRequest.Create(url);
			req.Accept = "text/plain";

			try
			{
				// read content
				System.IO.StreamReader sr = new System.IO.StreamReader(req.GetResponse().GetResponseStream());
				string response = sr.ReadToEnd();

				// check the value
				string web_version = response.Substring(response.LastIndexOfAny(new char[] {'\r', '\n', '\t', ' '}) + 1);

				// check if the version on the web is newer
				Version current = System.Reflection.Assembly.GetAssembly(typeof(NetSim.Core.Simulator)).GetName().Version;
				Version new_version = new Version(web_version);
				if (current < new_version)
				{
					NewVersionForm form = new NewVersionForm();
					form.CurrentVersion = current;
					form.NewVersion = new_version;
					form.ShowDialog(this);
				}
				else
				{
					MessageBox.Show(this, "No new version available.", "No new version", MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
			}
			catch (Exception exc)
			{
				MessageBox.Show(this, String.Format("Error while accessing {0}: {1}.", url, exc.Message),
					"Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void helpAboutMenuItem_Click(object sender, System.EventArgs e)
		{
			AboutForm form = new AboutForm();
			form.ShowDialog(this);
		}

		private void helpLicenseMenuItem_Click(object sender, System.EventArgs e)
		{
			bool ok = true;

			do
			{
				try
				{
					// find out where the license file is
					System.Reflection.Assembly core_assembly = System.Reflection.Assembly.GetAssembly(typeof(NetSim.Core.Simulator));
					object[] company_attr = core_assembly.GetCustomAttributes(typeof(System.Reflection.AssemblyCompanyAttribute), false);
					object[] product_attr = core_assembly.GetCustomAttributes(typeof(System.Reflection.AssemblyProductAttribute), false);
					if (company_attr.Length == 0 || product_attr.Length == 0)
					{
						ok = false;
						break;
					}

					RegistryKey rk = Registry.LocalMachine.OpenSubKey(String.Format("SOFTWARE\\{0}\\{1}",
						((System.Reflection.AssemblyCompanyAttribute)company_attr[0]).Company,
						((System.Reflection.AssemblyProductAttribute)product_attr[0]).Product));
					if (rk == null)
					{
						ok = false;
						break;
					}

					string file = rk.GetValue("InstallDir").ToString() + "License.txt";
					if (!System.IO.File.Exists(file))
					{
						ok = false;
						break;
					}

					System.Diagnostics.Process.Start(MainForm.Instance.generalData.ExternalEditor, file);
				}
				catch (Exception)
				{
					ok = false;
				}

			} while (false);
				
			if (!ok)
				MessageBox.Show(this, "Unable to locate license file or external editor.", "Error");
		}

		private void enableDisableMenuItem_Click(object sender, System.EventArgs e)
		{
			DisplayElement de = DisplayElement.LastSelected;
			if (de == null)
				return;

			SimulatorControl.Instance.EnableDisable(de.Config.Kind, de.Config.Name, !de.Enabled);
		}
	}

	public class GuiConsoleWriter : System.IO.StringWriter
	{
		private TextBoxBase textBox;

		public GuiConsoleWriter(TextBoxBase textBox)
		{
			this.textBox = textBox;
		}

		private void RewriteToConsole()
		{
			if (textBox.Disposing) // do not touch the control if it is closing..
				return;

			StringBuilder sb = GetStringBuilder();

			textBox.AppendText(sb.ToString());

			if (textBox.Visible)
			{
				textBox.Focus();
				textBox.SelectionLength = 0;
				textBox.SelectionStart = textBox.Text.Length;
			}
			sb.Remove(0, sb.Length);
		}

		#region Write methods

		public override void Write(bool value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(char value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(char[] buffer)
		{
			base.Write(buffer);
			RewriteToConsole();
		}

		public override void Write(char[] buffer, int index, int count)
		{
			base.Write(buffer, index, count);
			RewriteToConsole();
		}

		public override void Write(decimal value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(double value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(float value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(int value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(long value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(object value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(string format, object arg0)
		{
			base.Write(format, arg0);
			RewriteToConsole();
		}

		public override void Write(string format, object arg0, object arg1)
		{
			base.Write(format, arg0, arg1);
			RewriteToConsole();
		}

		public override void Write(string format, object arg0, object arg1, object arg2)
		{
			base.Write(format, arg0, arg1, arg2);
			RewriteToConsole();
		}

		public override void Write(string format, params object[] arg)
		{
			base.Write(format, arg);
			RewriteToConsole();
		}

		public override void Write(string value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(uint value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void Write(ulong value)
		{
			base.Write(value);
			RewriteToConsole();
		}

		public override void WriteLine()
		{
			base.WriteLine();
			RewriteToConsole();
		}

		public override void WriteLine(bool value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(char value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(char[] buffer)
		{
			base.WriteLine(buffer);
			RewriteToConsole();
		}

		public override void WriteLine(char[] buffer, int index, int count)
		{
			base.WriteLine(buffer, index, count);
			RewriteToConsole();
		}

		public override void WriteLine(decimal value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(double value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(float value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(int value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(long value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(object value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(string format, object arg0)
		{
			base.WriteLine(format, arg0);
			RewriteToConsole();
		}

		public override void WriteLine(string format, object arg0, object arg1)
		{
			base.WriteLine(format, arg0, arg1);
			RewriteToConsole();
		}

		public override void WriteLine(string format, object arg0, object arg1, object arg2)
		{
			base.WriteLine(format, arg0, arg1, arg2);
			RewriteToConsole();
		}

		public override void WriteLine(string format, params object[] arg)
		{
			base.WriteLine(format, arg);
			RewriteToConsole();
		}

		public override void WriteLine(string value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(uint value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		public override void WriteLine(ulong value)
		{
			base.WriteLine(value);
			RewriteToConsole();
		}

		#endregion
	}
}
