/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using NodeItemKind = NetSim.Core.NodeItemKind;

namespace NetSim.Gui
{
	/// <summary>
	/// Summary description for AdaptersListForm.
	/// </summary>
	public class NodeItemsListForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.ColumnHeader nameColumnHeader;
		private System.Windows.Forms.ColumnHeader classColumnHeader;
		private System.Windows.Forms.ColumnHeader descColumnHeader;
		private System.Windows.Forms.ListView adaptersListView;
		private System.Windows.Forms.Button okButton;
		private System.Windows.Forms.Button cancelButton;
		private System.ComponentModel.IContainer components = null;
		private System.Windows.Forms.Label tableDescLabel;
		private System.Windows.Forms.ColumnHeader iconColumnHeader;
		private System.Windows.Forms.ListView modulesListView;
		private System.Windows.Forms.ColumnHeader columnHeader1;
		private System.Windows.Forms.ColumnHeader columnHeader2;
		private System.Windows.Forms.ColumnHeader columnHeader3;
		private System.Windows.Forms.ColumnHeader columnHeader4;
		private System.Windows.Forms.ListView applicationsListView;
		private System.Windows.Forms.ColumnHeader columnHeader5;
		private System.Windows.Forms.ColumnHeader columnHeader6;
		private System.Windows.Forms.ColumnHeader columnHeader7;
		private System.Windows.Forms.ColumnHeader columnHeader8;

		/// <summary>
		/// Display element for which this form has been activated. If OK is pressed, the adapter will be
		/// added to the element.
		/// </summary>
		public NodeConfig CurrentNodeConfig
		{
			set
			{
				this.currentNodeConfig = value;
			}
		}
		private NodeConfig currentNodeConfig;

		private ListView currentListView;

		public NodeItemKind CurrentNodeItem
		{
			set
			{
				this.nodeItemKind = value;

				switch (nodeItemKind)
				{
					case NodeItemKind.Adapter:
						this.Text = "Adapters";
						tableDescLabel.Text = "Available adapters:";
						this.adaptersListView.Visible = true;
						this.modulesListView.Visible = false;
						this.applicationsListView.Visible = false;
						this.currentListView = adaptersListView;
						break;
					case NodeItemKind.Module:
						this.Text = "Modules";
						tableDescLabel.Text = "Available modules:";
						this.adaptersListView.Visible = false;
						this.modulesListView.Visible = true;
						this.applicationsListView.Visible = false;
						this.currentListView = modulesListView;
						break;
					case NodeItemKind.Application:
						this.Text = "Applications";
						tableDescLabel.Text = "Available applications:";
						this.adaptersListView.Visible = false;
						this.modulesListView.Visible = false;
						this.applicationsListView.Visible = true;
						this.currentListView = applicationsListView;
						break;
				}
			}
		}
		private NodeItemKind nodeItemKind;

		public NodeItemsListForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.adaptersListView.SmallImageList = MainForm.Instance.iconsImageList;
			this.modulesListView.SmallImageList = MainForm.Instance.iconsImageList;
			this.applicationsListView.SmallImageList = MainForm.Instance.iconsImageList;
		}

		/// <summary>
		/// Called after the main form and its general data is initialized.
		/// </summary>
		public void Initialize(NodeItemKind itemKind, IList itemList)
		{
			switch (itemKind)
			{
				case NodeItemKind.Adapter:
					AddItemsToList(MainForm.Instance.generalData.PredefinedAdapters, this.adaptersListView);
					break;
				case NodeItemKind.Module:
					AddItemsToList(MainForm.Instance.generalData.PredefinedModules, this.modulesListView);
					break;
				case NodeItemKind.Application:
					AddItemsToList(MainForm.Instance.generalData.PredefinedApplications, this.applicationsListView);
					break;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="objList">List of <see cref="PredefinedObject"/>s.</param>
		/// <param name="viewList"></param>
		private void AddItemsToList(IList objList, ListView viewList)
		{
			foreach (PredefinedObject po in objList)
			{
				ListViewItem item = new ListViewItem();
				item.Tag = po;

				item.ImageIndex = po.IconIndex;
				// first always empty - show just an icon in the first column
				// second is a name
				item.SubItems.Add(po.Caption);
				// third a class
				System.Xml.XmlNode config_node = po.GetConfigNode();
				string class_str = config_node.Attributes["class"] != null ? config_node.Attributes["class"].InnerText : String.Empty;
				item.SubItems.Add(class_str);
				// fourth is a description
				item.SubItems.Add(po.Description);

				viewList.Items.Add(item);
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.adaptersListView = new System.Windows.Forms.ListView();
			this.iconColumnHeader = new System.Windows.Forms.ColumnHeader();
			this.nameColumnHeader = new System.Windows.Forms.ColumnHeader();
			this.classColumnHeader = new System.Windows.Forms.ColumnHeader();
			this.descColumnHeader = new System.Windows.Forms.ColumnHeader();
			this.tableDescLabel = new System.Windows.Forms.Label();
			this.okButton = new System.Windows.Forms.Button();
			this.cancelButton = new System.Windows.Forms.Button();
			this.modulesListView = new System.Windows.Forms.ListView();
			this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader2 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader3 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader4 = new System.Windows.Forms.ColumnHeader();
			this.applicationsListView = new System.Windows.Forms.ListView();
			this.columnHeader5 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader6 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader7 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader8 = new System.Windows.Forms.ColumnHeader();
			this.SuspendLayout();
			// 
			// adaptersListView
			// 
			this.adaptersListView.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.adaptersListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																																											 this.iconColumnHeader,
																																											 this.nameColumnHeader,
																																											 this.classColumnHeader,
																																											 this.descColumnHeader});
			this.adaptersListView.FullRowSelect = true;
			this.adaptersListView.GridLines = true;
			this.adaptersListView.Location = new System.Drawing.Point(8, 24);
			this.adaptersListView.Name = "adaptersListView";
			this.adaptersListView.Size = new System.Drawing.Size(448, 208);
			this.adaptersListView.TabIndex = 0;
			this.adaptersListView.View = System.Windows.Forms.View.Details;
			this.adaptersListView.ItemActivate += new System.EventHandler(this.adaptersListView_ItemActivate);
			// 
			// iconColumnHeader
			// 
			this.iconColumnHeader.Text = "";
			this.iconColumnHeader.Width = 20;
			// 
			// nameColumnHeader
			// 
			this.nameColumnHeader.Text = "Name";
			this.nameColumnHeader.Width = 150;
			// 
			// classColumnHeader
			// 
			this.classColumnHeader.Text = "Class";
			this.classColumnHeader.Width = 150;
			// 
			// descColumnHeader
			// 
			this.descColumnHeader.Text = "Description";
			this.descColumnHeader.Width = 120;
			// 
			// tableDescLabel
			// 
			this.tableDescLabel.Location = new System.Drawing.Point(8, 8);
			this.tableDescLabel.Name = "tableDescLabel";
			this.tableDescLabel.Size = new System.Drawing.Size(184, 16);
			this.tableDescLabel.TabIndex = 0;
			this.tableDescLabel.Text = "Available adapters:";
			// 
			// okButton
			// 
			this.okButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.okButton.Location = new System.Drawing.Point(296, 240);
			this.okButton.Name = "okButton";
			this.okButton.TabIndex = 2;
			this.okButton.Text = "OK";
			this.okButton.Click += new System.EventHandler(this.okButton_Click);
			// 
			// cancelButton
			// 
			this.cancelButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.cancelButton.Location = new System.Drawing.Point(376, 240);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.cancelButton.TabIndex = 3;
			this.cancelButton.Text = "Cancel";
			// 
			// modulesListView
			// 
			this.modulesListView.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.modulesListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																																											this.columnHeader1,
																																											this.columnHeader2,
																																											this.columnHeader3,
																																											this.columnHeader4});
			this.modulesListView.FullRowSelect = true;
			this.modulesListView.GridLines = true;
			this.modulesListView.Location = new System.Drawing.Point(8, 24);
			this.modulesListView.Name = "modulesListView";
			this.modulesListView.Size = new System.Drawing.Size(448, 208);
			this.modulesListView.TabIndex = 1;
			this.modulesListView.View = System.Windows.Forms.View.Details;
			this.modulesListView.ItemActivate += new System.EventHandler(this.modulesListView_ItemActivate);
			// 
			// columnHeader1
			// 
			this.columnHeader1.Text = "";
			this.columnHeader1.Width = 20;
			// 
			// columnHeader2
			// 
			this.columnHeader2.Text = "Name";
			this.columnHeader2.Width = 150;
			// 
			// columnHeader3
			// 
			this.columnHeader3.Text = "Class";
			this.columnHeader3.Width = 150;
			// 
			// columnHeader4
			// 
			this.columnHeader4.Text = "Description";
			this.columnHeader4.Width = 120;
			// 
			// applicationsListView
			// 
			this.applicationsListView.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.applicationsListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																																													 this.columnHeader5,
																																													 this.columnHeader6,
																																													 this.columnHeader7,
																																													 this.columnHeader8});
			this.applicationsListView.FullRowSelect = true;
			this.applicationsListView.GridLines = true;
			this.applicationsListView.Location = new System.Drawing.Point(8, 24);
			this.applicationsListView.Name = "applicationsListView";
			this.applicationsListView.Size = new System.Drawing.Size(448, 208);
			this.applicationsListView.TabIndex = 0;
			this.applicationsListView.View = System.Windows.Forms.View.Details;
			this.applicationsListView.ItemActivate += new System.EventHandler(this.applicationsListView_ItemActivate);
			// 
			// columnHeader5
			// 
			this.columnHeader5.Text = "";
			this.columnHeader5.Width = 20;
			// 
			// columnHeader6
			// 
			this.columnHeader6.Text = "Name";
			this.columnHeader6.Width = 150;
			// 
			// columnHeader7
			// 
			this.columnHeader7.Text = "Class";
			this.columnHeader7.Width = 150;
			// 
			// columnHeader8
			// 
			this.columnHeader8.Text = "Description";
			this.columnHeader8.Width = 120;
			// 
			// NodeItemsListForm
			// 
			this.AcceptButton = this.okButton;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.cancelButton;
			this.ClientSize = new System.Drawing.Size(464, 269);
			this.ControlBox = false;
			this.Controls.Add(this.cancelButton);
			this.Controls.Add(this.okButton);
			this.Controls.Add(this.tableDescLabel);
			this.Controls.Add(this.modulesListView);
			this.Controls.Add(this.applicationsListView);
			this.Controls.Add(this.adaptersListView);
			this.Name = "NodeItemsListForm";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Items";
			this.Load += new System.EventHandler(this.NodeItemsListForm_Load);
			this.Closed += new System.EventHandler(this.NodeItemsListForm_Closed);
			this.ResumeLayout(false);

		}
		#endregion

		private void NodeItemsListForm_Load(object sender, System.EventArgs e)
		{
			if (this.currentNodeConfig != null)
			{
				okButton.Visible = true;
				cancelButton.Text = "Cancel";
			}
			else
			{
				okButton.Visible = false;
				cancelButton.Text = "Close";
			}
		}

		private void NodeItemsListForm_Closed(object sender, System.EventArgs e)
		{
			// remove current display - next time the form is displayed it has to be set again.
			this.CurrentNodeConfig = null;
		}

		private void okButton_Click(object sender, System.EventArgs e)
		{
			if (currentListView.SelectedItems.Count == 0)
				return;

			PredefinedObject predef_obj = (PredefinedObject)currentListView.SelectedItems[0].Tag;

			// this can occur also if the button is invisible (through item activation),
			// in that case the event should be ignored
			if (okButton.Visible)
			{
				MainForm.Instance.projectData.AddNodeItem(currentNodeConfig, predef_obj);
				this.Close();
			}
		}

		private void applicationsListView_ItemActivate(object sender, System.EventArgs e)
		{
			okButton_Click(sender, e);
		}

		private void adaptersListView_ItemActivate(object sender, System.EventArgs e)
		{
			okButton_Click(sender, e);
		}

		private void modulesListView_ItemActivate(object sender, System.EventArgs e)
		{
			okButton_Click(sender, e);
		}
	}
}
