/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using BasicObjectKind = NetSim.Core.BasicObjectKind;
using NodeItemKind = NetSim.Core.NodeItemKind;

namespace NetSim.Gui
{
	/// <summary>
	/// Summary description for NodeProperties.
	/// </summary>
	public class NodeProperties : System.Windows.Forms.Form
	{
		private System.Windows.Forms.ListBox adaptersListBox;
		private System.Windows.Forms.Button newAdapterButton;
		private System.Windows.Forms.Label adapterConnectedToLabel;
		private System.Windows.Forms.Button removeAdapterButton;
		private System.Windows.Forms.ListBox modulesListBox;
		private System.Windows.Forms.Button newModuleButton;
		private System.Windows.Forms.Button removeModuleButton;
		private System.Windows.Forms.Button moduleMoveUpButton;
		private System.Windows.Forms.Button moduleMoveDownButton;
		private System.Windows.Forms.Button configureAdapterButton;
		private System.Windows.Forms.Button configureModuleButton;
		private System.Windows.Forms.ListBox applicationsListBox;
		private System.Windows.Forms.Button newApplicationButton;
		private System.Windows.Forms.Button removeApplicationButton;
		private System.Windows.Forms.Button OKButton;
		private System.Windows.Forms.Button configureApplicationButton;
		private System.Windows.Forms.ComboBox linksComboBox;
		private System.Windows.Forms.TabControl tabControl1;
		private System.Windows.Forms.TabPage adaptersTabPage;
		private System.Windows.Forms.TabPage modulesTabPage;
		private System.Windows.Forms.TabPage applicationsTabPage;
		private System.Windows.Forms.Button renameModuleButton;
		private System.Windows.Forms.Button renameAdapterButton;
		private System.Windows.Forms.Button renameApplicationButton;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public NodeProperties()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

		}

		private NodeConfig nodeConfig;
		public void SetElement(DisplayElement de)
		{
			// init all fields
			this.Text = de.Config.Name + " - " + this.Text;

			nodeConfig = de.Config as NodeConfig;
			RefreshLists();
		}

		private void RefreshLists()
		{
			adaptersListBox.Items.Clear();
			modulesListBox.Items.Clear();
			applicationsListBox.Items.Clear();

			foreach (AdapterConfig c in nodeConfig.GetItems(NodeItemKind.Adapter))
				adaptersListBox.Items.Add(c);
			foreach (ModuleConfig c in nodeConfig.GetItems(NodeItemKind.Module))
				modulesListBox.Items.Add(c);
			foreach (ApplicationConfig c in nodeConfig.GetItems(NodeItemKind.Application))
				applicationsListBox.Items.Add(c);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.configureAdapterButton = new System.Windows.Forms.Button();
			this.removeAdapterButton = new System.Windows.Forms.Button();
			this.adapterConnectedToLabel = new System.Windows.Forms.Label();
			this.linksComboBox = new System.Windows.Forms.ComboBox();
			this.newAdapterButton = new System.Windows.Forms.Button();
			this.adaptersListBox = new System.Windows.Forms.ListBox();
			this.configureModuleButton = new System.Windows.Forms.Button();
			this.moduleMoveDownButton = new System.Windows.Forms.Button();
			this.moduleMoveUpButton = new System.Windows.Forms.Button();
			this.removeModuleButton = new System.Windows.Forms.Button();
			this.newModuleButton = new System.Windows.Forms.Button();
			this.modulesListBox = new System.Windows.Forms.ListBox();
			this.configureApplicationButton = new System.Windows.Forms.Button();
			this.removeApplicationButton = new System.Windows.Forms.Button();
			this.newApplicationButton = new System.Windows.Forms.Button();
			this.applicationsListBox = new System.Windows.Forms.ListBox();
			this.OKButton = new System.Windows.Forms.Button();
			this.tabControl1 = new System.Windows.Forms.TabControl();
			this.adaptersTabPage = new System.Windows.Forms.TabPage();
			this.renameAdapterButton = new System.Windows.Forms.Button();
			this.modulesTabPage = new System.Windows.Forms.TabPage();
			this.renameModuleButton = new System.Windows.Forms.Button();
			this.applicationsTabPage = new System.Windows.Forms.TabPage();
			this.renameApplicationButton = new System.Windows.Forms.Button();
			this.tabControl1.SuspendLayout();
			this.adaptersTabPage.SuspendLayout();
			this.modulesTabPage.SuspendLayout();
			this.applicationsTabPage.SuspendLayout();
			this.SuspendLayout();
			// 
			// configureAdapterButton
			// 
			this.configureAdapterButton.Location = new System.Drawing.Point(416, 48);
			this.configureAdapterButton.Name = "configureAdapterButton";
			this.configureAdapterButton.TabIndex = 6;
			this.configureAdapterButton.Text = "Configure";
			this.configureAdapterButton.Click += new System.EventHandler(this.configureAdapterButton_Click);
			// 
			// removeAdapterButton
			// 
			this.removeAdapterButton.Enabled = false;
			this.removeAdapterButton.Location = new System.Drawing.Point(88, 168);
			this.removeAdapterButton.Name = "removeAdapterButton";
			this.removeAdapterButton.TabIndex = 2;
			this.removeAdapterButton.Text = "Remove";
			this.removeAdapterButton.Click += new System.EventHandler(this.removeAdapterButton_Click);
			// 
			// adapterConnectedToLabel
			// 
			this.adapterConnectedToLabel.Location = new System.Drawing.Point(248, 152);
			this.adapterConnectedToLabel.Name = "adapterConnectedToLabel";
			this.adapterConnectedToLabel.Size = new System.Drawing.Size(100, 16);
			this.adapterConnectedToLabel.TabIndex = 3;
			this.adapterConnectedToLabel.Text = "Connected To:";
			// 
			// linksComboBox
			// 
			this.linksComboBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.linksComboBox.Enabled = false;
			this.linksComboBox.Location = new System.Drawing.Point(248, 168);
			this.linksComboBox.Name = "linksComboBox";
			this.linksComboBox.Size = new System.Drawing.Size(248, 21);
			this.linksComboBox.TabIndex = 4;
			this.linksComboBox.SelectedIndexChanged += new System.EventHandler(this.linksComboBox_SelectedIndexChanged);
			// 
			// newAdapterButton
			// 
			this.newAdapterButton.Location = new System.Drawing.Point(8, 168);
			this.newAdapterButton.Name = "newAdapterButton";
			this.newAdapterButton.TabIndex = 1;
			this.newAdapterButton.Text = "New";
			this.newAdapterButton.Click += new System.EventHandler(this.newAdapterButton_Click);
			// 
			// adaptersListBox
			// 
			this.adaptersListBox.Location = new System.Drawing.Point(8, 16);
			this.adaptersListBox.Name = "adaptersListBox";
			this.adaptersListBox.Size = new System.Drawing.Size(400, 134);
			this.adaptersListBox.TabIndex = 0;
			this.adaptersListBox.DoubleClick += new System.EventHandler(this.adaptersListBox_DoubleClick);
			this.adaptersListBox.SelectedIndexChanged += new System.EventHandler(this.adaptersListBox_SelectedIndexChanged);
			// 
			// configureModuleButton
			// 
			this.configureModuleButton.Location = new System.Drawing.Point(416, 48);
			this.configureModuleButton.Name = "configureModuleButton";
			this.configureModuleButton.TabIndex = 4;
			this.configureModuleButton.Text = "Configure";
			this.configureModuleButton.Click += new System.EventHandler(this.configureModuleButton_Click);
			// 
			// moduleMoveDownButton
			// 
			this.moduleMoveDownButton.Location = new System.Drawing.Point(416, 144);
			this.moduleMoveDownButton.Name = "moduleMoveDownButton";
			this.moduleMoveDownButton.Size = new System.Drawing.Size(75, 20);
			this.moduleMoveDownButton.TabIndex = 6;
			this.moduleMoveDownButton.Text = "Move Down";
			this.moduleMoveDownButton.Click += new System.EventHandler(this.moduleMoveDownButton_Click);
			// 
			// moduleMoveUpButton
			// 
			this.moduleMoveUpButton.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.moduleMoveUpButton.Location = new System.Drawing.Point(416, 120);
			this.moduleMoveUpButton.Name = "moduleMoveUpButton";
			this.moduleMoveUpButton.Size = new System.Drawing.Size(75, 20);
			this.moduleMoveUpButton.TabIndex = 5;
			this.moduleMoveUpButton.Text = "Move Up";
			this.moduleMoveUpButton.Click += new System.EventHandler(this.moduleMoveUpButton_Click);
			// 
			// removeModuleButton
			// 
			this.removeModuleButton.Enabled = false;
			this.removeModuleButton.Location = new System.Drawing.Point(88, 168);
			this.removeModuleButton.Name = "removeModuleButton";
			this.removeModuleButton.TabIndex = 2;
			this.removeModuleButton.Text = "Remove";
			this.removeModuleButton.Click += new System.EventHandler(this.removeModuleButton_Click);
			// 
			// newModuleButton
			// 
			this.newModuleButton.Location = new System.Drawing.Point(8, 168);
			this.newModuleButton.Name = "newModuleButton";
			this.newModuleButton.TabIndex = 1;
			this.newModuleButton.Text = "New";
			this.newModuleButton.Click += new System.EventHandler(this.newModuleButton_Click);
			// 
			// modulesListBox
			// 
			this.modulesListBox.Location = new System.Drawing.Point(8, 16);
			this.modulesListBox.Name = "modulesListBox";
			this.modulesListBox.Size = new System.Drawing.Size(400, 147);
			this.modulesListBox.TabIndex = 0;
			this.modulesListBox.DoubleClick += new System.EventHandler(this.modulesListBox_DoubleClick);
			this.modulesListBox.SelectedIndexChanged += new System.EventHandler(this.modulesListBox_SelectedIndexChanged);
			// 
			// configureApplicationButton
			// 
			this.configureApplicationButton.Location = new System.Drawing.Point(416, 48);
			this.configureApplicationButton.Name = "configureApplicationButton";
			this.configureApplicationButton.TabIndex = 4;
			this.configureApplicationButton.Text = "Configure";
			this.configureApplicationButton.Click += new System.EventHandler(this.configureApplicationButton_Click);
			// 
			// removeApplicationButton
			// 
			this.removeApplicationButton.Enabled = false;
			this.removeApplicationButton.Location = new System.Drawing.Point(88, 168);
			this.removeApplicationButton.Name = "removeApplicationButton";
			this.removeApplicationButton.TabIndex = 2;
			this.removeApplicationButton.Text = "Remove";
			this.removeApplicationButton.Click += new System.EventHandler(this.removeApplicationButton_Click);
			// 
			// newApplicationButton
			// 
			this.newApplicationButton.Location = new System.Drawing.Point(8, 168);
			this.newApplicationButton.Name = "newApplicationButton";
			this.newApplicationButton.TabIndex = 1;
			this.newApplicationButton.Text = "New";
			this.newApplicationButton.Click += new System.EventHandler(this.newApplicationButton_Click);
			// 
			// applicationsListBox
			// 
			this.applicationsListBox.Location = new System.Drawing.Point(8, 16);
			this.applicationsListBox.Name = "applicationsListBox";
			this.applicationsListBox.Size = new System.Drawing.Size(400, 147);
			this.applicationsListBox.TabIndex = 0;
			this.applicationsListBox.DoubleClick += new System.EventHandler(this.applicationsListBox_DoubleClick);
			this.applicationsListBox.SelectedIndexChanged += new System.EventHandler(this.applicationsListBox_SelectedIndexChanged);
			// 
			// OKButton
			// 
			this.OKButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.OKButton.Location = new System.Drawing.Point(216, 240);
			this.OKButton.Name = "OKButton";
			this.OKButton.TabIndex = 1;
			this.OKButton.Text = "OK";
			this.OKButton.Click += new System.EventHandler(this.OKButton_Click);
			// 
			// tabControl1
			// 
			this.tabControl1.Controls.Add(this.adaptersTabPage);
			this.tabControl1.Controls.Add(this.modulesTabPage);
			this.tabControl1.Controls.Add(this.applicationsTabPage);
			this.tabControl1.Dock = System.Windows.Forms.DockStyle.Top;
			this.tabControl1.Location = new System.Drawing.Point(0, 0);
			this.tabControl1.Name = "tabControl1";
			this.tabControl1.SelectedIndex = 0;
			this.tabControl1.Size = new System.Drawing.Size(506, 224);
			this.tabControl1.TabIndex = 0;
			// 
			// adaptersTabPage
			// 
			this.adaptersTabPage.Controls.Add(this.renameAdapterButton);
			this.adaptersTabPage.Controls.Add(this.adapterConnectedToLabel);
			this.adaptersTabPage.Controls.Add(this.linksComboBox);
			this.adaptersTabPage.Controls.Add(this.configureAdapterButton);
			this.adaptersTabPage.Controls.Add(this.newAdapterButton);
			this.adaptersTabPage.Controls.Add(this.adaptersListBox);
			this.adaptersTabPage.Controls.Add(this.removeAdapterButton);
			this.adaptersTabPage.Location = new System.Drawing.Point(4, 22);
			this.adaptersTabPage.Name = "adaptersTabPage";
			this.adaptersTabPage.Size = new System.Drawing.Size(498, 198);
			this.adaptersTabPage.TabIndex = 0;
			this.adaptersTabPage.Text = "Adapters";
			// 
			// renameAdapterButton
			// 
			this.renameAdapterButton.Location = new System.Drawing.Point(416, 16);
			this.renameAdapterButton.Name = "renameAdapterButton";
			this.renameAdapterButton.TabIndex = 5;
			this.renameAdapterButton.Text = "Rename";
			this.renameAdapterButton.Click += new System.EventHandler(this.renameAdapterButton_Click);
			// 
			// modulesTabPage
			// 
			this.modulesTabPage.Controls.Add(this.renameModuleButton);
			this.modulesTabPage.Controls.Add(this.configureModuleButton);
			this.modulesTabPage.Controls.Add(this.moduleMoveDownButton);
			this.modulesTabPage.Controls.Add(this.moduleMoveUpButton);
			this.modulesTabPage.Controls.Add(this.removeModuleButton);
			this.modulesTabPage.Controls.Add(this.newModuleButton);
			this.modulesTabPage.Controls.Add(this.modulesListBox);
			this.modulesTabPage.Location = new System.Drawing.Point(4, 22);
			this.modulesTabPage.Name = "modulesTabPage";
			this.modulesTabPage.Size = new System.Drawing.Size(498, 198);
			this.modulesTabPage.TabIndex = 1;
			this.modulesTabPage.Text = "Modules";
			// 
			// renameModuleButton
			// 
			this.renameModuleButton.Location = new System.Drawing.Point(416, 16);
			this.renameModuleButton.Name = "renameModuleButton";
			this.renameModuleButton.TabIndex = 3;
			this.renameModuleButton.Text = "Rename";
			this.renameModuleButton.Click += new System.EventHandler(this.renameModuleButton_Click);
			// 
			// applicationsTabPage
			// 
			this.applicationsTabPage.Controls.Add(this.renameApplicationButton);
			this.applicationsTabPage.Controls.Add(this.applicationsListBox);
			this.applicationsTabPage.Controls.Add(this.configureApplicationButton);
			this.applicationsTabPage.Controls.Add(this.removeApplicationButton);
			this.applicationsTabPage.Controls.Add(this.newApplicationButton);
			this.applicationsTabPage.Location = new System.Drawing.Point(4, 22);
			this.applicationsTabPage.Name = "applicationsTabPage";
			this.applicationsTabPage.Size = new System.Drawing.Size(498, 198);
			this.applicationsTabPage.TabIndex = 2;
			this.applicationsTabPage.Text = "Applications";
			// 
			// renameApplicationButton
			// 
			this.renameApplicationButton.Location = new System.Drawing.Point(416, 16);
			this.renameApplicationButton.Name = "renameApplicationButton";
			this.renameApplicationButton.TabIndex = 3;
			this.renameApplicationButton.Text = "Rename";
			this.renameApplicationButton.Click += new System.EventHandler(this.renameApplicationButton_Click);
			// 
			// NodeProperties
			// 
			this.AcceptButton = this.OKButton;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.OKButton;
			this.ClientSize = new System.Drawing.Size(506, 271);
			this.ControlBox = false;
			this.Controls.Add(this.tabControl1);
			this.Controls.Add(this.OKButton);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "NodeProperties";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Properties";
			this.Load += new System.EventHandler(this.NodeProperties_Load);
			this.tabControl1.ResumeLayout(false);
			this.adaptersTabPage.ResumeLayout(false);
			this.modulesTabPage.ResumeLayout(false);
			this.applicationsTabPage.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void OKButton_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void adaptersListBox_DoubleClick(object sender, System.EventArgs e)
		{
			configureAdapterButton_Click(sender, e);
		}

		private void configureAdapterButton_Click(object sender, System.EventArgs e)
		{
			AdapterConfig ac = adaptersListBox.SelectedItem as AdapterConfig;
			if (ac == null)
				return;

			try { ac.EditConfigFile(); }
			catch (Exception exc)
			{
				MessageBox.Show(this, exc.Message, "Cannot edit configuration file", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void modulesListBox_DoubleClick(object sender, System.EventArgs e)
		{
			configureModuleButton_Click(sender, e);
		}

		private void configureModuleButton_Click(object sender, System.EventArgs e)
		{
			ModuleConfig mc = modulesListBox.SelectedItem as ModuleConfig;
			if (mc == null)
				return;

			try {	mc.EditConfigFile(); }
			catch (Exception exc)
			{
				MessageBox.Show(this, exc.Message, "Cannot edit configuration file", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void applicationsListBox_DoubleClick(object sender, System.EventArgs e)
		{
			configureApplicationButton_Click(sender, e);
		}

		private void configureApplicationButton_Click(object sender, System.EventArgs e)
		{
			ApplicationConfig ac = applicationsListBox.SelectedItem as ApplicationConfig;
			if (ac == null)
				return;

			try { ac.EditConfigFile(); }
			catch (Exception exc)
			{
				MessageBox.Show(this, exc.Message, "Cannot edit configuration file", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void adaptersListBox_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			AdapterConfig nic = adaptersListBox.SelectedItem as AdapterConfig;
			if (nic != null)
			{
				this.configureAdapterButton.Enabled = nic.HasConfigFile();
				this.removeAdapterButton.Enabled = true;

				linksComboBox.Enabled = true;
				modulesListBox.SelectedIndex = -1;
				applicationsListBox.SelectedIndex = -1;

				// set appropriate link in combo box
				linksComboBox.SelectedIndex = linksComboBox.FindStringExact(nic.Link);
			}
			else
			{
				this.configureAdapterButton.Enabled = false;
				this.removeAdapterButton.Enabled = false;

				linksComboBox.SelectedIndex = -1; // if adapter not selected, link not selected
				linksComboBox.Enabled = false;
			}
		}

		private void modulesListBox_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			NodeItemConfig nic = modulesListBox.SelectedItem as NodeItemConfig;
			if (nic != null)
			{
				this.configureModuleButton.Enabled = nic.HasConfigFile();
				this.removeModuleButton.Enabled = true;

				adaptersListBox.SelectedIndex = -1;
				applicationsListBox.SelectedIndex = -1;
			}
			else
			{
				this.removeModuleButton.Enabled = false;
			}
		}

		private void applicationsListBox_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			NodeItemConfig nic = applicationsListBox.SelectedItem as NodeItemConfig;
			if (nic != null)
			{
				this.configureApplicationButton.Enabled = nic.HasConfigFile();
				this.removeApplicationButton.Enabled = true;

				adaptersListBox.SelectedIndex = -1;
				modulesListBox.SelectedIndex = -1;
			}
			else
			{
				this.removeApplicationButton.Enabled = false;
			}
		}

		private void NodeProperties_Load(object sender, System.EventArgs e)
		{
			// initialize fields
			foreach (DisplayElement de in MainForm.Instance.projectData.GetObjectList(BasicObjectKind.Link))
			{
				LinkConfig lc = de.Config as LinkConfig;
				if (lc != null)
					linksComboBox.Items.Add(lc);
			}
		}

		private void linksComboBox_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			LinkConfig lc = linksComboBox.SelectedItem as LinkConfig;
			AdapterConfig ac = adaptersListBox.SelectedItem as AdapterConfig;
			if (ac != null && lc != null)
			{
				// adapter selected, change link it connects to
				ac.SetLink(lc);
				
				// and redrawing of the connection might be needed
				MainForm.Instance.mainPanel.Invalidate();
			}
		}

		private void moduleMoveUpButton_Click(object sender, System.EventArgs e)
		{
			int index = modulesListBox.SelectedIndex;
			if (index > 0)
			{
				ModuleConfig mc1 = modulesListBox.Items[index] as ModuleConfig;
				ModuleConfig mc2 = modulesListBox.Items[index-1] as ModuleConfig;
				if (mc1 != null && mc2 != null)
				{
					this.nodeConfig.SwapItems(NodeItemKind.Module, index, index-1);

					modulesListBox.Items.RemoveAt(index);
					modulesListBox.Items.Insert(index-1, mc1);
				}

				// keep selection
				modulesListBox.SelectedIndex = index - 1;
			}
		}

		private void moduleMoveDownButton_Click(object sender, System.EventArgs e)
		{
			int index = modulesListBox.SelectedIndex;
			if (index < modulesListBox.Items.Count - 1)
			{
				ModuleConfig mc1 = modulesListBox.Items[index+1] as ModuleConfig;
				ModuleConfig mc2 = modulesListBox.Items[index] as ModuleConfig;
				if (mc1 != null && mc2 != null)
				{
					this.nodeConfig.SwapItems(NodeItemKind.Module, index, index+1);

					modulesListBox.Items.RemoveAt(index+1);
					modulesListBox.Items.Insert(index, mc1);
				}
			
				// keep selection
				modulesListBox.SelectedIndex = index + 1;
			}
		}

		private void removeAdapterButton_Click(object sender, System.EventArgs e)
		{
			AdapterConfig ac = adaptersListBox.SelectedItem as AdapterConfig;
			if (ac != null)
			{
				MainForm.Instance.projectData.RemoveNodeItem(nodeConfig, ac);
				adaptersListBox.Items.Remove(ac);
			}
		}

		private void removeModuleButton_Click(object sender, System.EventArgs e)
		{
			ModuleConfig mc = modulesListBox.SelectedItem as ModuleConfig;
			if (mc != null)
			{
				MainForm.Instance.projectData.RemoveNodeItem(nodeConfig, mc);
				modulesListBox.Items.Remove(mc);
			}
		}

		private void removeApplicationButton_Click(object sender, System.EventArgs e)
		{
			ApplicationConfig ac = applicationsListBox.SelectedItem as ApplicationConfig;
			if (ac != null)
			{
				MainForm.Instance.projectData.RemoveNodeItem(nodeConfig, ac);
				applicationsListBox.Items.Remove(ac);
			}
		}

		private void newAdapterButton_Click(object sender, System.EventArgs e)
		{
			MainForm.Instance.nodeItemsListForm.CurrentNodeConfig = this.nodeConfig;
			MainForm.Instance.nodeItemsListForm.CurrentNodeItem = NodeItemKind.Adapter;
			MainForm.Instance.nodeItemsListForm.ShowDialog(this);
		
			RefreshLists();
		}

		private void newModuleButton_Click(object sender, System.EventArgs e)
		{
			MainForm.Instance.nodeItemsListForm.CurrentNodeConfig = this.nodeConfig;
			MainForm.Instance.nodeItemsListForm.CurrentNodeItem = NodeItemKind.Module;
			MainForm.Instance.nodeItemsListForm.ShowDialog(this);

			RefreshLists();
		}

		private void newApplicationButton_Click(object sender, System.EventArgs e)
		{
			MainForm.Instance.nodeItemsListForm.CurrentNodeConfig = this.nodeConfig;
			MainForm.Instance.nodeItemsListForm.CurrentNodeItem = NodeItemKind.Application;
			MainForm.Instance.nodeItemsListForm.ShowDialog(this);

			RefreshLists();
		}

		private void renameAdapterButton_Click(object sender, System.EventArgs e)
		{
			AdapterConfig ac = adaptersListBox.SelectedItem as AdapterConfig;
			if (ac != null)
			{
				RenameItem rename_dlg = new RenameItem();
				rename_dlg.SetItem(ac);
				rename_dlg.ShowDialog(this);
				rename_dlg.Dispose();

				RefreshLists();
			}
		}

		private void renameModuleButton_Click(object sender, System.EventArgs e)
		{
			ModuleConfig mc = modulesListBox.SelectedItem as ModuleConfig;
			if (mc != null)
			{
				RenameItem rename_dlg = new RenameItem();
				rename_dlg.SetItem(mc);
				rename_dlg.ShowDialog(this);
				rename_dlg.Dispose();
			
				RefreshLists();
			}
		}

		private void renameApplicationButton_Click(object sender, System.EventArgs e)
		{
			ApplicationConfig ac = applicationsListBox.SelectedItem as ApplicationConfig;
			if (ac != null)
			{
				RenameItem rename_dlg = new RenameItem();
				rename_dlg.SetItem(ac);
				rename_dlg.ShowDialog(this);
				rename_dlg.Dispose();
			
				RefreshLists();
			}
		}
	}

	internal class NodeProperty
	{
		private XmlNode configNode;

		public NodeProperty(XmlNode configNode)
		{
			this.configNode = configNode;
		}

		public override string ToString()
		{
			XmlAttribute caption_attr = configNode.Attributes["name"];
			string caption = caption_attr != null ? caption_attr.InnerText : "<unnamed>";

			XmlAttribute class_attr = configNode.Attributes["class"];
			if (class_attr != null)
				caption += " (" + class_attr.InnerText + ")";

			return caption;
		}

	}
}
