/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

#include "netsimspi.h"
#include "assert.h"

NETSIMWSPPROC_TABLE gWrapperProcTable;
extern pfnNestingCount gNestingCount;

__declspec(align(32)) long gNestingCounter = 0;

int __cdecl InitProcTable(LPWSPPROC_TABLE lpProcTable)
{
	lpProcTable->lpWSPAccept              = NetSimWSPAccept;
	lpProcTable->lpWSPAddressToString     = NetSimWSPAddressToString;
	lpProcTable->lpWSPAsyncSelect         = NetSimWSPAsyncSelect;
	lpProcTable->lpWSPBind                = NetSimWSPBind;
	lpProcTable->lpWSPCancelBlockingCall  = NetSimWSPCancelBlockingCall;
	lpProcTable->lpWSPCleanup             = NetSimWSPCleanup;
	lpProcTable->lpWSPCloseSocket         = NetSimWSPCloseSocket;
	lpProcTable->lpWSPConnect             = NetSimWSPConnect;
	lpProcTable->lpWSPDuplicateSocket     = NetSimWSPDuplicateSocket;
	lpProcTable->lpWSPEnumNetworkEvents   = NetSimWSPEnumNetworkEvents;
	lpProcTable->lpWSPEventSelect         = NetSimWSPEventSelect;
	lpProcTable->lpWSPGetOverlappedResult = NetSimWSPGetOverlappedResult;
	lpProcTable->lpWSPGetPeerName         = NetSimWSPGetPeerName;
	lpProcTable->lpWSPGetSockName         = NetSimWSPGetSockName;
	lpProcTable->lpWSPGetSockOpt          = NetSimWSPGetSockOpt;
	lpProcTable->lpWSPGetQOSByName        = NetSimWSPGetQOSByName;
	lpProcTable->lpWSPIoctl               = NetSimWSPIoctl;
	lpProcTable->lpWSPJoinLeaf            = NetSimWSPJoinLeaf;
	lpProcTable->lpWSPListen              = NetSimWSPListen;
	lpProcTable->lpWSPRecv                = NetSimWSPRecv;
	lpProcTable->lpWSPRecvDisconnect      = NetSimWSPRecvDisconnect;
	lpProcTable->lpWSPRecvFrom            = NetSimWSPRecvFrom;
	lpProcTable->lpWSPSelect              = NetSimWSPSelect;
	lpProcTable->lpWSPSend                = NetSimWSPSend;
	lpProcTable->lpWSPSendDisconnect      = NetSimWSPSendDisconnect;
	lpProcTable->lpWSPSendTo              = NetSimWSPSendTo;
	lpProcTable->lpWSPSetSockOpt          = NetSimWSPSetSockOpt;
	lpProcTable->lpWSPShutdown            = NetSimWSPShutdown;
	lpProcTable->lpWSPSocket              = NetSimWSPSocket;
	lpProcTable->lpWSPStringToAddress     = NetSimWSPStringToAddress;

	return NO_ERROR;
}

SOCKET gDebugMaxSockValue = 50;
void ErrorSockValue(int value)
{
	volatile int i = 0;
	return;
}

SOCKET WSPAPI NetSimWSPAccept(
	SOCKET          s,                      
	struct sockaddr FAR * addr,  
	LPINT           addrlen,                 
	LPCONDITIONPROC lpfnCondition,  
	DWORD_PTR       dwCallbackData,          
	LPINT           lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	SOCKET result = gWrapperProcTable.lpWSPAccept(s, addr, addrlen, lpfnCondition, dwCallbackData,
		lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPAddressToString(
	LPSOCKADDR          lpsaAddress,            
	DWORD               dwAddressLength,               
	LPWSAPROTOCOL_INFOW lpProtocolInfo,   
	LPWSTR              lpszAddressString,            
	LPDWORD             lpdwAddressStringLength,   
	LPINT               lpErrno
)
{
	REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPAddressToString(lpsaAddress, dwAddressLength, lpProtocolInfo,
		lpszAddressString, lpdwAddressStringLength, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPAsyncSelect(
	SOCKET       s,
	HWND         hWnd,
	unsigned int wMsg,
	long         lEvent,
	LPINT        lpErrno
	)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPAsyncSelect(s, hWnd, wMsg, lEvent, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPBind(
	SOCKET                s,
	const struct sockaddr FAR * name,
	int                   namelen,
	LPINT                 lpErrno
	)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPBind(s, name, namelen, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPCancelBlockingCall(
	LPINT lpErrno
	)
{
	REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPCancelBlockingCall(lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPCleanup(
	LPINT lpErrno  
	)
{
	REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPCleanup(lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPCloseSocket(  
	SOCKET s,        
	LPINT  lpErrno
	)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPCloseSocket(s, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPConnect(
	SOCKET                s,
	const struct sockaddr FAR * name,
	int                   namelen,
	LPWSABUF              lpCallerData,
	LPWSABUF              lpCalleeData,
	LPQOS                 lpSQOS,
	LPQOS                 lpGQOS,
	LPINT                 lpErrno
	)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPConnect(s, name, namelen, lpCallerData, lpCalleeData,
		lpSQOS, lpGQOS, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPDuplicateSocket(
	SOCKET              s,
	DWORD               dwProcessId,                      
	LPWSAPROTOCOL_INFOW lpProtocolInfo,   
	LPINT               lpErrno
	)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPDuplicateSocket(s, dwProcessId, lpProtocolInfo, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPEnumNetworkEvents(  
	SOCKET             s,
	WSAEVENT           hEventObject,
	LPWSANETWORKEVENTS lpNetworkEvents,
	LPINT              lpErrno
	)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPEnumNetworkEvents(s, hEventObject, lpNetworkEvents, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPEventSelect(
	SOCKET   s,
	WSAEVENT hEventObject,
	long     lNetworkEvents,
	LPINT    lpErrno
	)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPEventSelect(s, hEventObject, lNetworkEvents, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

BOOL WSPAPI NetSimWSPGetOverlappedResult(
	SOCKET          s,
	LPWSAOVERLAPPED lpOverlapped,
	LPDWORD         lpcbTransfer,
	BOOL            fWait,
	LPDWORD         lpdwFlags,
	LPINT           lpErrno
	)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	BOOL result = gWrapperProcTable.lpWSPGetOverlappedResult(s, lpOverlapped, lpcbTransfer,
		fWait, lpdwFlags, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPGetPeerName(  
	SOCKET          s,
	struct sockaddr FAR * name,
	LPINT           namelen,
	LPINT           lpErrno
	)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPGetPeerName(s, name, namelen, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPGetSockName(
	SOCKET          s,
	struct sockaddr FAR * name,
	LPINT           namelen,
	LPINT           lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPGetSockName(s, name, namelen, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPGetSockOpt(
	SOCKET     s,
	int        level,
	int        optname,
	char FAR * optval,
	LPINT      optlen,
	LPINT      lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPGetSockOpt(s, level, optname, optval, optlen, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

BOOL WSPAPI NetSimWSPGetQOSByName(
	SOCKET   s,
	LPWSABUF lpQOSName,
	LPQOS    lpQOS,
	LPINT    lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	BOOL result = gWrapperProcTable.lpWSPGetQOSByName(s, lpQOSName, lpQOS, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPIoctl(
	SOCKET          s,
	DWORD           dwIoControlCode,
	LPVOID          lpvInBuffer,
	DWORD           cbInBuffer,
	LPVOID          lpvOutBuffer,
	DWORD           cbOutBuffer,
	LPDWORD         lpcbBytesReturned,
	LPWSAOVERLAPPED lpOverlapped,
	LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
	LPWSATHREADID   lpThreadId,
	LPINT           lpErrno
)
{
	// if overlapped requested, it must be via completion routine
	assert(lpCompletionRoutine != NULL || lpOverlapped == NULL);

	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPIoctl(s, dwIoControlCode, lpvInBuffer, cbInBuffer, lpvOutBuffer,
		cbOutBuffer, lpcbBytesReturned, lpOverlapped, lpCompletionRoutine, lpThreadId, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

SOCKET WSPAPI NetSimWSPJoinLeaf(
	SOCKET       s,
	const struct sockaddr FAR * name,
	int          namelen,
	LPWSABUF     lpCallerData,
	LPWSABUF     lpCalleeData,
	LPQOS        lpSQOS,
	LPQOS        lpGQOS,
	DWORD        dwFlags,
	LPINT        lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	SOCKET result = gWrapperProcTable.lpWSPJoinLeaf(s, name, namelen, lpCallerData, lpCalleeData,
		lpSQOS, lpGQOS, dwFlags, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPListen(
	SOCKET s,        
	int    backlog,     
	LPINT  lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPListen(s, backlog, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPRecv(
	SOCKET          s,
	LPWSABUF        lpBuffers,
	DWORD           dwBufferCount,
	LPDWORD         lpNumberOfBytesRecvd,
	LPDWORD         lpFlags,
	LPWSAOVERLAPPED lpOverlapped,
	LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
	LPWSATHREADID   lpThreadId,
	LPINT           lpErrno
)
{
	// if overlapped requested, it must be via completion routine
	assert(lpCompletionRoutine != NULL || lpOverlapped == NULL);

	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPRecv(s, lpBuffers, dwBufferCount, lpNumberOfBytesRecvd, lpFlags,
		lpOverlapped, lpCompletionRoutine, lpThreadId, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPRecvDisconnect(
	SOCKET   s,
	LPWSABUF lpInboundDisconnectData,
	LPINT    lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPRecvDisconnect(s, lpInboundDisconnectData, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPRecvFrom(
	SOCKET          s,
	LPWSABUF        lpBuffers,
	DWORD           dwBufferCount,
	LPDWORD         lpNumberOfBytesRecvd,
	LPDWORD         lpFlags,
	struct sockaddr FAR * lpFrom,
	LPINT           lpFromLen,
	LPWSAOVERLAPPED lpOverlapped,
	LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
	LPWSATHREADID   lpThreadId,
	LPINT           lpErrno
)
{
	// if overlapped requested, it must be via completion routine
	assert(lpCompletionRoutine != NULL || lpOverlapped == NULL);

	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPRecvFrom(s, lpBuffers, dwBufferCount, lpNumberOfBytesRecvd,
		lpFlags, lpFrom, lpFromLen, lpOverlapped, lpCompletionRoutine, lpThreadId, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPSelect(
	int          nfds,
	fd_set FAR * readfds,
	fd_set FAR * writefds,
	fd_set FAR * exceptfds,
	const struct timeval FAR * timeout,
	LPINT        lpErrno
)
{
	REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPSelect(nfds, readfds, writefds, exceptfds, timeout, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPSend(
	SOCKET          s,
	LPWSABUF        lpBuffers,
	DWORD           dwBufferCount,
	LPDWORD         lpNumberOfBytesSent,
	DWORD           dwFlags,
	LPWSAOVERLAPPED lpOverlapped, // ignored, since we do not support overlapped transfers
	LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,   
	LPWSATHREADID   lpThreadId,                                 
	LPINT           lpErrno                                             
)
{
	// if overlapped requested, it must be via completion routine
	assert(lpCompletionRoutine != NULL || lpOverlapped == NULL);

	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPSend(s, lpBuffers, dwBufferCount, lpNumberOfBytesSent, dwFlags,
		lpOverlapped, lpCompletionRoutine, lpThreadId, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPSendDisconnect(
	SOCKET   s,
	LPWSABUF lpOutboundDisconnectData,
	LPINT    lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPSendDisconnect(s, lpOutboundDisconnectData, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPSendTo(
	SOCKET          s,
	LPWSABUF        lpBuffers,
	DWORD           dwBufferCount,
	LPDWORD         lpNumberOfBytesSent,
	DWORD           dwFlags,
	const struct sockaddr FAR * lpTo,
	int             iToLen,
	LPWSAOVERLAPPED lpOverlapped,
	LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
	LPWSATHREADID   lpThreadId,
	LPINT           lpErrno
)
{
	// if overlapped requested, it must be via completion routine
	assert(lpCompletionRoutine != NULL || lpOverlapped == NULL);

	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPSendTo(s, lpBuffers, dwBufferCount, lpNumberOfBytesSent, dwFlags,
		lpTo, iToLen, lpOverlapped, lpCompletionRoutine, lpThreadId, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPSetSockOpt(
	SOCKET     s,
	int        level,
	int        optname,
	const char FAR * optval,   
	int        optlen,
	LPINT      lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPSetSockOpt(s, level, optname, optval, optlen, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPShutdown(
	SOCKET s,
	int    how,
	LPINT  lpErrno
)
{
	if (s > gDebugMaxSockValue) ErrorSockValue(s); REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPShutdown(s, how, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}

SOCKET WSPAPI NetSimWSPSocket(
	int                 af,
	int                 type,
	int                 protocol,
	LPWSAPROTOCOL_INFOW lpProtocolInfo,
	GROUP               g,
	DWORD               dwFlags,
	LPINT               lpErrno
)
{
	REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;

	SOCKET result = gWrapperProcTable.lpWSPSocket(af, type, protocol, lpProtocolInfo,
		g, dwFlags, lpErrno, &redirection);

	gNestingCounter--;

	return result;
}

int WSPAPI NetSimWSPStringToAddress(
	LPWSTR              AddressString,
	INT                 AddressFamily,
	LPWSAPROTOCOL_INFOW lpProtocolInfo,   
	LPSOCKADDR          lpAddress,
	LPINT               lpAddressLength,
	LPINT               lpErrno
)
{
	REDIRECTION redirection;

	if (!NetSimRedirectCurrentCall(&redirection)) // redirection not valid
	{
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
	}

	gNestingCounter++;
	int result = gWrapperProcTable.lpWSPStringToAddress(AddressString, AddressFamily, lpProtocolInfo, lpAddress, lpAddressLength, lpErrno, &redirection);
	gNestingCounter--;

	return result;
}