/*

Copyright (c) 2006 Pavel Novak.

The use and distribution terms for this software are contained in the file named License.txt, 
which can be found in the root of the NetSim distribution. By using this software 
in any fashion, you are agreeing to be bound by the terms of this license.

You must not remove this notice from this software.

*/
#include "stdafx.h"

#include "LSPWrapper.h"


namespace NetSim
{

	// Initialize method is used instead of static constructor because for some reason (I don't know why)
	// the static constructor is not called at the right time...
	void RemoteSockets::Initialize()
	{
		if (initialized)
			return;

		// double check
		initializingLock->WaitOne();
		if (initialized)
		{
			initializingLock->Set();
			return;
		}

		// Future version: (Currently only shared memory used)
		// create and register the tcp channel
		//System::Collections::Specialized::ListDictionary __gc * tcpChannelProperties = __gc new System::Collections::Specialized::ListDictionary();
		//tcpChannelProperties->Add(__gc new String("port"), __box(0)); // let the system assign the port
		//tcpChannelProperties->Add(__gc new String("name"), __gc new String("")); // avoid collisions with other tcp channels
		//BinaryServerFormatterSinkProvider * tcp_server_prov = __gc new BinaryServerFormatterSinkProvider();
		//tcp_server_prov->TypeFilterLevel = System::Runtime::Serialization::Formatters::TypeFilterLevel::Full; // because of security restrictions
		//tcpChannel = __gc new TcpChannel(tcpChannelProperties, __gc new BinaryClientFormatterSinkProvider(), tcp_server_prov);
		//ChannelServices::RegisterChannel(tcpChannel);

		// create and register the shm channel
		BinaryServerFormatterSinkProvider __gc * shm_server_prov = __gc new BinaryServerFormatterSinkProvider();
		shm_server_prov->TypeFilterLevel = System::Runtime::Serialization::Formatters::TypeFilterLevel::Full; // because of security restrictions
		System::Collections::Specialized::ListDictionary __gc * shmChannelProperties = __gc new System::Collections::Specialized::ListDictionary();
		shmChannelProperties->Add(__gc new String("name"), __gc new String("LSPWrapperShmChannel")); // let the system assign the port
		// do not specify section name - it will be generated > we do not have to care about collisions in names
		shmChannel = __gc new PHP::Core::ShmChannel(shmChannelProperties, __gc new BinaryClientFormatterSinkProvider(), shm_server_prov);
		ChannelServices::RegisterChannel(shmChannel);

		// initialize static data
		callbackObjects = __gc new Hashtable();
		remoteSocketsManagers = __gc new ArrayList();
		remoteSocketsManagersLock = __gc new AutoResetEvent(true);

		// initialize SocketCallbackObject callback delegates to call us
		SocketCallbackObject::SetWrapperReceiveDelegate(__gc new NetSim::Core::WrapperReceiveDelegate(NULL, &Callbacks::ReceiveCallback));
		SocketCallbackObject::SetWrapperSendDelegate(__gc new NetSim::Core::WrapperSendDelegate(NULL, &Callbacks::SendCallback));
		SocketCallbackObject::SetWrapperAsyncSelectDelegate(__gc new NetSim::Core::WrapperAsyncSelectDelegate(NULL, &Callbacks::AsyncSelectCallback));

		initialized = true;
		initializingLock->Set();
	}

	RemoteSocketsManager __gc * RemoteSockets::GetManager(System::String * computer, UInt16 port)
	{
		if (!initialized)
			Initialize();

		RemoteSocketsManager __gc * manager = NULL;

		//// lock while possibly modifying table
		//remoteSocketsManagersLock->WaitOne();

		//// find the appropriate remote object (sockets manager)
		//for (int i = 0; i < remoteSocketsManagers->Count; i++)
		//{
		//	RemoteSocketsManagersEntry * entry = 
		//		dynamic_cast<RemoteSocketsManagersEntry *>(remoteSocketsManagers->get_Item(i));
		//	
		//	if (entry->computer->Equals(computer) && entry->port == port)
		//	{
		//		manager = entry->manager;
		//		break;
		//	}
		//}

		Exception * error_exception = NULL;
		if (manager == NULL) // manager not found, try to connect
		{
			try
			{
				// there is some define to GetObject which makes problems here..
#ifdef GetObject
#undef GetObject
#endif

				// use shared memory for local computer, TCP channel for remote computers
				//if (computer->Equals(__gc new System::String("localhost")) || computer->Equals(__gc new System::String("127.0.0.1")))
				manager = dynamic_cast<RemoteSocketsManager __gc *>(Activator::GetObject(__typeof(RemoteSocketsManager), System::String::Format("shm://NetSimServer/{0}", Simulator::RemoteSocketsManagerObjectName)));
				//else
				//	manager = dynamic_cast<RemoteSocketsManager __gc *>(Activator::GetObject(__typeof(RemoteSocketsManager), System::String::Format("tcp://{0}:{1}/{2}", computer, __box(port), Simulator::RemoteSocketsManagerObjectName)));
			}
			catch (Exception * e) // catch everything here, we have to unlock later
			{
				error_exception = e;
			}

			//// manager reference succesfuly found, add to the table and return
			//if (manager != NULL)
			//{
			//	RemoteSocketsManagersEntry * entry = new RemoteSocketsManagersEntry(computer, port, manager);
			//	remoteSocketsManagers->Add(entry);
			//}
		}
		//// unlock
		//remoteSocketsManagersLock->Set();

		// if manager is valid, return the reference
		if (manager != NULL)
			return manager;

		// manager not found and reference cannot be obtained
		System::String * err_msg = System::String::Format("Cannot redirect WinSock API call to {0}:{1} - unable to connect.", computer, __box(port));
		IntPtr str_ptr = System::Runtime::InteropServices::Marshal::StringToHGlobalAuto(err_msg);
		::MessageBox(NULL, static_cast<LPCTSTR>(str_ptr.ToPointer()), "Socket redirection error", MB_OK);
		System::Runtime::InteropServices::Marshal::FreeHGlobal(str_ptr);

		throw new RemoteSocketsException();
	}

	void RemoteSockets::SetCallbackObject(int socket, SocketCallbackObject __gc * co)
	{
		if (!initialized)
			Initialize();

		if (callbackObjects->ContainsKey(__box(socket)))
			throw __gc new InvalidOperationException(__gc new String("That socket already exists."));

		callbackObjects->Add(__box(socket), co);
	}

	void RemoteSockets::DeleteCallbackObject(int socket)
	{
		if (!initialized)
			Initialize();

		callbackObjects->Remove(__box(socket));
	}

	SocketCallbackObject __gc * RemoteSockets::GetCallbackObject(int socket)
	{
		if (!initialized)
			Initialize();

		return static_cast<SocketCallbackObject __gc *>(callbackObjects->get_Item(__box(socket)));
	}

}