/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

#ifndef _LSPWRAPPER_H_
#define _LSPWRAPPER_H_

#pragma once
#include "spi.h"

using namespace System;
using namespace System::Runtime::Remoting;
using namespace System::Runtime::Remoting::Channels;
using namespace System::Runtime::Remoting::Channels::Tcp;
using namespace System::Collections;
using namespace System::Threading;
using namespace NetSim::Core;


namespace NetSim
{
	/// <summary>
	/// Stores a reference to the RemoteSocketsManager along with information where the manager
	/// resides (computer and port).
	/// </summary>
	public __gc class RemoteSocketsManagersEntry
	{
	public:
		/// <summary>Computer name or IP address</summary>
		System::String * computer;
		/// <summary>TCP port where the manager is listening</summary>
		UInt16 port;
		/// <summary>Reference to the manager</summary>
		RemoteSocketsManager * manager;

		RemoteSocketsManagersEntry(System::String * computer, UInt16 port, RemoteSocketsManager * manager)
		{
			this->computer = computer;
			this->port = port;
			this->manager = manager;
		}
	};

	public __gc class RemoteSocketsException : public System::Exception
	{
	};

	public __gc class RemoteSockets
	{
	private:
		// TCP channel to the simulator (or other program with real sockets)
		static IChannel __gc * tcpChannel;

		// Shared memory channel to the simulator (or other program with real sockets)
		static IChannel __gc * shmChannel;

		// remote reference to object maintaining real sockets
		static ArrayList __gc * remoteSocketsManagers;
		static AutoResetEvent __gc * remoteSocketsManagersLock;

		// local objects called by remote manager
		static Hashtable __gc * callbackObjects;

		static AutoResetEvent __gc * initializingLock = __gc new AutoResetEvent(true);
		static bool initialized = false;

	public:

		// Initialize method is used instead of static constructor because for some reason (I don't know why)
		// the static constructor is not called at the right time...
		static void Initialize();

		static RemoteSocketsManager __gc * GetManager(System::String * computer, UInt16 port);
		static void SetCallbackObject(int socket, SocketCallbackObject __gc * co);
		static void DeleteCallbackObject(int socket);
		static SocketCallbackObject __gc * GetCallbackObject(int socket);
	};

}

#endif