/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

#include "stdafx.h"

#include <assert.h>
#include "spi.h"
#include "LSPWrapper.h"

using namespace System::Collections;
using namespace System::Net;
using namespace System::Net::Sockets;
using namespace NetSim::Core;


namespace NetSim
{
	WSPUPCALLTABLE gMainUpCallTable;

	__declspec(dllexport) int __cdecl NestingCount(int add)
	{
		return NestingCounter::Count(add);
	}

	__declspec(dllexport) UINT __cdecl GetThreadStartAddress(int pid, int tid)
	{
		return NestingCounter::GetThreadStartAddress(pid, tid);
	}

	__declspec(dllexport) int __cdecl InitProcTable(LPNETSIMWSPPROC_TABLE lpProcTable, LPWSPUPCALLTABLE lpMainUpCallTable)
	{
		// save upcall table first
		memcpy(&gMainUpCallTable, lpMainUpCallTable, sizeof(gMainUpCallTable));

		// initialize proc table
		lpProcTable->lpWSPAccept              = NetSimWSPAccept;
		lpProcTable->lpWSPAddressToString     = NetSimWSPAddressToString;
		lpProcTable->lpWSPAsyncSelect         = NetSimWSPAsyncSelect;
		lpProcTable->lpWSPBind                = NetSimWSPBind;
		lpProcTable->lpWSPCancelBlockingCall  = NetSimWSPCancelBlockingCall;
		lpProcTable->lpWSPCleanup             = NetSimWSPCleanup;
		lpProcTable->lpWSPCloseSocket         = NetSimWSPCloseSocket;
		lpProcTable->lpWSPConnect             = NetSimWSPConnect;
		lpProcTable->lpWSPDuplicateSocket     = NetSimWSPDuplicateSocket;
		lpProcTable->lpWSPEnumNetworkEvents   = NetSimWSPEnumNetworkEvents;
		lpProcTable->lpWSPEventSelect         = NetSimWSPEventSelect;
		lpProcTable->lpWSPGetOverlappedResult = NetSimWSPGetOverlappedResult;
		lpProcTable->lpWSPGetPeerName         = NetSimWSPGetPeerName;
		lpProcTable->lpWSPGetSockName         = NetSimWSPGetSockName;
		lpProcTable->lpWSPGetSockOpt          = NetSimWSPGetSockOpt;
		lpProcTable->lpWSPGetQOSByName        = NetSimWSPGetQOSByName;
		lpProcTable->lpWSPIoctl               = NetSimWSPIoctl;
		lpProcTable->lpWSPJoinLeaf            = NetSimWSPJoinLeaf;
		lpProcTable->lpWSPListen              = NetSimWSPListen;
		lpProcTable->lpWSPRecv                = NetSimWSPRecv;
		lpProcTable->lpWSPRecvDisconnect      = NetSimWSPRecvDisconnect;
		lpProcTable->lpWSPRecvFrom            = NetSimWSPRecvFrom;
		lpProcTable->lpWSPSelect              = NetSimWSPSelect;
		lpProcTable->lpWSPSend                = NetSimWSPSend;
		lpProcTable->lpWSPSendDisconnect      = NetSimWSPSendDisconnect;
		lpProcTable->lpWSPSendTo              = NetSimWSPSendTo;
		lpProcTable->lpWSPSetSockOpt          = NetSimWSPSetSockOpt;
		lpProcTable->lpWSPShutdown            = NetSimWSPShutdown;
		lpProcTable->lpWSPSocket              = NetSimWSPSocket;
		lpProcTable->lpWSPStringToAddress     = NetSimWSPStringToAddress;

		return NO_ERROR;
	}

	//
	// Helper functions
	//
	IPEndPoint __gc * SockAddrToEndPoint(const sockaddr * name, int namelen = 0)
	{
		if (namelen > 0 && namelen < sizeof(sockaddr_in)) // to small to be internet address
			return NULL;
		
		if (name->sa_family != AF_INET) // no internet family
			return NULL;

		const sockaddr_in * sa_in = reinterpret_cast<const sockaddr_in *>(name);

		IPEndPoint __gc * ep = __gc new IPEndPoint(sa_in->sin_addr.S_un.S_addr,
			IPAddress::NetworkToHostOrder((short)sa_in->sin_port));

		return ep;
	}

	int EndPointToSockAddr(IPEndPoint __gc * ep, sockaddr * name, int * namelen)
	{
		if (*namelen < sizeof(sockaddr_in)) // to small to be internet address
			return SOCKET_ERROR;

		*namelen = sizeof(sockaddr_in);

		sockaddr_in * sa_in = reinterpret_cast<sockaddr_in *>(name);
		sa_in->sin_family = AF_INET;
		unsigned char addr_bytes __gc[] = ep->Address->GetAddressBytes();
		sa_in->sin_addr.S_un.S_un_b.s_b1 = addr_bytes[0];
		sa_in->sin_addr.S_un.S_un_b.s_b2 = addr_bytes[1];
		sa_in->sin_addr.S_un.S_un_b.s_b3 = addr_bytes[2];
		sa_in->sin_addr.S_un.S_un_b.s_b4 = addr_bytes[3];
		sa_in->sin_port = IPAddress::HostToNetworkOrder((short)ep->Port);

		return 0;
	}

	SOCKET WSPAPI NetSimWSPAccept(
		SOCKET          s,                      
		struct sockaddr FAR * addr,  
		LPINT           addrlen,                 
		LPCONDITIONPROC lpfnCondition,  // ignored
		DWORD_PTR       dwCallbackData,          
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		SOCKET result;
		EndPoint __gc * localEndPoint;

		SocketCallbackObject __gc * callback_object = __gc new SocketCallbackObject();
		ObjRef __gc * co_ref = RemotingServices::Marshal(callback_object);

		try
		{
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			result = manager->Accept(s, &localEndPoint, co_ref);
		}
		catch (System::InvalidOperationException __gc *)
		{
			*lpErrno = WSAEINVAL;
			return SOCKET_ERROR;
		}
		catch (SocketException __gc *)
		{
			*lpErrno = WSAEACCES;
			return SOCKET_ERROR;
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		// remember callback object for this socket
		callback_object->SetHashCode(result);
		RemoteSockets::SetCallbackObject(result, callback_object);

		if (addr)
		{
			EndPointToSockAddr(static_cast<IPEndPoint __gc *>(localEndPoint), addr, addrlen);
		}

		return result;
	}

	int WSPAPI NetSimWSPAddressToString(
		LPSOCKADDR          lpsaAddress,            
		DWORD               dwAddressLength,               
		LPWSAPROTOCOL_INFOW lpProtocolInfo,   
		LPWSTR              lpszAddressString,            
		LPDWORD             lpdwAddressStringLength,   
		LPINT               lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		EndPoint __gc * ep = SockAddrToEndPoint(lpsaAddress, dwAddressLength);
		if (ep == NULL)
		{
			*lpErrno = WSAEINVAL;
			return SOCKET_ERROR;
		}

		String __gc * str = ep->ToString();
		if (str->Length*sizeof(WCHAR) > *lpdwAddressStringLength)
		{
			*lpErrno = WSAEFAULT;
			return SOCKET_ERROR;
		}

		System::CharEnumerator __gc * enumerator = str->GetEnumerator();
		int i = 0;
		while (enumerator->MoveNext())
			lpszAddressString[i++] = enumerator->get_Current();

		*lpdwAddressStringLength = str->Length*sizeof(WCHAR);

		return 0;
	}

	int WSPAPI NetSimWSPAsyncSelect(
		SOCKET       s,
		HWND         hWnd,
		unsigned int wMsg,
		long         lEvent,
		LPINT        lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
		int event_val = 0;
		if ((lEvent & FD_READ) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::Read);
		if ((lEvent & FD_WRITE) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::Write);
		if ((lEvent & FD_OOB) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::OutOfBand);
		if ((lEvent & FD_ACCEPT) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::Accept);
		if ((lEvent & FD_CONNECT) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::Connect);
		if ((lEvent & FD_CLOSE) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::Close);
		if ((lEvent & FD_QOS) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::Qos);
		if ((lEvent & FD_GROUP_QOS) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::GroupQos);
		if ((lEvent & FD_ROUTING_INTERFACE_CHANGE) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::RoutingIfaceGange);
		if ((lEvent & FD_ADDRESS_LIST_CHANGE) != 0) event_val |= System::Convert::ToInt32(AsyncSelectEvent::AddressListChange);

		try
		{
			System::Object __gc * event = System::Enum::ToObject(__typeof(AsyncSelectEvent), event_val);
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->AsyncSelect(s, hWnd, wMsg, *dynamic_cast<__box AsyncSelectEvent *>(event));
		}
		catch (ObjectDisposedException __gc *)
		{
			*lpErrno = WSAENOTSOCK;
			return SOCKET_ERROR;
		}
		catch (InvalidOperationException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}
		catch (SocketException __gc *)
		{
			*lpErrno = WSAEADDRINUSE;
			return SOCKET_ERROR;
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	int WSPAPI NetSimWSPBind(
		SOCKET                s,
		const struct sockaddr FAR * name,
		int                   namelen,
		LPINT                 lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
		// only AF_INET is supported
		if ( ((sockaddr_in*)name)->sin_family != AF_INET )
		{
			*lpErrno = WSAEADDRNOTAVAIL;
			return SOCKET_ERROR;
		}

		// create end point
		IPEndPoint __gc * endpoint = NULL;
		try
		{
			IPAddress __gc * ip_address = NULL;
			if (((sockaddr_in*)name)->sin_addr.s_addr == ADDR_ANY)
				ip_address = IPAddress::Any;
			else
				ip_address = __gc new IPAddress(((sockaddr_in*)name)->sin_addr.s_addr);

			endpoint = __gc new IPEndPoint(
				ip_address, IPAddress::NetworkToHostOrder((short)((sockaddr_in*)name)->sin_port) );
		}
		catch (ArgumentOutOfRangeException __gc *)
		{
			*lpErrno = WSAEADDRNOTAVAIL;
			return SOCKET_ERROR;
		}

		// finally bind
		try
		{
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->Bind(s, endpoint);
		}
		catch (ObjectDisposedException __gc *)
		{
			*lpErrno = WSAENOTSOCK;
			return SOCKET_ERROR;
		}
		catch (InvalidOperationException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}
		catch (SocketException __gc *)
		{
			*lpErrno = WSAEADDRINUSE;
			return SOCKET_ERROR;
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	int WSPAPI NetSimWSPCancelBlockingCall(
		LPINT lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPCancelBlockingCall()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	int WSPAPI NetSimWSPCleanup(
		LPINT lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPCleanup()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	int WSPAPI NetSimWSPCloseSocket(  
		SOCKET s,        
		LPINT  lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
		try
		{
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->CloseSocket(s);
            RemoteSockets::DeleteCallbackObject(s);
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	int WSPAPI NetSimWSPConnect(
		SOCKET                s,
		const struct sockaddr FAR * name,
		int                   namelen,
		LPWSABUF              lpCallerData, // ignored in this implementation
		LPWSABUF              lpCalleeData, // ignored
		LPQOS                 lpSQOS, // ignored
		LPQOS                 lpGQOS,
		LPINT                 lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
		EndPoint __gc * ep = SockAddrToEndPoint(name, namelen);

		try
		{
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->Connect(s, ep);
		}
		catch (SocketException __gc * e)
		{
			*lpErrno = e->ErrorCode;
			return SOCKET_ERROR;
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	int WSPAPI NetSimWSPDuplicateSocket(
		SOCKET              s,
		DWORD               dwProcessId,                      
		LPWSAPROTOCOL_INFOW lpProtocolInfo,   
		LPINT               lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPDuplicateSocket()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	int WSPAPI NetSimWSPEnumNetworkEvents(  
		SOCKET             s,
		WSAEVENT           hEventObject,
		LPWSANETWORKEVENTS lpNetworkEvents,
		LPINT              lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPEnumNetworkEvents()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	int WSPAPI NetSimWSPEventSelect(
		SOCKET   s,
		WSAEVENT hEventObject,
		long     lNetworkEvents,
		LPINT    lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPEventSelect()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	BOOL WSPAPI NetSimWSPGetOverlappedResult(
		SOCKET          s,
		LPWSAOVERLAPPED lpOverlapped,
		LPDWORD         lpcbTransfer,
		BOOL            fWait,
		LPDWORD         lpdwFlags,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPGetOverlappedResult()"));
#else
		*lpErrno = WSAENETDOWN;
		return FALSE;
#endif
	}

	int WSPAPI NetSimWSPGetPeerName(  
		SOCKET          s,
		struct sockaddr FAR * name,
		LPINT           namelen,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
		)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPGetPeerName()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	int WSPAPI NetSimWSPGetSockName(
		SOCKET          s,
		struct sockaddr FAR * name,
		LPINT           namelen,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		if (*namelen < sizeof(sockaddr_in))
		{
			*lpErrno = WSAEFAULT;
			return SOCKET_ERROR;
		}

		try
		{
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			EndPoint __gc * local_ep = manager->GetLocalEndPoint(s);

			if (local_ep->AddressFamily != AddressFamily::InterNetwork)
			{
				*lpErrno = WSAENETDOWN;
				return SOCKET_ERROR;
			}
			
			// endpoint is InterNetwork, it can be safely casted to IPEndPoint
			IPEndPoint __gc * ip_ep = static_cast<IPEndPoint __gc *>(local_ep);
			EndPointToSockAddr(ip_ep, name, namelen);
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	int WSPAPI NetSimWSPGetSockOpt(
		SOCKET     s,
		int        level,
		int        optname,
		char FAR * optval,
		LPINT      optlen,
		LPINT      lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		SocketOptionLevel socket_option_level;
		switch (level)
		{
		case SOL_SOCKET:
			socket_option_level = SocketOptionLevel::Socket; break;
		case IPPROTO_IP:
		case IPPROTO_IPV4:
			socket_option_level = SocketOptionLevel::IP; break;
		case IPPROTO_IPV6:
			socket_option_level = SocketOptionLevel::IPv6; break;
		case IPPROTO_TCP:
			socket_option_level = SocketOptionLevel::Tcp; break;
		case IPPROTO_UDP:
			socket_option_level = SocketOptionLevel::Udp; break;
		default:
			*lpErrno = WSAEINVAL;
			return SOCKET_ERROR;
		}

		SocketOptionName socket_option_name;
		switch (optname)
		{
		case SO_DEBUG:
			socket_option_name = SocketOptionName::Debug;
			break;
		case SO_KEEPALIVE:
			socket_option_name = SocketOptionName::KeepAlive;
			break;
		case SO_LINGER:
			socket_option_name = SocketOptionName::Linger;
			break;
		case SO_REUSEADDR:
			socket_option_name = SocketOptionName::ReuseAddress;
			break;
		case SO_SNDBUF:
			socket_option_name = SocketOptionName::SendBuffer;
			break;
		case SO_RCVBUF:
			socket_option_name = SocketOptionName::SendBuffer;
			break;
		case SO_ACCEPTCONN:
			socket_option_name = SocketOptionName::AcceptConnection;
			break;
		case SO_DONTROUTE:
			socket_option_name = SocketOptionName::DontRoute;
			break;
		case SO_BROADCAST:
			socket_option_name = SocketOptionName::Broadcast;
			break;
		case SO_USELOOPBACK:
			socket_option_name = SocketOptionName::UseLoopback;
			break;
		case SO_OOBINLINE:
			socket_option_name = SocketOptionName::OutOfBandInline;
			break;
		case SO_DONTLINGER:
			socket_option_name = SocketOptionName::DontLinger;
			break;
		case SO_EXCLUSIVEADDRUSE:
			socket_option_name = SocketOptionName::ExclusiveAddressUse;
			break;
		case SO_SNDLOWAT:  /* send low-water mark */
			socket_option_name = SocketOptionName::SendLowWater;
			break;
		case SO_RCVLOWAT:  /* receive low-water mark */
			socket_option_name = SocketOptionName::ReceiveLowWater;
			break;
		case SO_SNDTIMEO:  /* send timeout */
			socket_option_name = SocketOptionName::SendTimeout;
			break;
		case SO_RCVTIMEO:  /* receive timeout */
			socket_option_name = SocketOptionName::ReceiveTimeout;
			break;
		case SO_ERROR:     /* get error status and clear */
			socket_option_name = SocketOptionName::Error;
			break;
		case SO_TYPE:      /* get socket type */
			socket_option_name = SocketOptionName::Type;
			break;
		default:
			*lpErrno = WSAENOPROTOOPT;
			return SOCKET_ERROR;
		}

		try
		{
			Byte val __gc[] = __gc new Byte[*optlen];
			Byte (__gc * valptr) __gc [] = &val;
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->GetSockOpt(s, socket_option_level, socket_option_name, valptr, optlen);
			
			// copy value back
			for (int i = 0; i < *optlen; i++)
				optval[i] = (*valptr)[i];
		}
		catch (SocketException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}
		catch (System::InvalidOperationException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	BOOL WSPAPI NetSimWSPGetQOSByName(
		SOCKET   s,
		LPWSABUF lpQOSName,
		LPQOS    lpQOS,
		LPINT    lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPGetQOSByName()"));
#else
		*lpErrno = WSAENETDOWN;
		return FALSE;
#endif
	}

	int WSPAPI NetSimWSPIoctl(
		SOCKET          s,
		DWORD           dwIoControlCode,
		LPVOID          lpvInBuffer,
		DWORD           cbInBuffer,
		LPVOID          lpvOutBuffer,
		DWORD           cbOutBuffer,
		LPDWORD         lpcbBytesReturned,
		LPWSAOVERLAPPED lpOverlapped,
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
		LPWSATHREADID   lpThreadId,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		//IoctlControlCode code;
		switch (dwIoControlCode)
		{
		case FIONBIO:
			break;
		case FIONREAD:
			break;
		case SIOCATMARK:
			break;
		case SIO_ASSOCIATE_HANDLE:
			break;
		case SIO_ENABLE_CIRCULAR_QUEUEING:
			break;
		case SIO_FIND_ROUTE:
			break;
		case SIO_FLUSH:
			break;
		case SIO_GET_BROADCAST_ADDRESS:
			break;
		case SIO_GET_EXTENSION_FUNCTION_POINTER:
			break;
		case SIO_GET_QOS:
			break;
		case SIO_GET_GROUP_QOS:
			break;
		case SIO_MULTIPOINT_LOOPBACK:
			break;
		case SIO_MULTICAST_SCOPE:
			break;
		case SIO_SET_QOS:
			break;
		case SIO_SET_GROUP_QOS:
			break;
		case SIO_TRANSLATE_HANDLE:
			break;
		case SIO_ROUTING_INTERFACE_QUERY:
			break;
		case SIO_ROUTING_INTERFACE_CHANGE:
			break;
		case SIO_ADDRESS_LIST_QUERY:
			break;
		case SIO_ADDRESS_LIST_CHANGE:
			break;
		case SIO_QUERY_TARGET_PNP_HANDLE:
			break;
		case SIO_ADDRESS_LIST_SORT:
			break;
		}

#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPIoctl()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	SOCKET WSPAPI NetSimWSPJoinLeaf(
		SOCKET       s,
		const struct sockaddr FAR * name,
		int          namelen,
		LPWSABUF     lpCallerData,
		LPWSABUF     lpCalleeData,
		LPQOS        lpSQOS,
		LPQOS        lpGQOS,
		DWORD        dwFlags,
		LPINT        lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPJoinLeaf()"));
#else
		*lpErrno = WSAENETDOWN;
		return INVALID_SOCKET;
#endif
	}

	int WSPAPI NetSimWSPListen(
		SOCKET s,        
		int    backlog,     
		LPINT  lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		try
		{
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->Listen(s, backlog);
		}
		catch (System::InvalidOperationException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	int WSPAPI NetSimWSPRecv(
		SOCKET          s,
		LPWSABUF        lpBuffers,
		DWORD           dwBufferCount,
		LPDWORD         lpNumberOfBytesRecvd,
		LPDWORD         lpFlags,
		LPWSAOVERLAPPED lpOverlapped,
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
		LPWSATHREADID   lpThreadId,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	)	
	{
		// we should always have a completion routine if overlapped operation is requested
		// because we are called from our LSP
		// (lpCompletionRoutine == NULL) => (lpOverlapped == NULL)
		assert(lpCompletionRoutine != NULL || lpOverlapped == NULL);
		
		RecvContext __gc * r_context = new RecvContext();
		r_context->lpBuffers = lpBuffers;
		r_context->dwBufferCount = dwBufferCount;
		r_context->lpNumberOfBytesRecvd = lpNumberOfBytesRecvd;
		r_context->lpFlags = lpFlags;
		r_context->lpOverlapped = lpOverlapped;
		r_context->lpCompletionRoutine = lpCompletionRoutine;
		r_context->lpThreadId = lpThreadId;
		r_context->lpErrno = lpErrno;

		try
		{
			// remember the context at the socket callback object
			RemoteSockets::GetCallbackObject(s)->SetPendingReceive(r_context->GetHashCode(), r_context);

			// count the space for data we have
			int bytes = 0;
			for (DWORD i = 0; i < dwBufferCount; i++)
				bytes += lpBuffers[i].len;

			// begin receive operation
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->Receive(s, bytes, r_context->GetHashCode());

			if (lpCompletionRoutine == NULL) // we should block and wait for data
			{
				WaitForSingleObject(r_context->waitForData, INFINITE);
			}
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	int WSPAPI NetSimWSPRecvDisconnect(
		SOCKET   s,
		LPWSABUF lpInboundDisconnectData,
		LPINT    lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPRecvDisconnect()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	int WSPAPI NetSimWSPRecvFrom(
		SOCKET          s,
		LPWSABUF        lpBuffers,
		DWORD           dwBufferCount,
		LPDWORD         lpNumberOfBytesRecvd,
		LPDWORD         lpFlags,
		struct sockaddr FAR * lpFrom,
		LPINT           lpFromLen,
		LPWSAOVERLAPPED lpOverlapped,
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
		LPWSATHREADID   lpThreadId,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPRecvFrom()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	int WSPAPI NetSimWSPSelect(
		int          nfds,           // ignored according to specification
		fd_set FAR * readfds,
		fd_set FAR * writefds,
		fd_set FAR * exceptfds,
		const struct timeval FAR * timeout,
		LPINT        lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		int read_count = readfds ? readfds->fd_count : 0;
		int write_count = writefds ? writefds->fd_count : 0;
		int except_count = exceptfds ? exceptfds->fd_count : 0;

		ArrayList __gc * read_descriptors = __gc new ArrayList(read_count);
		ArrayList __gc * write_descriptors = __gc new ArrayList(write_count);
		ArrayList __gc * except_descriptors = __gc new ArrayList(except_count);

		for (int i = 0; i < read_count; i++)
			read_descriptors->Add(__box((int)readfds->fd_array[i]));
		for (int i = 0; i < write_count; i++)
			write_descriptors->Add(__box((int)writefds->fd_array[i]));
		for (int i = 0; i < except_count; i++)
			except_descriptors->Add(__box((int)exceptfds->fd_array[i]));

		// if timeout is null, wait "forever"
		int timeout_ms = timeout ? timeout->tv_sec*1000000 + timeout->tv_usec : -1;

		int result = 0;
		try
		{
			ArrayList __gc * __gc * read_pp = & read_descriptors;
			ArrayList __gc * __gc * write_pp = & write_descriptors;
			ArrayList __gc * __gc * except_pp = & except_descriptors;

			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			result = manager->Select(read_pp, write_pp, except_pp, timeout_ms);
		}
		catch (System::InvalidOperationException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			result = SOCKET_ERROR;
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		// update descriptors sets
		if (readfds)
		{
			FD_ZERO(readfds);
			for (int i = 0; i < read_descriptors->Count; i++)
				FD_SET(*(dynamic_cast<System::Int32 __gc *>(read_descriptors->get_Item(i))), readfds);
		}
		if (writefds)
		{
			FD_ZERO(writefds);
			for (int i = 0; i < write_descriptors->Count; i++)
				FD_SET(*(dynamic_cast<System::Int32 __gc *>(write_descriptors->get_Item(i))), writefds);
		}
		if (exceptfds)
		{
			FD_ZERO(exceptfds);
			for (int i = 0; i < except_descriptors->Count; i++)
				FD_SET(*(dynamic_cast<System::Int32 __gc *>(except_descriptors->get_Item(i))), exceptfds);
		}

		return result;
	}

	int WSPAPI NetSimWSPSend(
		SOCKET          s,
		LPWSABUF        lpBuffers,
		DWORD           dwBufferCount,
		LPDWORD         lpNumberOfBytesSent,
		DWORD           dwFlags,
		LPWSAOVERLAPPED lpOverlapped, // ignored, since we do not support overlapped transfers
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,   
		LPWSATHREADID   lpThreadId,                                 
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		// we should always have a completion routine if overlapped operation is requested
		// because we are called from our LSP
		// (lpCompletionRoutine == NULL) => (lpOverlapped == NULL)
		assert(lpCompletionRoutine != NULL || lpOverlapped == NULL);
		
		SendContext __gc * s_context = new SendContext();
		s_context->lpBuffers = lpBuffers;
		s_context->dwBufferCount = dwBufferCount;
		s_context->lpNumberOfBytesSent = lpNumberOfBytesSent;
		s_context->dwFlags = dwFlags;
		s_context->lpOverlapped = lpOverlapped;
		s_context->lpCompletionRoutine = lpCompletionRoutine;
		s_context->lpThreadId = lpThreadId;
		s_context->lpErrno = lpErrno;

		try
		{
			// remember the context at the socket callback object
			RemoteSockets::GetCallbackObject(s)->SetPendingSend(s_context->GetHashCode(), s_context);

			// count the the amount of data we should send
			int bytes = 0;
			for (DWORD i = 0; i < dwBufferCount; i++)
				bytes += lpBuffers[i].len;
			
			// copy data to the managed buffer
			System::Byte buffer __gc[] = __gc new System::Byte __gc[bytes];
			int current_buffer = 0;
			int buf_pos = 0;
			int data_pos = 0;
			while (data_pos < bytes)
			{
				assert(current_buffer < (int)dwBufferCount);

				if (buf_pos == lpBuffers[current_buffer].len)
				{
					current_buffer++;
					buf_pos = 0;
					continue;
				}

				buffer[data_pos] = lpBuffers[current_buffer].buf[buf_pos];
				data_pos++;
				buf_pos++;
			}
			
			// begin send operation
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->Send(s, buffer, bytes, s_context->GetHashCode());

			if (lpCompletionRoutine == NULL) // we should block and wait for data
			{
				WaitForSingleObject(s_context->waitForCompletion, INFINITE);
			}
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	int WSPAPI NetSimWSPSendDisconnect(
		SOCKET   s,
		LPWSABUF lpOutboundDisconnectData,
		LPINT    lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPSendDisconnect()"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}

	int WSPAPI NetSimWSPSendTo(
		SOCKET          s,
		LPWSABUF        lpBuffers,
		DWORD           dwBufferCount,
		LPDWORD         lpNumberOfBytesSent,
		DWORD           dwFlags,
		const struct sockaddr FAR * lpTo,
		int             iToLen,
		LPWSAOVERLAPPED lpOverlapped,
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
		LPWSATHREADID   lpThreadId,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPSendTo)"));
#else
		*lpErrno = WSAENETDOWN;
		return SOCKET_ERROR;
#endif
	}


	enum SockOptType { SockOptBool, SockOptInt, SockOptStructLinger };

	int WSPAPI NetSimWSPSetSockOpt(
		SOCKET     s,
		int        level,
		int        optname,
		const char FAR * optval,   
		int        optlen,
		LPINT      lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		SocketOptionLevel socket_option_level;
		switch (level)
		{
		case SOL_SOCKET:
			socket_option_level = SocketOptionLevel::Socket; break;
		case IPPROTO_IP:
		case IPPROTO_IPV4:
			socket_option_level = SocketOptionLevel::IP; break;
		case IPPROTO_IPV6:
			socket_option_level = SocketOptionLevel::IPv6; break;
		case IPPROTO_TCP:
			socket_option_level = SocketOptionLevel::Tcp; break;
		case IPPROTO_UDP:
			socket_option_level = SocketOptionLevel::Udp; break;
		default:
			*lpErrno = WSAEINVAL;
			return SOCKET_ERROR;
		}

		SocketOptionName socket_option_name;
		switch (optname)
		{
		case SO_DEBUG:
			socket_option_name = SocketOptionName::Debug;
			break;
		case SO_KEEPALIVE:
			socket_option_name = SocketOptionName::KeepAlive;
			break;
		case SO_LINGER:
			socket_option_name = SocketOptionName::Linger;
			break;
		case SO_REUSEADDR:
			socket_option_name = SocketOptionName::ReuseAddress;
			break;
		case SO_SNDBUF:
			socket_option_name = SocketOptionName::SendBuffer;
			break;
		case SO_RCVBUF:
			socket_option_name = SocketOptionName::SendBuffer;
			break;
		case SO_ACCEPTCONN:
			socket_option_name = SocketOptionName::AcceptConnection;
			break;
		case SO_DONTROUTE:
			socket_option_name = SocketOptionName::DontRoute;
			break;
		case SO_BROADCAST:
			socket_option_name = SocketOptionName::Broadcast;
			break;
		case SO_USELOOPBACK:
			socket_option_name = SocketOptionName::UseLoopback;
			break;
		case SO_OOBINLINE:
			socket_option_name = SocketOptionName::OutOfBandInline;
			break;
		case SO_DONTLINGER:
			socket_option_name = SocketOptionName::DontLinger;
			break;
		case SO_EXCLUSIVEADDRUSE:
			socket_option_name = SocketOptionName::ExclusiveAddressUse;
			break;
		case SO_SNDLOWAT:  /* send low-water mark */
			socket_option_name = SocketOptionName::SendLowWater;
			break;
		case SO_RCVLOWAT:  /* receive low-water mark */
			socket_option_name = SocketOptionName::ReceiveLowWater;
			break;
		case SO_SNDTIMEO:  /* send timeout */
			socket_option_name = SocketOptionName::SendTimeout;
			break;
		case SO_RCVTIMEO:  /* receive timeout */
			socket_option_name = SocketOptionName::ReceiveTimeout;
			break;
		case SO_ERROR:     /* get error status and clear */
			socket_option_name = SocketOptionName::Error;
			break;
		default:
			*lpErrno = WSAENOPROTOOPT;
			return SOCKET_ERROR;
		}

		try
		{
			int * option = reinterpret_cast<int *>(const_cast<char *>(optval));
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->SetSockOpt(s, socket_option_level, socket_option_name, option, optlen);
		}
		catch (SocketException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}
		catch (System::InvalidOperationException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	int WSPAPI NetSimWSPShutdown(
		SOCKET s,
		int    how,
		LPINT  lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		SocketShutdown shutdown_how;
		switch (how)
		{
		case SD_RECEIVE:
			shutdown_how = SocketShutdown::Receive; break;
		case SD_SEND:
			shutdown_how = SocketShutdown::Send; break;
		case SD_BOTH:
			shutdown_how = SocketShutdown::Both; break;
		default:
			*lpErrno = WSAEINVAL;
			return SOCKET_ERROR;
		}

		try
		{
			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			manager->Shutdown(s, shutdown_how);
		}
		catch (SocketException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}
		catch (ApplicationException __gc *) // we are going to unmanaged code, catch all exceptions..
		{
			*lpErrno = WSAENETDOWN;
			return SOCKET_ERROR;
		}

		return 0;
	}

	SOCKET WSPAPI NetSimWSPSocket(
		int                 af,
		int                 type,
		int                 protocol,
		LPWSAPROTOCOL_INFOW lpProtocolInfo,
		GROUP               g,
		DWORD               dwFlags,
		LPINT               lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
		AddressFamily address_family;
		switch (af)
		{
		case AF_INET:
			address_family = AddressFamily::InterNetwork; break;
		default:
			*lpErrno = WSAEAFNOSUPPORT;
			return INVALID_SOCKET;
		}

		SocketType socket_type;
		switch (type)
		{
		case SOCK_STREAM:
			socket_type = SocketType::Stream; break;
		case SOCK_DGRAM:
			socket_type = SocketType::Dgram; break;
		case SOCK_RAW:
			socket_type = SocketType::Raw; break;
		default:
			*lpErrno = WSAESOCKTNOSUPPORT;
			return INVALID_SOCKET;
		}

		ProtocolType protocol_type;
		switch (protocol)
		{
		case IPPROTO_TCP:
			protocol_type = ProtocolType::Tcp; break;
		case IPPROTO_UDP:
			protocol_type = ProtocolType::Udp; break;
		case IPPROTO_IP:
		case IPPROTO_IPV4:
			protocol_type = ProtocolType::IP; break;
		default:
			*lpErrno = WSAEPROTONOSUPPORT;
			return INVALID_SOCKET;
		}

		SOCKET s = INVALID_SOCKET;
		SocketCallbackObject __gc * callback_object = NULL;
		try
		{
			//------------------------------------------------------
			//Future version:
			// Without this block of code the server cannot be reached. Find out why and make it better

			RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);

			SocketCallbackObject __gc * sc = __gc new SocketCallbackObject();
			ObjRef __gc * rf = RemotingServices::Marshal(sc);
			String __gc * uri = String::Concat(__gc new String("shm://NetSimServer/"), Simulator::RemoteSocketsManagerObjectName);
#ifdef GetObject
#undef GetObject
#endif
			RemoteSocketsManager __gc * rsm = dynamic_cast<RemoteSocketsManager __gc *>(Activator::GetObject(__typeof(RemoteSocketsManager), uri));
			rsm->Test(rf);
			int i = sc->i;
			//------------------------------------------------------

			//
			// create local object for callbacks
			//

			callback_object = __gc new SocketCallbackObject();
			ObjRef __gc * co_ref = RemotingServices::Marshal(callback_object);

			RemoteSocketsManager * manager = RemoteSockets::GetManager(lpRedirection->Computer, lpRedirection->Port);
			s = (SOCKET) manager->Socket(address_family, socket_type, protocol_type, co_ref, lpRedirection->NodeName);
		}
		catch (System::InvalidOperationException __gc * e)
		{
			String __gc * str = e->ToString();
			*lpErrno = WSAENETDOWN;
			s = INVALID_SOCKET;
		}
		catch (System::Net::Sockets::SocketException __gc * e)
		{
			String __gc * str = e->ToString();
			*lpErrno = WSAENETDOWN;
			s = INVALID_SOCKET;
		}
		catch (ApplicationException __gc *)
		{
			*lpErrno = WSAENETDOWN;
			s = INVALID_SOCKET;
		}

		if (s != INVALID_SOCKET)
		{
			// remember callback object for this socket
			callback_object->SetHashCode(s);
			RemoteSockets::SetCallbackObject(s, callback_object);
		}

		return s;
	}

	int WSPAPI NetSimWSPStringToAddress(
		LPWSTR              AddressString,
		INT                 AddressFamily,
		LPWSAPROTOCOL_INFOW lpProtocolInfo,   
		LPSOCKADDR          lpAddress,
		LPINT               lpAddressLength,
		LPINT               lpErrno,
		LPREDIRECTION lpRedirection
	)
	{
#ifdef DEBUG
		throw new NotImplementedException(new String("NetSimWSPStringToAddress()"));
#else
		*lpErrno = WSAEINVAL;
		return SOCKET_ERROR;
#endif
	}

	void Callbacks::ReceiveCallback(System::Byte data __gc[], int bytesTransferred, System::Object __gc * context)
	{
		RecvContext __gc * r_context = static_cast<RecvContext __gc *>(context);
		
		// copy data to the buffer
		*(r_context->lpNumberOfBytesRecvd) = bytesTransferred;
		int current_buffer = 0;
		int buf_pos = 0;
		int data_pos = 0;
		while (data_pos < bytesTransferred)
		{
			if (buf_pos == r_context->lpBuffers[current_buffer].len)
			{
				current_buffer++;
				buf_pos = 0;
				continue;
			}

			r_context->lpBuffers[current_buffer].buf[buf_pos] = data[data_pos];
			data_pos++;
			buf_pos++;
		}

		if (r_context->lpCompletionRoutine == NULL)
		{
			// no completion routine => the thread is blocked in WSPRecv - unblock it
			ReleaseSemaphore(r_context->waitForData, 1, NULL);
		}
		else
		{
			// call the completion routine
			r_context->lpCompletionRoutine(0, bytesTransferred, r_context->lpOverlapped, 0);
		}
	}

	void Callbacks::SendCallback(int bytesTransferred, System::Object __gc * context)
	{
		SendContext __gc * s_context = static_cast<SendContext __gc *>(context);
		
		// set info about number of bytes send
		*(s_context->lpNumberOfBytesSent) = bytesTransferred;

		if (s_context->lpCompletionRoutine == NULL)
		{
			// no completion routine => the thread is blocked in WSPRecv - unblock it
			ReleaseSemaphore(s_context->waitForCompletion, 1, NULL);
		}
		else
		{
			// call the completion routine
			s_context->lpCompletionRoutine(0, bytesTransferred, s_context->lpOverlapped, 0);
		}
	}

	void Callbacks::AsyncSelectCallback(int socket, System::IntPtr hWnd, unsigned int wMsg, AsyncSelectEvent events)
	{
		unsigned int e = 0;
		if ((events & AsyncSelectEvent::Read) != 0) e |= FD_READ;
		if ((events & AsyncSelectEvent::Write) != 0) e |= FD_WRITE;
		if ((events & AsyncSelectEvent::OutOfBand) != 0) e |= FD_OOB;
		if ((events & AsyncSelectEvent::Accept) != 0) e |= FD_ACCEPT;
		if ((events & AsyncSelectEvent::Connect) != 0) e |= FD_CONNECT;
		if ((events & AsyncSelectEvent::Close) != 0) e |= FD_CLOSE;
		if ((events & AsyncSelectEvent::Qos) != 0) e |= FD_QOS;
		if ((events & AsyncSelectEvent::GroupQos) != 0) e |= FD_GROUP_QOS;
		if ((events & AsyncSelectEvent::RoutingIfaceGange) != 0) e |= FD_ROUTING_INTERFACE_CHANGE;
		if ((events & AsyncSelectEvent::AddressListChange) != 0) e |= FD_ADDRESS_LIST_CHANGE;
		
		System::IntPtr __gc * w_param = __nogc new System::IntPtr(socket);
		System::IntPtr __gc * l_param = __nogc new System::IntPtr(reinterpret_cast<void*>(e));

		gMainUpCallTable.lpWPUPostMessage(reinterpret_cast<HWND>(hWnd.ToPointer()), wMsg,
			reinterpret_cast<WPARAM>(w_param->ToPointer()), reinterpret_cast<LPARAM>(l_param->ToPointer()));
	}
}