/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

#ifndef _SPI_H_
#define _SPI_H_

#using <NetSimCore.dll>
#using <ShmChannel.dll>

#include "../Lsp/nonifslsp/netsimspecific.h"

namespace NetSim
{
//	__declspec(dllexport) int __cdecl NestingCount(int add);

	// Called by LSP to get our functions entry points
	__declspec(dllexport) int __cdecl InitProcTable(LPWSPPROC_TABLE lpProcTable, LPWSPUPCALLTABLE lpMainUpCallTable);

	//
	// Class holding data while send is called
	//
	public __gc class SendContext
	{
	public:
		SendContext()
		{
			waitForCompletion = CreateSemaphore(NULL, 0, 1, NULL);
		}

		~SendContext()
		{
			CloseHandle(waitForCompletion);
		}

		LPWSABUF        lpBuffers;
		DWORD           dwBufferCount;
		LPDWORD         lpNumberOfBytesSent;
		DWORD           dwFlags;
		LPWSAOVERLAPPED lpOverlapped;
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine;
		LPWSATHREADID   lpThreadId;
		LPINT           lpErrno;

		HANDLE waitForCompletion;

		virtual int GetHashCode()
		{
			// buffers should be unique for each pending operation
			return reinterpret_cast<int>(lpBuffers);
		}
	};

	//
	// Class holding data while receive is called
	//
	public __gc class RecvContext
	{
	public:
		RecvContext()
		{
			//waitForData = __gc new System::Threading::Mutex(false);
			waitForData = CreateSemaphore(NULL, 0, 1, NULL);
		}

		~RecvContext()
		{
			CloseHandle(waitForData);
		}

		LPWSABUF        lpBuffers;
		DWORD           dwBufferCount;
		LPDWORD         lpNumberOfBytesRecvd;
		LPDWORD         lpFlags;
		LPWSAOVERLAPPED lpOverlapped;
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine;
		LPWSATHREADID   lpThreadId;
		LPINT           lpErrno;

		//System::Threading::Mutex __gc * waitForData;
		HANDLE waitForData;

		virtual int GetHashCode()
		{
			// buffers should be unique for each pending operation
			return reinterpret_cast<int>(lpBuffers);
		}
	};

	//
	// NetSim socket functions
	//

	SOCKET WSPAPI NetSimWSPAccept(
		SOCKET          s,                      
		struct sockaddr FAR * addr,  
		LPINT           addrlen,                 
		LPCONDITIONPROC lpfnCondition,  
		DWORD_PTR       dwCallbackData,          
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPAddressToString(
		LPSOCKADDR          lpsaAddress,            
		DWORD               dwAddressLength,               
		LPWSAPROTOCOL_INFOW lpProtocolInfo,   
		LPWSTR              lpszAddressString,            
		LPDWORD             lpdwAddressStringLength,   
		LPINT               lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPAsyncSelect(
		SOCKET       s,
		HWND         hWnd,
		unsigned int wMsg,
		long         lEvent,
		LPINT        lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPBind(
		SOCKET                s,
		const struct sockaddr FAR * name,
		int                   namelen,
		LPINT                 lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPCancelBlockingCall(
		LPINT lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPCleanup(
		LPINT lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPCloseSocket(  
		SOCKET s,        
		LPINT  lpErrno,
		LPREDIRECTION lpRedirection
		);

	int WSPAPI NetSimWSPConnect(
		SOCKET                s,
		const struct sockaddr FAR * name,
		int                   namelen,
		LPWSABUF              lpCallerData,
		LPWSABUF              lpCalleeData,
		LPQOS                 lpSQOS,
		LPQOS                 lpGQOS,
		LPINT                 lpErrno,
		LPREDIRECTION lpRedirection
		);

	int WSPAPI NetSimWSPDuplicateSocket(
		SOCKET              s,
		DWORD               dwProcessId,                      
		LPWSAPROTOCOL_INFOW lpProtocolInfo,   
		LPINT               lpErrno,
		LPREDIRECTION lpRedirection
		);

	int WSPAPI NetSimWSPEnumNetworkEvents(  
		SOCKET             s,
		WSAEVENT           hEventObject,
		LPWSANETWORKEVENTS lpNetworkEvents,
		LPINT              lpErrno,
		LPREDIRECTION lpRedirection
		);

	int WSPAPI NetSimWSPEventSelect(
		SOCKET   s,
		WSAEVENT hEventObject,
		long     lNetworkEvents,
		LPINT    lpErrno,
		LPREDIRECTION lpRedirection
		);

	BOOL WSPAPI NetSimWSPGetOverlappedResult(
		SOCKET          s,
		LPWSAOVERLAPPED lpOverlapped,
		LPDWORD         lpcbTransfer,
		BOOL            fWait,
		LPDWORD         lpdwFlags,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
		);

	int WSPAPI NetSimWSPGetPeerName(  
		SOCKET          s,
		struct sockaddr FAR * name,
		LPINT           namelen,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
		);

	int WSPAPI NetSimWSPGetSockName(
		SOCKET          s,
		struct sockaddr FAR * name,
		LPINT           namelen,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPGetSockOpt(
		SOCKET     s,
		int        level,
		int        optname,
		char FAR * optval,
		LPINT      optlen,
		LPINT      lpErrno,
		LPREDIRECTION lpRedirection
	);

	BOOL WSPAPI NetSimWSPGetQOSByName(
		SOCKET   s,
		LPWSABUF lpQOSName,
		LPQOS    lpQOS,
		LPINT    lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPIoctl(
		SOCKET          s,
		DWORD           dwIoControlCode,
		LPVOID          lpvInBuffer,
		DWORD           cbInBuffer,
		LPVOID          lpvOutBuffer,
		DWORD           cbOutBuffer,
		LPDWORD         lpcbBytesReturned,
		LPWSAOVERLAPPED lpOverlapped,
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
		LPWSATHREADID   lpThreadId,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	);

	SOCKET WSPAPI NetSimWSPJoinLeaf(
		SOCKET       s,
		const struct sockaddr FAR * name,
		int          namelen,
		LPWSABUF     lpCallerData,
		LPWSABUF     lpCalleeData,
		LPQOS        lpSQOS,
		LPQOS        lpGQOS,
		DWORD        dwFlags,
		LPINT        lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPListen(
		SOCKET s,        
		int    backlog,     
		LPINT  lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPRecv(
		SOCKET          s,
		LPWSABUF        lpBuffers,
		DWORD           dwBufferCount,
		LPDWORD         lpNumberOfBytesRecvd,
		LPDWORD         lpFlags,
		LPWSAOVERLAPPED lpOverlapped,
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
		LPWSATHREADID   lpThreadId,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPRecvDisconnect(
		SOCKET   s,
		LPWSABUF lpInboundDisconnectData,
		LPINT    lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPRecvFrom(
		SOCKET          s,
		LPWSABUF        lpBuffers,
		DWORD           dwBufferCount,
		LPDWORD         lpNumberOfBytesRecvd,
		LPDWORD         lpFlags,
		struct sockaddr FAR * lpFrom,
		LPINT           lpFromLen,
		LPWSAOVERLAPPED lpOverlapped,
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
		LPWSATHREADID   lpThreadId,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPSelect(
		int          nfds,
		fd_set FAR * readfds,
		fd_set FAR * writefds,
		fd_set FAR * exceptfds,
		const struct timeval FAR * timeout,
		LPINT        lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPSend(
		SOCKET          s,
		LPWSABUF        lpBuffers,
		DWORD           dwBufferCount,
		LPDWORD         lpNumberOfBytesSent,
		DWORD           dwFlags,
		LPWSAOVERLAPPED lpOverlapped,                             
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,   
		LPWSATHREADID   lpThreadId,                                 
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection                               
	);

	int WSPAPI NetSimWSPSendDisconnect(
		SOCKET   s,
		LPWSABUF lpOutboundDisconnectData,
		LPINT    lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPSendTo(
		SOCKET          s,
		LPWSABUF        lpBuffers,
		DWORD           dwBufferCount,
		LPDWORD         lpNumberOfBytesSent,
		DWORD           dwFlags,
		const struct sockaddr FAR * lpTo,
		int             iToLen,
		LPWSAOVERLAPPED lpOverlapped,
		LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine,
		LPWSATHREADID   lpThreadId,
		LPINT           lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPSetSockOpt(
		SOCKET     s,
		int        level,
		int        optname,
		const char FAR * optval,   
		int        optlen,
		LPINT      lpErrno,
		LPREDIRECTION lpRedirection
	);

	int WSPAPI NetSimWSPShutdown(
		SOCKET s,
		int    how,
		LPINT  lpErrno,
		LPREDIRECTION lpRedirection
	);

	SOCKET WSPAPI NetSimWSPSocket(
		int                 af,
		int                 type,
		int                 protocol,
		LPWSAPROTOCOL_INFOW lpProtocolInfo,
		GROUP               g,
		DWORD               dwFlags,
		LPINT               lpErrno,
		LPREDIRECTION lpRedirection
		);

	int WSPAPI NetSimWSPStringToAddress(
		LPWSTR              AddressString,
		INT                 AddressFamily,
		LPWSAPROTOCOL_INFOW lpProtocolInfo,   
		LPSOCKADDR          lpAddress,
		LPINT               lpAddressLength,
		LPINT               lpErrno,
		LPREDIRECTION lpRedirection
	);


	public __gc class Callbacks
	{
	public:
		// Data transfer
		static void ReceiveCallback(System::Byte data __gc[], int bytesTransferred, System::Object __gc * context);
		static void SendCallback(int bytesTransferred, System::Object __gc * context);

		// Async notifications
		static void AsyncSelectCallback(int socket, System::IntPtr hWnd, unsigned int wMsg, NetSim::Core::AsyncSelectEvent events);
	};

}

#endif