/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;
using NetSim.Core;
using System.Diagnostics;

namespace NetSim.Library.Modules
{
	/// <summary>
	/// Handles ICMP Echo requests and sends Echo replies.
	/// </summary>
	public class IcmpModule : Module
	{
		public IcmpModule()
		{
		}

		public override void ReceivePacket(ref PacketInfo pInfo)
		{
			IcmpPacket icmp_p = pInfo.Packet as IcmpPacket;
			if (icmp_p == null) // handle only ICMP packets
				return;

			if (icmp_p.MsgType == IcmpPacket.MessageType.EchoRequest)
			{
				ProcessEchoRequest((IcmpEchoPacket)icmp_p, pInfo.RemoteAddress, pInfo.ThisMachine);
				pInfo = null;
			}
		}

		private void ProcessEchoRequest(IcmpEchoPacket echoPacket, NetworkAddress destination, NetworkAddress thisMachine)
		{
			Debug.Assert(echoPacket.MsgType == IcmpPacket.MessageType.EchoRequest);

			IcmpEchoPacket reply = IcmpEchoPacket.SwitchToEchoReply(echoPacket);
			PacketInfo pi = new PacketInfo(reply);
			pi.RemoteAddress = destination;
			pi.ThisMachine = thisMachine;
			Node.NewPacketToSend(this.ID, pi);
		}

		public override void SendPacket(ref PacketInfo pInfo)
		{
			base.SendPacket (ref pInfo);
		}

		public void UnreacheableDestination(IPv4Packet packet, IcmpUnreacheablePacket.Code code)
		{
			IcmpUnreacheablePacket icmp_p = IcmpUnreacheablePacket.CreatePacket(packet, code);
			PacketInfo pi = new PacketInfo(icmp_p);
			pi.RemoteAddress = packet.SourceIP;
			Node.NewPacketToSend(this.ID, pi);
		}

		public void TimeExceeded(IPv4Packet packet, IcmpTimeExceededPacket.Code code)
		{
			IcmpTimeExceededPacket icmp_p = IcmpTimeExceededPacket.CreatePacket(packet, code);
			PacketInfo pi = new PacketInfo(icmp_p);
			pi.RemoteAddress = packet.SourceIP;
			Node.NewPacketToSend(this.ID, pi);
		}
	}
}
