/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;
using NetSim.Core;

namespace NetSim.Library.Modules
{
	/// <summary>
	/// </summary>
	public class UdpModule : Module
	{
		/// <summary>
		/// Creates new <see cref="UdpModule"/>.
		/// </summary>
		public UdpModule()
		{
		}

		public override void ReceivePacket(ref PacketInfo pInfo)
		{
			UdpPacket udp_p = pInfo.Packet as UdpPacket;
			if (udp_p == null)
				return;

			if (udp_p.HigherLayerPacket == null)
			{
				pInfo = null; // no upper layer packet - nothing to deliver
				return;
			}

			SocketDataPacket sdp = udp_p.HigherLayerPacket as SocketDataPacket;
			if (sdp == null) // unable to parse the higher layer packet - but it still can be processed by some other module
			{
				return;
			}

			// set endpoints appropriately
			sdp.LocalPort = udp_p.DestinationPort;
			sdp.RemotePort = udp_p.SourcePort;

			pInfo.Packet = sdp;
		}

		public override void SendPacket(ref PacketInfo pInfo)
		{
			// if the packet is for us, wrap with UDP packet
			SocketDataPacket sdp = pInfo.Packet as SocketDataPacket;
			if (sdp == null || sdp.Protocol != System.Net.Sockets.ProtocolType.Udp)
				return;

			UdpPacket udp_p = new UdpPacket(sdp.LocalPort, sdp.RemotePort);
			udp_p.HigherLayerPacket = sdp;
			pInfo.Packet = udp_p;
		}

	}
}
