/*

 Copyright (c) 2006 Pavel Novak.

 The use and distribution terms for this software are contained in the file named License.txt, 
 which can be found in the root of the NetSim distribution. By using this software 
 in any fashion, you are agreeing to be bound by the terms of this license.
 
 You must not remove this notice from this software.

*/

using System;
using NetSim.Core;
using System.Xml;

namespace NetSim.Library
{
	#region IPSettings
	public class IPSettings : ProtocolSettings
	{
		private IPAddress ipAddress;
		private IPAddress mask;

		public override void Configure(XmlNode settingsNode)
		{
			XmlNode subnode = settingsNode.FirstChild;
			while (subnode != null)
			{
				if (subnode.Name == "IPAddress")
				{
					this.ipAddress = IPAddress.Parse(subnode.InnerText);
				}
				else if (subnode.Name == "NetworkMask")
				{
					this.mask = IPAddress.Parse(subnode.InnerText);
				}
				subnode = subnode.NextSibling;
			}
		}

		public override bool CanSendPacketTo(NetworkAddress address)
		{
			IPAddress ip_a = address as IPAddress;
			if (ip_a == null)
				return false;

			return IPAddress.IsInNetwork(ip_a, ipAddress, mask);
		}

		public override bool ShouldReceive(NetworkAddress address, out NetworkAddress localAddress)
		{
			localAddress = null;

			IPAddress ip_a = address as IPAddress;
			if (ip_a == null)
				return false;

			if (ip_a.Equals(ipAddress) || ip_a.Equals(IPv4Address.Broadcast))
			{
				localAddress = this.Address;
				return true;
			}

			return false;
		}

		public override NetworkAddress Address
		{
			get { return this.ipAddress; }
		}

	}
	#endregion
}
