using System;
using System.Collections;

namespace Tamir.IPLib
{
	/// <summary>
	///Represent an IP Address and a Subnet Mask.
	///This struct hold two parameters:
	///address -  string of the IP Address
	///mask - a string of the subnet mask
	/// </summary>
	public struct IPAddress
	{
		private string _address, _mask;
		internal IPAddress(string address, string mask)
		{
			this._address = address;
			this._mask = mask;
		}

		/// <summary>
		/// IP Address
		/// </summary>
		public string Address
		{
			get
			{
				return _address;
			}
		}

		/// <summary>
		/// Subnet Mask
		/// </summary>
		public string Mask
		{
			get
			{
				return _mask;
			}
		}

		public override string ToString()
		{
			return (Address+"/"+Util.Convert.MaskStringToBits(Mask));
		}

	}

	/// <summary>
	/// Summary description for IPAddressList.
	/// </summary>
	public class IPAddressList : CollectionBase
	{
		public IPAddress this[ int index ]  
		{
			get  
			{
				return( (IPAddress) List[index] );
			}
			set  
			{
				List[index] = value;
			}
		}

		public int Add( IPAddress value )  
		{
			return( List.Add( value ) );
		}

		public int IndexOf( IPAddress value )  
		{
			return( List.IndexOf( value ) );
		}

		public void Insert( int index, IPAddress value )  
		{
			List.Insert( index, value );
		}

		public void Remove( IPAddress value )  
		{
			List.Remove( value );
		}

		public bool Contains( IPAddress value )  
		{
			// If value is not of type IPAddress, this will return false.
			return( List.Contains( value ) );
		}

		public bool ContainsIp( string Ip )
		{
			foreach (IPAddress item in this)
			{
				if (item.Address == Ip)
					return true;
			}
			return false;
		}

		//		protected override void OnInsert( int index, Object value )  
		//		{
		//			// Insert additional code to be run only when inserting values.
		//		}
		//
		//		protected override void OnRemove( int index, Object value )  
		//		{
		//			// Insert additional code to be run only when removing values.
		//		}
		//
		//		protected override void OnSet( int index, Object oldValue, Object newValue )  
		//		{
		//			// Insert additional code to be run only when setting values.
		//		}

		protected override void OnValidate( Object value )  
		{
			if ( value.GetType() != Type.GetType("Tamir.IPLib.IPAddress") )
				throw new ArgumentException( "value must be of type IPAddress.", "value" );
		}
	}
}
