// $Id: ICMPMessage.java,v 1.4 2001/06/20 06:24:37 pcharles Exp $

/// <summary>************************************************************************
/// Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
/// Distributed under the Mozilla Public License                            *
/// http://www.mozilla.org/NPL/MPL-1.1.txt                                *
/// *************************************************************************
/// </summary>
namespace Tamir.IPLib.Packets
{
	using System;
	using HashMap = System.Collections.Hashtable;
	/// <summary> ICMP message utility class.
	/// *
	/// </summary>
	/// <author>  Patrick Charles and Jonas Lehmann
	/// </author>
	/// <version>  $Revision: 1.4 $
	/// @lastModifiedBy $Author: pcharles $
	/// @lastModifiedAt $Date: 2001/06/20 06:24:37 $
	/// 
	/// </version>
	public class ICMPMessage : ICMPMessages
	{
		/// <summary> Fetch an ICMP message.
		/// </summary>
		/// <param name="code">the code associated with the message.
		/// </param>
		/// <returns> a message describing the significance of the ICMP code.
		/// 
		/// </returns>
		public static System.String getDescription(int code)
		{
			System.Int32 c = code;
			if (messages.ContainsKey(c))
				return (System.String) messages[c];
			else
				return "unknown";
		}
		
		/// <summary> 'Human-readable' ICMP messages.
		/// </summary>
		//UPGRADE_NOTE: The initialization of  'messages' was moved to static method 'Tamir.IPLib.Packets.ICMPMessage'. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1005"'
		private static HashMap messages;
		
		
		private System.String _rcsid = "$Id: ICMPMessage.java,v 1.4 2001/06/20 06:24:37 pcharles Exp $";
		static ICMPMessage()
		{
			messages = new HashMap();
			{
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.ECHO_REPLY]= "echo reply";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.ECHO]= "echo request";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_NET]= "net unreachable";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_HOST]= "host unreachable";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_PROTOCOL]= "bad protocol";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_PORT]= "port unreachable";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_NEEDFRAG]= "ip_df drop";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_SRCFAIL] = "source route failed";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_NET_UNKNOWN] = "unknown network";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_HOST_UNKNOWN] = "unknown host";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_ISOLATED] = "source host isolated";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_NET_PROHIB] = "net access prohibited";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_HOST_PROHIB] = "host access prohibited";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_TOSNET] = "tos for net invalid";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.UNREACH_TOSHOST] = "tos for host invalid";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.SOURCE_QUENCH] = "packet lost";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.REDIRECT_NET] = "redirect to network";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.REDIRECT_HOST] = "redirect to host";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.REDIRECT_TOSNET] = "tos redirect to network";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.REDIRECT_TOSHOST] = "tos redirect to host";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.ROUTER_ADVERT] = "router advert";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.ROUTER_SOLICIT] = "router solicit";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.TIME_EXCEED_INTRANS] = "transit time exceeded";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.TIME_EXCEED_REASS] = "reass time exceeded";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.PARAM_PROB] = "bad ip header";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.TSTAMP] = "timestamp request";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.TSTAMP_REPLY] = "timestamp reply";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.IREQ] = "information request";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.IREQ_REPLY] = "information reply";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.MASK_REQ] = "address mask request";
				messages[Tamir.IPLib.Packets.ICMPMessages_Fields.MASK_REPLY] = "address mask reply";
			}
		}
	}
}