// $Id: LinkLayer.java] =v 1.5 2001/07/02 04:02:57 pcharles Exp $

/// <summary>************************************************************************
/// Copyright (C) 2001] = Patrick Charles and Jonas Lehmann                   *
/// Distributed under the Mozilla Public License                            *
/// http://www.mozilla.org/NPL/MPL-1.1.txt                                *
/// *************************************************************************
/// </summary>
namespace Tamir.IPLib.Packets
{
	using System;
	using HashMap =  System.Collections.Hashtable;
	/// <summary> Information about network link layers.
	/// *
	/// </summary>
	/// <author>  Patrick Charles and Jonas Lehmann
	/// </author>
	/// <version>  $Revision: 1.5 $
	/// @lastModifiedBy $Author: pcharles $
	/// @lastModifiedAt $Date: 2001/07/02 04:02:57 $
	/// 
	/// </version>
	public class LinkLayer
	{
		/// <summary> Fetch the header length associated with various link-layer types.
		/// </summary>
		/// <param name="layerType">the link-layer code
		/// </param>
		/// <returns> the length of the header for the specified link-layer
		/// 
		/// </returns>
		public static int getLinkLayerLength(int layerType)
		{
			switch (layerType)
			{
				
				case Tamir.IPLib.Packets.LinkLayers.ARCNET: 
					return 6;
				
				case Tamir.IPLib.Packets.LinkLayers.SLIP: 
					return 16;
				
				case Tamir.IPLib.Packets.LinkLayers.SLIP_BSDOS: 
					return 24;
				
				case Tamir.IPLib.Packets.LinkLayers.NULL: 
				case Tamir.IPLib.Packets.LinkLayers.LOOP: 
					return 4;
				
				case Tamir.IPLib.Packets.LinkLayers.PPP: 
				case Tamir.IPLib.Packets.LinkLayers.CHDLC: 
				case Tamir.IPLib.Packets.LinkLayers.PPP_SERIAL: 
					return 4;
				
				case Tamir.IPLib.Packets.LinkLayers.PPP_BSDOS: 
					return 24;
				
				case Tamir.IPLib.Packets.LinkLayers.FDDI: 
					return 21;
				
				case Tamir.IPLib.Packets.LinkLayers.IEEE802_11: 
					return 22;
				
				case Tamir.IPLib.Packets.LinkLayers.ATM_RFC1483: 
					return 8;
				
				case Tamir.IPLib.Packets.LinkLayers.RAW: 
					return 0;
				
				case Tamir.IPLib.Packets.LinkLayers.ATM_CLIP: 
					return 8;
				
				case Tamir.IPLib.Packets.LinkLayers.LINUX_SLL: 
					return 16;
				
				case Tamir.IPLib.Packets.LinkLayers.EN10MB: 
				default: 
					return 14;
				}
		}
		
		/// <summary> Fetch the offset into the link-layer header where the protocol code
		/// can be found. Returns -1 if there is no embedded protocol code.
		/// </summary>
		/// <param name="layerType">the link-layer code
		/// </param>
		/// <returns> the offset in bytes
		/// 
		/// </returns>
		public static int getProtoOffset(int layerType)
		{
			switch (layerType)
			{
				
				case Tamir.IPLib.Packets.LinkLayers.ARCNET: 
					return 2;
				
				case Tamir.IPLib.Packets.LinkLayers.SLIP: 
					return - 1;
				
				case Tamir.IPLib.Packets.LinkLayers.SLIP_BSDOS: 
					return - 1;
				
				case Tamir.IPLib.Packets.LinkLayers.NULL: 
				case Tamir.IPLib.Packets.LinkLayers.LOOP: 
					return 0;
				
				case Tamir.IPLib.Packets.LinkLayers.PPP: 
				case Tamir.IPLib.Packets.LinkLayers.CHDLC: 
				case Tamir.IPLib.Packets.LinkLayers.PPP_SERIAL: 
					return 2;
				
				case Tamir.IPLib.Packets.LinkLayers.PPP_BSDOS: 
					return 5;
				
				case Tamir.IPLib.Packets.LinkLayers.FDDI: 
					return 13;
				
				case Tamir.IPLib.Packets.LinkLayers.IEEE802_11: 
					return 14;
				
				case Tamir.IPLib.Packets.LinkLayers.ATM_RFC1483: 
					return 6;
				
				case Tamir.IPLib.Packets.LinkLayers.RAW: 
					return - 1;
				
				case Tamir.IPLib.Packets.LinkLayers.ATM_CLIP: 
					return 6;
				
				case Tamir.IPLib.Packets.LinkLayers.LINUX_SLL: 
					return 14;
				
				case Tamir.IPLib.Packets.LinkLayers.EN10MB: 
				default: 
					return 12;
				}
		}
		
		/// <summary> Fetch a link-layer type description.
		/// </summary>
		/// <param name="code">the code associated with the description.
		/// </param>
		/// <returns> a description of the link-layer type.
		/// 
		/// </returns>
		public static System.String getDescription(int code)
		{
			System.Int32 c = code;
			if (descriptions.ContainsKey(c))
				return (System.String) descriptions[c];
			else
				return "unknown";
		}
		
		/// <summary> 'Human-readable' link-layer type descriptions.
		/// </summary>
		//UPGRADE_NOTE: The initialization of  'descriptions' was moved to static method 'Tamir.IPLib.Packets.LinkLayer'. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1005"'
		private static HashMap descriptions;
		
		
		private System.String _rcsid = "$Id: LinkLayer.java] =v 1.5 2001/07/02 04:02:57 pcharles Exp $";
		static LinkLayer()
		{
			descriptions = new HashMap();
			{
				descriptions[Tamir.IPLib.Packets.LinkLayers.NULL] = "no link-layer encapsulation";
				descriptions[Tamir.IPLib.Packets.LinkLayers.EN10MB] = "10/100Mb ethernet";
				descriptions[Tamir.IPLib.Packets.LinkLayers.EN3MB] = "3Mb experimental ethernet";
				descriptions[Tamir.IPLib.Packets.LinkLayers.AX25] = "AX.25 amateur radio";
				descriptions[Tamir.IPLib.Packets.LinkLayers.PRONET] = "proteon pronet token ring";
				descriptions[Tamir.IPLib.Packets.LinkLayers.CHAOS] = "chaos";
				descriptions[Tamir.IPLib.Packets.LinkLayers.IEEE802] = "IEEE802 network";
				descriptions[Tamir.IPLib.Packets.LinkLayers.ARCNET] = "ARCNET";
				descriptions[Tamir.IPLib.Packets.LinkLayers.SLIP] = "serial line IP";
				descriptions[Tamir.IPLib.Packets.LinkLayers.PPP] = "point-to-point protocol";
				descriptions[Tamir.IPLib.Packets.LinkLayers.FDDI] = "FDDI";
				descriptions[Tamir.IPLib.Packets.LinkLayers.ATM_RFC1483] = "LLC/SNAP encapsulated ATM";
				descriptions[Tamir.IPLib.Packets.LinkLayers.RAW] = "raw IP";
				descriptions[Tamir.IPLib.Packets.LinkLayers.SLIP_BSDOS] = "BSD SLIP";
				descriptions[Tamir.IPLib.Packets.LinkLayers.PPP_BSDOS] = "BSD PPP";
				descriptions[Tamir.IPLib.Packets.LinkLayers.ATM_CLIP] = "IP over ATM";
				descriptions[Tamir.IPLib.Packets.LinkLayers.PPP_SERIAL] = "PPP over HDLC";
				descriptions[Tamir.IPLib.Packets.LinkLayers.CHDLC] = "Cisco HDLC";
				descriptions[Tamir.IPLib.Packets.LinkLayers.IEEE802_11] = "802.11 wireless";
				descriptions[Tamir.IPLib.Packets.LinkLayers.LOOP] = "OpenBSD loopback";
				descriptions[Tamir.IPLib.Packets.LinkLayers.LINUX_SLL] = "Linux cooked sockets";
				descriptions[Tamir.IPLib.Packets.LinkLayers.UNKNOWN] = "unknown link-layer type";
			}
		}
	}
}